/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package musicbox.backend;

import java.text.ParseException;
import musicbox.backend.filefilters.ImageFilter;
import musicbox.backend.comparators.AlbumComparator;
import musicbox.backend.comparators.ArtistComparator;
import musicbox.database.Database;
import java.awt.Image;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.ResourceBundle.Control;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.swing.ImageIcon;
import javazoom.jl.decoder.Bitstream;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.AudioHeader;
import org.jaudiotagger.audio.mp3.MP3AudioHeader;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.datatype.Artwork;
import javax.sound.sampled.AudioPermission;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.SwingUtilities;
import javax.swing.text.html.HTMLDocument;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;
import musicbox.backend.comparators.IDComparator;
import musicbox.gui.CustomTableModel;
import musicbox.gui.MiniPlayer;
import musicbox.gui.MusicBoxView;
import noTalent.MusicOutputDesign;
import noTalent.SocketObject;
import org.kc7bfi.jflac.sound.spi.Flac2PcmAudioInputStream;
import org.kc7bfi.jflac.sound.spi.FlacAudioFileReader;

/**
 * The application's backbone.
 *
 * @author Isaac Hammon
 */
public class Backend extends Thread 
{

    /**
     *
     */
    public String fFilename = null;
    /**
     *
     */
    public MP3AudioHeader audioHeader;
    /**
     *
     */
    public AudioHeader aH;
    /**
     *
     */
    public org.kc7bfi.jflac.apps.Player flacPlayer;
    /**
     *
     */
    public String filePath;
    /**
     *
     */
    public Vector<Integer> playList;
    /**
     *
     */
    public int currentLocation = 0;
    private String artist;
    private String previousArtist;
    /**
     *
     */
    public ScrollText sT;
    /**
     *
     */
    public SourceDataLine line;
    /**
     *
     */
    public Control c;
    /**
     *
     */
    public AudioInputStream din;
    /**
     *
     */
    public int nBytesRead;
    /**
     *
     */
    public long doSeek = -1;
    /**
     *
     */
    public long toSkip = 0;
    /**
     *
     */
    public AudioFormat decodedFormat;
    /**
     *
     */
    public AudioInputStream in;
    /**
     *
     */
    public AudioFileFormat aFF;
    /**
     *
     */
    public long fileLength;
    /**
     *
     */
    public long numFrames;
    /**
     *
     */
    public long currentFrame;
    /**
     *
     */
    public AudioPermission aP = new AudioPermission("play");
    /**
     *
     */
    public Database db;
    /**
     *
     */
    public MusicOutputDesign currentTrack;
    /**
     *
     */
    public Vector<String> googleImages = new Vector<String>();
    public boolean cont = true;
    public MusicBoxView mbv;

    /**
     *Shuffles the current playlist when called.
     */
    public void shuffle() 
    {

        //Collections.shuffle(playList);
        Collections.shuffle(mbv.currentlyPlaying2);
        currentLocation = 0;
        new Thread()
        {
            public void run()
            {
                db.updatePlayOrder(mbv.currentlyPlaying2);
            }
        }.start();
        //TODO
        //mbv.updateTable(playList, mbv.currentlyPlayingTable, "flag");
        CustomTableModel ctm = (CustomTableModel) mbv.currentlyPlayingTable.getModel();
        ctm.removeAllRows();
        //mbv.populateTableModel(ctm, playList);
        mbv.populateTableModel2(ctm, mbv.currentlyPlaying2);
    }

    /**
     *Returns the name of the current file being played.
     *
     * @return fFilename
     */
    public String getFileName() 
    {
        return fFilename;
    }

    /**
     * Returns the file path of the current file being played.
     *
     * @return filePath
     */
    public String getFilePath() 
    {
        return filePath;
    }

    public void setImagePath(String imagePath, String value) 
    {
        int loc = 0;
        //System.out.println("Value: " + value);
        if (mbv.jTabbedPane2.getSelectedIndex() == 0) 
        {
            Collections.sort(mbv.allMusic, new ArtistComparator());
            loc = binarySearchArtist(mbv.allMusic, value, 0, mbv.allMusic.size());
        } 
        else 
        {
            //System.out.println("Album Search");
            Collections.sort(mbv.allMusic, new AlbumComparator());
            loc = binarySearchAlbum(mbv.allMusic, value, 0, mbv.allMusic.size());
        }
        if (loc != -1) 
        {
            mbv.allMusic.elementAt(loc).setImagePath(imagePath);
            try 
            {
                java.sql.Statement stmt = db.getConnection().createStatement();
                mbv.db.updateImagePath(stmt, mbv.allMusic.elementAt(loc));
                stmt.close();
            } 
            catch (SQLException ex) 
            {
                //ex.printStackTrace();
                mbv.showErrorDialog(ex);
            }
        }
        else
        {
            //System.out.println("Could not find " + value);
        }
    }

    private int binarySearchArtist(Vector<MusicOutputDesign> v, String value, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        if (mid > v.size())
        {
            return -1;
        }
        String s = v.elementAt(mid).getArtist().replace("\\", "/");
        if (s.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchArtist(v, value, low, mid - 1);
        } 
        else if (s.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchArtist(v, value, mid + 1, high);
        }
        else 
        {
            return mid; // found
        }
    }

    private int binarySearchAlbum(Vector<MusicOutputDesign> v, String value, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        String s = v.elementAt(mid).getAlbum().replace("\\", "/");
        if (s.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchAlbum(v, value, low, mid - 1);
        } 
        else if (s.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchAlbum(v, value, mid + 1, high);
        } 
        else 
        {
            return mid; // found
        }
    }
    
    private int binarySearchID(Vector<MusicOutputDesign> v, String value, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        if (mid > v.size())
        {
            return -1;
        }
        String s = v.elementAt(mid).getId().replace("\\", "/");
        if (s.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchID(v, value, low, mid - 1);
        } 
        else if (s.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchID(v, value, mid + 1, high);
        }
        else 
        {
            return mid; // found
        }
    }

    /**
     *
     * @param m
     * @param r
     */
    @SuppressWarnings("static-access")
    public void updateRating(MusicOutputDesign m, int r) 
    {
        try 
        {
            m.setRating("" + r);
            java.sql.Statement stmt = db.getConnection().createStatement();
            db.updateRating(stmt, m);
            stmt.close();
            /*if (playList != null) {
            if (!playList.isEmpty()) {
            mbv.updateVector(mbv.allMusic, m, mbv.currentlyPlayingTable);
            }
            }*/
            if (!mbv.playList.isEmpty()) 
            {
                mbv.updateVector(mbv.allMusic, m, mbv.playlistTable);
            }
            mbv.updateVector(mbv.allMusic, m, mbv.currentlyShowingTable);
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    public void updateRating(MusicOutputDesign m, int r, String flag) 
    {
        m.setRating("" + r);

        try 
        {
            java.sql.Statement stmt = db.getConnection().createStatement();
            db.updateRating(stmt, m);
            stmt.close();
        } 
        catch (Exception e) 
        {
            //e.printStackTrace();
            mbv.showErrorDialog(e);
        }
        if (!mbv.playList.isEmpty()) 
        {
            mbv.updateVector(mbv.allMusic, m, mbv.playlistTable);
        }
        mbv.updateVector(mbv.allMusic, m, mbv.currentlyShowingTable);
    }

    /**
     * Gets the current play list selected by the user.
     * @return currentlyPlaying
     */
    /*public Vector<Integer> getPlaylist() 
    {
        return mbv.currentlyPlaying;
    }*/
    
    public Vector<String> getPlayList()
    {
        return mbv.currentlyPlaying2;
    }

    /**
     * TODO:
     * @param filePath
     * @param file
     * @param playCount
     */
    @SuppressWarnings("static-access")
    private void updatePlayCount(String filePath, String file, String playCount) 
    {
        try 
        {
            int count = Integer.parseInt(playCount);
            count += 1;
            String c1 = "" + count;
            currentTrack.setPlayCount(c1);
            //playList.elementAt(currentLocation).playCount = c1;
            java.sql.Statement stmt = db.getConnection().createStatement();
            db.updatePlayCount(stmt, currentTrack);
            stmt.execute("COMMIT");
            stmt.close();
        } 
        catch (Exception e) 
        {
            mbv.showErrorDialog(e);
            //e.printStackTrace();
        }
        //mbv.updateTable(playList, mbv.currentlyPlayingTable, "flag");
        //mbv.updateVectorAtLocation(playList, currentLocation, currentTrack, mbv.currentlyPlayingTable);
        mbv.updateVectorAtLocation(mbv.currentlyPlaying2, currentLocation, currentTrack.getId(), currentTrack, mbv.currentlyPlayingTable);
    }

    /**
     * Gets the next item in the play list.
     * @return MusicPutputDesign Returns the next item.
     */
    public int getNext() 
    {
        return playList.elementAt(currentLocation);
    }
    
    private String getNextID()
    {
        return getPlayList().elementAt(currentLocation);
    }

    /**
     * 
     * @param txt
     * @param v
     * @return
     */
    public Vector<MusicOutputDesign> search(String txt, Vector<MusicOutputDesign> v) 
    {
        Vector<MusicOutputDesign> tmp = new Vector<MusicOutputDesign>();
        for (int i = 0; i < v.size(); i++) 
        {
            if (v.elementAt(i).getAlbum().toLowerCase().contains(txt.toLowerCase())) 
            {
                tmp.add(v.elementAt(i));
            } 
            else if (v.elementAt(i).getArtist().toLowerCase().contains(txt.toLowerCase())) 
            {
                tmp.add(v.elementAt(i));
            } 
            else if (v.elementAt(i).getTrackTitle().toLowerCase().contains(txt.toLowerCase())) 
            {
                tmp.add(v.elementAt(i));
            }
        }
        return tmp;
    }

    @Override
    /**
     * Main run method.
     */
    @SuppressWarnings("static-access")
    public void run() 
    {
        try 
        {
            setName("Backend");
            //playList = getPlaylist();
            SocketObject objectToPlay = null;
            while (currentLocation < mbv.currentlyPlaying2.size()) 
            {
                if (currentLocation == 0) 
                {
                    mbv.previousTrackButton.setEnabled(false);
                    if (mbv.mini != null && mbv.mini.isVisible()) 
                    {
                        mbv.mini.previousButton.setEnabled(false);
                    }
                } 
                else 
                {
                    mbv.previousTrackButton.setEnabled(true);
                    if (mbv.mini != null && mbv.mini.isVisible()) 
                    {
                        mbv.mini.previousButton.setEnabled(true);
                    }
                }
                googleImages.clear();
                if (!mbv.currentlyPlaying2.isEmpty()) 
                {
                    //int next = getNext();
                    String nextID = getNextID();
                    if (/*next >= 0 && */!nextID.startsWith("sharedLibraryTable")) 
                    {
                        //currentTrack = mbv.allMusic.elementAt(next);
                        currentTrack = findTrack(nextID);
                        db.updateCurrentTrack(this.currentTrack.getId());
                    } 
                    else 
                    {
                        //currentTrack = mbv.sharedLibrary.elementAt((next + 1) * -1);
                        Vector<MusicOutputDesign> tmp = (Vector<MusicOutputDesign>)mbv.sharedLibrary.clone();
                        Collections.sort(tmp, new IDComparator());
                        currentTrack = findTrack(tmp,nextID.replace("sharedLibraryTable-", ""));
                        try 
                        {
                            mbv.getSocketListener().getAudioStream(((String) mbv.addressList.getModel().getElementAt(mbv.getJList4PreviousSelection())), currentTrack.getFilePath() + "/" + currentTrack.getFile());
                            for (int wait = 0; wait < 120; wait++) 
                            {
                                objectToPlay = mbv.getSocketListener().getObjectToPlay();
                                if (objectToPlay != null) 
                                {
                                    break;
                                }
                                else 
                                {
                                    Thread.sleep(500);
                                }
                            }
                            if (objectToPlay == null) 
                            {
                                System.out.println("ObjectToPlay is null");
                                continue;
                            }
                            else if(objectToPlay.getMessageType().equals("FileNotFound"))
                            {
                                System.out.println("FileNotFoundExpection thrown");
                                objectToPlay = null;
                                continue;
                            }
                        } 
                        catch (Exception e) 
                        {
                            mbv.showErrorDialog(e);
                            //e.printStackTrace();
                        }
                    }
                    mbv.showRating();
                    mbv.showSongSplash(currentTrack);
                    java.awt.EventQueue.invokeLater(new Runnable() 
                    {
                        @Override
                        public void run() 
                        {
                            mbv.currentlyPlayingTable.changeSelection(currentLocation, 0, false, false);
                        }
                    });

                    fFilename = currentTrack.getFilePath() + "/" + currentTrack.getFile();
                    filePath = currentTrack.getFilePath();
                    String trackTitle = currentTrack.getTrackTitle();
                    if (artist == null) 
                    {
                        previousArtist = "";
                    } 
                    else 
                    {
                        previousArtist = artist;
                    }
                    artist = currentTrack.getArtist();
                    String album = currentTrack.getAlbum();
                    String show1 = trackTitle;
                    String show2 = "By: " + artist;
                    String show3 = "Album: " + album;
                    if(sT == null)
                    {
                        sT = new ScrollText(show1, show2, show3, mbv);
                        sT.start();
                    }
                    else
                    {
                        sT.setLines(show1, show2, show3);
                    }
                    if (objectToPlay == null) 
                    {
                        play();
                    } 
                    else 
                    {
                        play(objectToPlay);
                    }
                    //sT.stop();
                    if (objectToPlay == null && currentTrack != null) 
                    {
                        updatePlayCount(currentTrack.getFilePath(), currentTrack.getFile(), currentTrack.getPlayCount());
                    } 
                    else if(objectToPlay != null) 
                    {
                        mbv.getSocketListener().setObjectToPlay(null);
                        objectToPlay = null;
                    }
                    currentLocation += 1;
                    mbv.clicked = false;
                    java.awt.EventQueue.invokeLater(new Runnable() 
                    {
                        @Override
                        public void run() 
                        {
                            mbv.jProgressBar1.setValue(0);
                        }
                    });

                }
            }
            mbv.playButton.setEnabled(true);
            mbv.pauseButton.setEnabled(false);
            mbv.stopButton.setEnabled(false);
            mbv.playing = false;
            currentLocation = 0;
            mbv.currentlyPlaying2.removeAllElements();
            //TODO populate table
            //mbv.updateTable(playList, mbv.currentlyPlayingTable, "flag");
            mbv.fin();
        } 
        catch (Exception ex) 
        {
            System.err.println(ex);
            //ex.printStackTrace(System.err);
            mbv.showErrorDialog(ex);
        }
    }

    /**
     * Empty constructor.
     */
    public Backend(int currentLocation) 
    {
        this.currentLocation = currentLocation;
        currentTrack = new MusicOutputDesign();
    }
    
    public MusicOutputDesign findTrack(String findID)
    {
        MusicOutputDesign mod = new MusicOutputDesign();
        mod = db.selectTrack(findID);
        return mod;
    }
    
    public MusicOutputDesign findTrack(Vector<MusicOutputDesign> v, String findID)
    {
        return v.elementAt(binarySearchID(v,findID,0,v.size()));
    }

    class GetArt extends TimerTask
    {
        @Override
        public void run() 
        {
            showAlbumArt();
        }
    }

    /**
     * Gets and displays album art of the file currently playing.
     */
    public void showAlbumArt() 
    {
        mbv.albumArtLabel.setVisible(true);
        mbv.jButton7.setEnabled(false);
        mbv.jButton8.setEnabled(false);
        boolean foundArt = false;
        if (currentTrack.getImagePath().compareTo("NoImage") != 0 && currentTrack.getImagePath().compareTo("embedded") != 0) 
        {
            ImageIcon iI = new ImageIcon();
            if (currentTrack.getImagePath().contains("http://")) 
            {
                try 
                {
                    if (mbv.internetConnection) 
                    {
                        iI = new ImageIcon(new URL(currentTrack.getImagePath()));
                    }
                } 
                catch (MalformedURLException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            } 
            else 
            {
                iI = new ImageIcon(currentTrack.getImagePath());
            }
            Image k = iI.getImage();
            Image l = k.getScaledInstance(100, 100, Image.SCALE_SMOOTH);
            ImageIcon iL = new ImageIcon(l);
            mbv.albumArtLabel.setSize(100, 100);
            mbv.albumArtLabel.setIcon(iL);
            foundArt = true;
        } 
        else 
        {
            try 
            {
                AudioFile file = AudioFileIO.read(new File(fFilename));
                Artwork albumArt = file.getTag().getFirstArtwork();
                byte[] b1 = albumArt.getBinaryData();
                ImageIcon iI = new ImageIcon(b1);
                int h = iI.getIconHeight();
                int w = iI.getIconWidth();
                Image i = iI.getImage();
                Image j = i.getScaledInstance(100, 100, Image.SCALE_SMOOTH);
                ImageIcon iJ = new ImageIcon(j);

                mbv.albumArtLabel.setSize(100, 100);
                mbv.albumArtLabel.setIcon(iJ);
                foundArt = true;
                if (h < 0 && w < 0) 
                {
                    foundArt = false;
                }
            } 
            catch (Exception ex) 
            {
                if (filePath != null) 
                {
                    File myDir = new File(filePath);
                    String[] dirNames = myDir.list(new ImageFilter());
                    if (dirNames != null && dirNames.length > 0) 
                    {
                        ImageIcon max = new ImageIcon(filePath + "/" + dirNames[0]);
                        for (int p1 = 0; p1 < dirNames.length; p1++) 
                        {
                            ImageIcon iK = new ImageIcon(filePath + "/" + dirNames[p1]);
                            if (iK.getIconHeight() + iK.getIconWidth() > max.getIconHeight() + iK.getIconWidth()) 
                            {
                                max = iK;
                            }

                        }
                        Image k = max.getImage();

                        Image l = k.getScaledInstance(100, 100, Image.SCALE_SMOOTH);
                        ImageIcon iL = new ImageIcon(l);
                        mbv.albumArtLabel.setSize(100, 100);
                        mbv.albumArtLabel.setIcon(iL);
                        foundArt = true;
                    }
                }
            }
            if (foundArt == false)
            {
                if (mbv.internetConnection) 
                {
                    mbv.jButton7.setEnabled(true);
                    googleImageSearch();
                    ImageIcon icon;
                    try 
                    {
                        if (!googleImages.isEmpty()) 
                        {
                            icon = new ImageIcon(new URL(googleImages.elementAt(mbv.googleImageLocation)));
                            ImageIcon ico = new ImageIcon(icon.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH));
                            mbv.albumArtLabel.setIcon(ico);
                        }
                    } 
                    catch (MalformedURLException ex1) 
                    {
                        //ex1.printStackTrace();
                        mbv.showErrorDialog(ex1);
                    }
                }
            }
        }
    }

    /**
     *
     */
    public void googleImageSearch() 
    {
        if (artist.compareToIgnoreCase(previousArtist) != 0) 
        {
            mbv.googleImageLocation = 0;
            try 
            {
                String u = "http://images.google.com/images?q=" + currentTrack.getArtist() + " - " + currentTrack.getAlbum() + "&sa=N&start=0&ndsp=21";
                if (u.contains(" ")) 
                {
                    u = u.replace(" ", "+");
                }
                URL url = new URL(u);

                HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
                httpcon.addRequestProperty("User-Agent", "Mozilla/4.76");
                BufferedReader readIn = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));
                
                String text = "";
                String lin = "";
                while((lin = readIn.readLine()) != null)
                {
                    text += lin;
                }
                readIn.close();
                if(text.contains("\n"))
                {
                    text = text.replace("\n", "");
                }
                String[] array = text.split("\\Qhref=\"/imgres?imgurl=\\E");
                for(String s: array)
                {
                    if(s.startsWith("http://") || s.startsWith("https://") && s.contains("&amp;"))
                    {
                        String s1 = s.substring(0, s.indexOf("&amp;"));
                        googleImages.add(s1);
                    }
                }
            } 
            catch (Exception ex4) 
            {
                mbv.showErrorDialog(ex4);
                //ex4.printStackTrace();
            }
        }
    }

    /**
     *
     * @param search
     * @param start
     */
    public void googleImageSearch(String search, String start) 
    {
        try 
        {
            String u = "http://images.google.com/images?q=" + search + start;
            if (u.contains(" ")) 
            {
                u = u.replace(" ", "+");
            }
            URL url = new URL(u);

            HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
            httpcon.addRequestProperty("User-Agent", "Mozilla/4.76");
            BufferedReader readIn = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));
            googleImages.clear();
            
            String text = "";
            String lin = "";
            while((lin = readIn.readLine()) != null)
            {
                text += lin;
            }
            readIn.close();
            if(text.contains("\n"))
            {
                text = text.replace("\n", "");
            }
            String[] array = text.split("\\Qhref=\"/imgres?imgurl=\\E");
            for(String s: array)
            {
                if(s.startsWith("http://") || s.startsWith("https://") && s.contains("&amp;"))
                {
                    String s1 = s.substring(0, s.indexOf("&amp;"));
                    googleImages.add(s1);
                }
            }
        } 
        catch (Exception ex4) 
        {
            mbv.showErrorDialog(ex4);
            //ex4.printStackTrace();
        }
        mbv.jButton7.setEnabled(true);
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(googleImages.elementAt(mbv.googleImageLocation)));
            ImageIcon ico = new ImageIcon(icon.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH));
            mbv.albumArtLabel.setIcon(ico);
        } 
        catch (MalformedURLException ex1) 
        {
            //ex1.printStackTrace();
            mbv.showErrorDialog(ex1);
        }

    }

    class GetWiki extends TimerTask 
    {
        @Override
        public void run() 
        {
            if (mbv.internetConnection) 
            {
                getBandWikiPage();
            }
        }
    }

    /**
     * Gets a bands wikipedia page and displays it.
     */
    public void getBandWikiPage() 
    {
        if (artist.compareToIgnoreCase(previousArtist) != 0) 
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                @Override
                public void run() 
                {
                    try 
                    {
                        String s = "http://en.wikipedia.org/wiki/" + replaceSpace(artist);
                        URL url = new URL(s);
                        mbv.jEditorPane1.setPage(url);
                        /*((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().importStyleSheet(new URL("http://bits.wikimedia.org/w/extensions/UsabilityInitiative/css/combined.min.css"));
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().importStyleSheet(new URL("http://bits.wikimedia.org/w/extensions/UsabilityInitiative/css/vector/jquery-ui-1.7.2.css"));
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().importStyleSheet(new URL("http://bits.wikimedia.org/skins-1.5/vector/main-ltr.css"));
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().importStyleSheet(new URL("http://bits.wikimedia.org/skins-1.5/common/shared.css"));
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().importStyleSheet(new URL("http://bits.wikimedia.org/skins-1.5/common/commonPrint.css"));*/

                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("BODY { font-size: 12pt; text-align: left; width:50px;}");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("P { font-size: 12pt; }");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("A { font-size: 12pt; text-decoration:underline;  color: blue;}");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("LI { font-size: 12pt; }");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("DIV { font-size: 12pt; }");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("SPAN { font-size: 12pt; }");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("TD { font-size: 12pt; }");
                        ((HTMLDocument) mbv.jEditorPane1.getDocument()).getStyleSheet().addRule("TH { font-size: 12pt; }");
                    } 
                    catch (Exception ex) 
                    {
                        mbv.showErrorDialog(ex);
                        //ex.printStackTrace();
                    }
                }
            });

        }
    }

    /**
     * Plays audio file.
     */
    public void play() 
    {
        try 
        {
            new Timer().schedule(new GetArt(), 100);
            new Timer().schedule(new GetWiki(), 200);
            mbv.playing = true;
            mbv.timeSkipped = 0;
            mbv.jLabel2.setText(currentTrack.getTrackLength());
            if (fFilename.toLowerCase().endsWith(".mp3")) 
            {
                MP3File f = (MP3File) AudioFileIO.read(new File(fFilename));
                audioHeader = (MP3AudioHeader) f.getAudioHeader();
                numFrames = audioHeader.getNumberOfFrames();
                mbv.trackLength = audioHeader.getPreciseTrackLength();
            } 
            else 
            {
                if (!fFilename.toLowerCase().endsWith(".ape") && !fFilename.toLowerCase().endsWith(".au")
                        && !fFilename.toLowerCase().endsWith(".aif")) 
                {
                    AudioFile aF = AudioFileIO.read(new File(fFilename));
                    aH = aF.getAudioHeader();
                    mbv.trackLength = aH.getTrackLength();
                } 
                else 
                {
                    AudioFileFormat aff = AudioSystem.getAudioFileFormat(new File(fFilename));

                    Map map = aff.properties();
                    if (map.containsKey("duration")) 
                    {
                        mbv.trackLength = ((Long) map.get("duration")).doubleValue();
                    }
                }
            }
            if (fFilename.toLowerCase().endsWith(".flac")) 
            {
                FlacAudioFileReader fafr = new FlacAudioFileReader();
                in = fafr.getAudioInputStream(new File(fFilename));
                AudioFormat format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100.0F, 16, 2, 4, 44100.0F, false);
                Flac2PcmAudioInputStream f2pcmStream = new Flac2PcmAudioInputStream(in, format, 100);
                if(in.markSupported())
                {
                    in.mark(in.available());
                }
                din = AudioSystem.getAudioInputStream(format, f2pcmStream);
                rawplay(format, din);
            } 
            else if (fFilename.toLowerCase().endsWith(".ogg") || fFilename.toLowerCase().endsWith(".mp3")
                    || fFilename.toLowerCase().endsWith(".wav") || fFilename.toLowerCase().endsWith(".aifc")
                    || fFilename.toLowerCase().endsWith(".aiff") || fFilename.toLowerCase().endsWith(".aif")
                    || fFilename.toLowerCase().endsWith(".au") || fFilename.toLowerCase().endsWith(".ape")) 
            {
                try 
                {
                    File file1 = new File(fFilename);
                    // Get AudioInputStream from given file.
                    fileLength = file1.length();
                    in = null;
                    try 
                    {
                        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file1));
                        //System.out.println("File size: " + file1.length());
                        /*while (bis.available() < file1.length())
                            System.out.print("");*/
                        //System.out.println("Available: " + bis.available());
                        /*byte[] buf = new byte[bis.available()];
                        bis.read(buf, 0, buf.length);
                        bis.close();
                        ByteArrayInputStream bais = new ByteArrayInputStream(buf);*/
                        //bais.read(buffer);
                        //System.out.println("bais available: " + bais.available());
                        //bais.read(buf, 0, buf.length);
                        in = AudioSystem.getAudioInputStream(bis/*file1*/);
                    }
                    catch(UnsupportedAudioFileException ex)
                    {
                        mbv.showErrorDialog(ex);
                    }
                    catch (IOException ex1) 
                    {
                        mbv.showErrorDialog(ex1);
                        //if (!fFilename.toLowerCase().endsWith(".wav") && !fFilename.toLowerCase().endsWith(".au") && !fFilename.toLowerCase().endsWith(".aif")) {
                        FileInputStream f_in = new FileInputStream(fFilename);
                        Bitstream m = new Bitstream(f_in);
                        long start = m.header_pos();    // need to open the stream again

                        fileLength = fileLength - start;
                        try 
                        {
                            m.close();
                        }
                        catch (Exception ex) 
                        {
                            mbv.showErrorDialog(ex);
                            //ex.printStackTrace();
                        }
                        f_in = new FileInputStream(fFilename);  // skip the header
                        long actual = f_in.skip(start);
                        in = AudioSystem.getAudioInputStream(f_in);
                        /*try
                        {
                            f_in.close();
                        }
                        catch(IOException ex)
                        {
                            mbv.showErrorDialog(ex);
                        }*/
                    } //else {
                    //     in = AudioSystem.getAudioInputStream(file1);
                    //}

                    din = null;
                    if (in != null) 
                    {
                        if(in.markSupported())
                        {
                            in.mark((int)fileLength);
                        }
                        AudioFormat baseFormat = in.getFormat();

                        decodedFormat = new AudioFormat(
                                AudioFormat.Encoding.PCM_SIGNED,
                                baseFormat.getSampleRate(),
                                16,
                                baseFormat.getChannels(),
                                baseFormat.getChannels() * 2,
                                baseFormat.getSampleRate(),
                                false);
                        // Get AudioInputStream that will be decoded by underlying VorbisSPI
                        din = AudioSystem.getAudioInputStream(decodedFormat, in);
                        //aFF = AudioSystem.getAudioFileFormat(file1);

                        // Play now !
                        rawplay(decodedFormat, din);

                        in.close();
                    }
                } 
                catch (Exception ex1) 
                {
                    mbv.showErrorDialog(ex1);
                    //ex1.printStackTrace();
                    System.err.println("Cannot play file: " + fFilename);
                }
            }
            mbv.albumArtLabel.setText("Nothing is playing");

        } 
        catch (Exception ex) 
        {
            mbv.showErrorDialog(ex);
            if(ex.getLocalizedMessage().indexOf("Unable to find:") > -1)
            {
                currentTrack = mbv.showFileUnavailable(currentTrack);
                if(currentTrack != null)
                {
                    fFilename = currentTrack.getFilePath() + "/" + currentTrack.getFile();
                    play();
                    return;
                }
                else
                {
                    //mbv.next();
                    return;
                }
            }
            //mbv.stop();
            //ex.printStackTrace();
        }
        mbv.changed = 0;
        mbv.timeSkipped = 0;
    }

    public void play(SocketObject objectToPlay) 
    {
        try 
        {
            new Timer().schedule(new GetArt(), 100);
            new Timer().schedule(new GetWiki(), 200);
            mbv.playing = true;
            mbv.timeSkipped = 0;
            mbv.jLabel2.setText(currentTrack.getTrackLength());

            String time = currentTrack.getTrackLength();
            SimpleDateFormat format = new SimpleDateFormat("mm:ss");
            Date d = format.parse(time);
            Calendar cal = Calendar.getInstance();
            cal.setTime(d);
            int t = ((cal.get(Calendar.MINUTE)*60) + cal.get(Calendar.SECOND));
            mbv.trackLength = (((double)t));
        } 
        catch (ParseException ex) 
        {
            mbv.stop();
            mbv.showErrorDialog(ex);
        }

        if (currentTrack.getFile().toLowerCase().endsWith(".flac")) 
        {
            FlacAudioFileReader fafr = new FlacAudioFileReader();
            try
            {
                AudioInputStream flacAudioInputStream = fafr.getAudioInputStream(new ByteArrayInputStream(objectToPlay.getAudioBytes()));
                AudioFormat format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, 44100.0F, 16, 2, 4, 44100.0F, false);
                Flac2PcmAudioInputStream f2pcmStream = new Flac2PcmAudioInputStream(flacAudioInputStream, format, 100);
                din = AudioSystem.getAudioInputStream(format, f2pcmStream);
                rawplay(format, din);
            }
            catch(UnsupportedAudioFileException ex)
            {
                mbv.showErrorDialog(ex);
            }
            catch(IOException ex)
            {
                mbv.showErrorDialog(ex);
            }
            catch(LineUnavailableException ex)
            {
                mbv.showErrorDialog(ex);
            }
        } 
        else 
        {
            byte[] buf = objectToPlay.getAudioBytes();
            fileLength = buf.length;
            ByteArrayInputStream bais = new ByteArrayInputStream(buf);
            try 
            {
                in = AudioSystem.getAudioInputStream(bais/*file1*/);
            } 
            catch (Exception ex1) 
            {
                mbv.showErrorDialog(ex1);
                Bitstream m = new Bitstream(bais);
                long start = m.header_pos();    // need to open the stream again

                fileLength = fileLength - start;
                try 
                {
                    m.close();
                } catch (Exception ex) 
                {
                    mbv.showErrorDialog(ex);
                }
                bais = new ByteArrayInputStream(buf);
                bais.skip(start);
                try 
                {
                    in = AudioSystem.getAudioInputStream(bais);
                }
                catch (UnsupportedAudioFileException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
                catch (IOException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            }
            din = null;
            if (in != null) 
            {
                try 
                {
                    AudioFormat baseFormat = in.getFormat();
                    decodedFormat = new AudioFormat(
                    AudioFormat.Encoding.PCM_SIGNED,
                    baseFormat.getSampleRate(),
                    16,
                    baseFormat.getChannels(),
                    baseFormat.getChannels() * 2,
                    baseFormat.getSampleRate(),
                    false);
                       // Get AudioInputStream that will be decoded by underlying SPI
                    din = AudioSystem.getAudioInputStream(decodedFormat, in);
                    //aFF = AudioSystem.getAudioFileFormat(file1);
                    // Play now !
                    rawplay(decodedFormat, din);
                    in.close();
                } 
                catch (IOException ex) 
                {
                    mbv.showErrorDialog(ex);
                } 
                catch (LineUnavailableException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            }
        }
        mbv.changed = 0;
        mbv.timeSkipped = 0;
    }

    private String replaceSpace(String s) 
    {
        if (s.contains(" ")) 
        {
            s = s.replace(' ', '_');
        }
        return s;
    }

    @SuppressWarnings("static-access")
    public void rawplay(AudioFormat targetFormat, AudioInputStream d) throws IOException, LineUnavailableException 
    {
        aP = new AudioPermission("*");
        line = getLine(targetFormat);
        byte[] data = new byte[4096/*line.getBufferSize()*/]; //4096 was original
        //SourceDataLine source = (SourceDataLine)line;
        //mbv.jProgressBar1.setMaximum(100);
        mbv.executeTask();
        if (line != null)
        {
            // Start
            line.start();
            /*if (d.markSupported()) 
            {
                d.mark(Integer.MAX_VALUE);
            }*/
            nBytesRead = 0;
            int nBytesWritten = 0;
            while (nBytesRead != -1) 
            {
                if (doSeek == -1) 
                {

                    nBytesRead = d.read(data, 0, data.length);
                    if (nBytesRead != -1) 
                    {

                        nBytesWritten = line.write(data, 0, nBytesRead);
                    }
                } 
                else 
                {
                    line.flush();
                    line.close();
                    line.stop();
                    line = getLine(targetFormat);
                    in.reset();
                    d = AudioSystem.getAudioInputStream(targetFormat, in);
                    //din.close();
                    int skip = 0;
                    long skipped = 0;
                    long skippy = 0;
                    long previousSkipped = -1;
                    long previousS = -1;
                    //toSkip = (int) doSeek;
                    //doSeek = doSeek - doSeek % 4;
                    while (skippy < toSkip) 
                    {
                        skipped = d.skip(doSeek);
                        skippy += skipped;
                        doSeek = doSeek - skipped;
                        if (skipped == previousSkipped) 
                        {
                            if (previousSkipped == previousS) 
                            {
                                break;
                            } 
                            else 
                            {
                                previousS = previousSkipped;
                            }
                        }
                        previousSkipped = skipped;
                    }
                    double ratio = (double) skippy / (double) fileLength;
                    mbv.timeSkipped = ratio * mbv.trackLength;
                    if ((int) (ratio * 100) >= (int) (mbv.finalPercent * 100 - 1)) 
                    {
                        doSeek = -1;
                    } 
                    else 
                    {
                        doSeek = toSkip;
                    }
                    line.start();
                }
            }
            // Stop
            line.flush();
            line.drain();
            line.stop();
            line.close();
            d.close();
            //in.close();
        }

    }

    /**
     * 
     * @param audioFormat
     * @return
     * @throws LineUnavailableException 
     */
    private SourceDataLine getLine(AudioFormat audioFormat) throws LineUnavailableException 
    {
        SourceDataLine res = null;

        DataLine.Info info = new DataLine.Info(SourceDataLine.class, audioFormat);
        res = (SourceDataLine) AudioSystem.getLine(info);
        res.open(audioFormat);
        return res;
    }

    /**
     *
     */
    public void calculatePlaylistTime() 
    {
        if (mbv.currentlyPlaying2.size() > 0) 
        {
            String leng;
            if (!mbv.currentlyPlaying2.elementAt(0).startsWith("sharedLibraryTable")) 
            {
                leng = findTrack(mbv.currentlyPlaying2.elementAt(0)).getTrackLength();//mbv.allMusic.elementAt(mbv.currentlyPlaying.elementAt(0)).getTrackLength();
            } 
            else 
            {
                Vector<MusicOutputDesign> tmp = (Vector<MusicOutputDesign>)mbv.sharedLibrary.clone();
                Collections.sort(tmp, new IDComparator());
                leng = this.findTrack(tmp, mbv.currentlyPlaying2.elementAt(0).replace("sharedLibraryTable-", "")).getTrackLength();//mbv.sharedLibrary.elementAt((mbv.currentlyPlaying.elementAt(0)+1) * -1).getTrackLength();
            }
            int a = 0;
            int a1 = 0;
            if (leng.contains(":")) 
            {
                a = Integer.parseInt(leng.substring(0, leng.indexOf(":")));
                a1 = Integer.parseInt(leng.substring(leng.lastIndexOf(":") + 1, leng.length()));
            }
            Long m = new Long(minutesToMillis(a) + secondsToMillis(a1));
            try 
            {
                boolean shared = false;
                boolean local = false;
                Vector<MusicOutputDesign> temp1 = null;
                Vector<MusicOutputDesign> temp2 = null;
                for (int i = 1; i < mbv.currentlyPlaying2.size(); i++) 
                {
                    String l2 = "";
                    if(!mbv.currentlyPlaying2.elementAt(i).startsWith("sharedLibraryTable"))
                    {
                        if(!local)
                        {
                            temp1 = (Vector<MusicOutputDesign>)mbv.allMusic.clone();
                            Collections.sort(temp1, new IDComparator());
                            local = true;
                        }
                        l2 = findTrack(temp1, mbv.currentlyPlaying2.elementAt(i)).getTrackLength();
                    }
                    else
                    {
                        if(!shared)
                        {
                            temp2 = (Vector<MusicOutputDesign>)mbv.sharedLibrary.clone();
                            Collections.sort(temp2, new IDComparator());
                            shared = true;
                        }
                        l2 = findTrack(temp2, mbv.currentlyPlaying2.elementAt(i).replace("sharedLibraryTable-","")).getTrackLength();
                    }
                    /*if(!mbv.currentlyPlaying2.elementAt(i).startsWith("sharedLibraryTable"))
                    {
                        l2 = mbv.allMusic.elementAt(mbv.currentlyPlaying.elementAt(i)).getTrackLength();
                    }
                    else
                    {
                        l2 = mbv.sharedLibrary.elementAt((mbv.currentlyPlaying.elementAt(i)+1)*-1 ).getTrackLength();
                    }*/
                    if (l2.contains(":")) 
                    {
                        int b1 = Integer.parseInt(l2.substring(0, l2.indexOf(":")));
                        int b2 = Integer.parseInt(l2.substring(l2.lastIndexOf(":") + 1, l2.length()));
                        m = m + new Long(minutesToMillis(b1) + secondsToMillis(b2));
                    }
                }
                temp1 = null;
                temp2 = null;
                DatatypeFactory df = DatatypeFactory.newInstance();
                Duration d = df.newDuration(m);
                String totalTime = new String();
                if (d.getDays() > 0) 
                {
                    totalTime = totalTime + d.getDays() + " days, ";
                }
                if (d.getHours() > 0) 
                {
                    totalTime = totalTime + d.getHours() % 24 + " hours, ";
                }
                if (d.getMinutes() > 0) 
                {
                    totalTime = totalTime + d.getMinutes() % 60 + " minutes, ";
                }
                if (d.getSeconds() > 0) 
                {
                    totalTime = totalTime + d.getSeconds() % 60 + " seconds";
                }

                mbv.jLabel5.setText(totalTime);
            } 
            catch (DatatypeConfigurationException ex) 
            {
                ex.printStackTrace();
            }
        }
    }

    /**
     *
     * @param i
     * @return i Converts parameter i to milliseconds
     */
    public int minutesToMillis(int i) 
    {
        return i * 60000;
    }

    /**
     *
     * @param i
     * @return i Converts parameter i to milliseconds
     */
    public int secondsToMillis(int i) 
    {
        return i * 1000;
    }

    public void setMusicBoxView(MusicBoxView mbv) 
    {
        this.mbv = mbv;
        db = mbv.db;
    }

    public MusicBoxView getMusicBoxView() 
    {
        return mbv;
    }
}
/**
 * Playing file from URL (Streaming).
 */
