/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package musicbox.backend;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;
import java.lang.reflect.InvocationTargetException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.sql.SQLException;
import java.util.TimerTask;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.SwingUtilities;
import javax.swing.table.TableRowSorter;
import musicbox.backend.comparators.AlbumStringComparator;
import musicbox.backend.comparators.ArtistStringComparator;
import musicbox.backend.comparators.PlayCountStringComparator;
import musicbox.backend.comparators.TrackNumberStringComparator;
import musicbox.gui.CustomTableModel;
import musicbox.gui.MusicBoxView;
import noTalent.MusicOutputDesign;
import noTalent.SocketObject;

/**
 *
 * @author Isaac Hammon
 */
public class SocketListener
{

    private Socket socket;
    private MusicBoxView mbv;
    private java.util.Timer rec;
    private ObjectOutputStream oos;
    private ObjectInputStream ois;
    private SocketObject objectToPlay;
    private int songSize;

    public SocketListener(final Socket socket, MusicBoxView mBV) 
    {
        this.socket = socket;
        this.mbv = mBV;
        try 
        {
            this.socket.setSoTimeout(5000);
        }
        catch (SocketException ex) 
        {
            mbv.showErrorDialog(ex);
        }
        rec = new java.util.Timer();
        //rec.sch
        rec.schedule(new recMessage(), 1000/*, 500*/);
        new Thread() 
        {
            public void run() 
            {
                setName("Initialize ObjectInputStream");
                try 
                {
                    //System.out.println("initializing input stream");
                    ois = new ObjectInputStream(socket.getInputStream())
                    {
                        @Override
                        protected void readStreamHeader() throws IOException, StreamCorruptedException
                        {
                            
                        }
                        
                        @Override
                        protected Object readObjectOverride() throws IOException, ClassNotFoundException
                        {
                            Object o = readObject();
                            return o;
                        }
                    };
                    //System.out.println("initialized input stream");
                } 
                catch (IOException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            }
        }.start();
        
        new Thread() 
        {
            public void run() 
            {
                setName("Initialize ObjectOutputStream");
                try 
                {
                    oos = new ObjectOutputStream(socket.getOutputStream()) 
                    {
                        @Override
                        protected void writeStreamHeader() throws IOException 
                        {
                            // do not write a header
                        }
                    };
                    oos.flush();
                    SocketObject so = new SocketObject("GetAllAddresses");
                    new SocketWriteThread(oos,so,mbv).start();
                }
                catch (IOException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            }
        }.start();
    }
    
    public ObjectOutputStream getObjectOutputStream()
    {
        return oos;
    }
    
    class recMessage extends TimerTask 
    {
        public void run() 
        {
            while (socket.isConnected()) 
            {
                try 
                {
                    if (ois == null) 
                    {
                        //System.out.println("continuing");
                        continue;//return;
                    }
                    while(socket.getInputStream().available() == 0)
                    {
                        System.out.print("");
                    }
                    SocketObject so = null;
                    Object o = null;
                    try
                    {
                        o = ois.readObject();
                    }
                    catch(Exception e)
                    {
                        continue;
                    }
                    if(o == null)
                    {
                        //System.out.println("o is null");
                        continue;//return;
                    }
                        so = (SocketObject) o;
                        //System.out.println("MessageType: " + so.getMessageType());
                    final SocketObject so1 = so;
                    if (so.getMessageType().equalsIgnoreCase("Size")/*s.contains("<Size>")*/) {
                        try 
                        {
                            String size = so.getMessage();//s.substring(s.indexOf("<Size>") + 6, s.indexOf("</Size>"));
                            songSize = Integer.parseInt(size);
                        } 
                        catch (Exception e) 
                        {
                            mbv.showErrorDialog(e);
                        }

                    }
                    if (so.getMessageType().equalsIgnoreCase("SendLibraryDone")/*s.contains("<SendLibraryDone>")*/) 
                    {
                        updateSharedLibraryTable();
                    }
                    if (so.getMessageType().equalsIgnoreCase("GetLibrary")/*s.contains("<GetLibrary>")*/) 
                    {
                        String sendTo = so.getSource();//s.substring(s.indexOf("<TO>") + 4, s.indexOf("</TO>"));
                        sendLibraryTo(sendTo);
                    }
                    if(so.getMessageType().equalsIgnoreCase("GetAllPlaylists"))
                    {
                        sendAllPlaylistsTo(so.getSource());
                    }
                    if(so.getMessageType().equalsIgnoreCase("PlayListName"))
                    {
                        updateSharedPlayList(so);
                    }
                    if(so.getMessageType().equalsIgnoreCase("GetPlaylist"))
                    {
                        sendPlayList(so);
                    }
                    if (so.getMessageType().equalsIgnoreCase("Address")/*s.contains("<Address>")*/) 
                    {
                        addAddressToList(so.getSource()/*s*/);
                    }
                    if(so.getMessageType().equalsIgnoreCase("Library"))
                    {
                        new Thread() {

                            public void run() 
                            {
                                setName("calling addToSharedLibrary");
                                addToSharedLibrary(so1/*,socket.getInputStream()*/);
                                //pool = "";
                            }
                        }.start();
                    }
                    if (so.getMessageType().equalsIgnoreCase("MusicOutputDesignDestination")/*s.contains("<MusicOutputDesignDestination>")*/ /*&& !recievingLibraryData*/) 
                    {
                        new Thread() 
                        {
                            public void run()
                            {
                                setName("calling addToSharedLibrary");
                                addToSharedLibrary(so1/*,socket.getInputStream()*/);
                            }
                        }.start();
                    }
                    if (so.getMessageType().equalsIgnoreCase("GetStreamFrom")/*s.contains("<GetStreamFrom>")*/)
                    {

                        final String address = so.getSource();//s.substring(s.indexOf("<SendTo>") + 8, s.indexOf("</SendTo>"));
                        final String audioFile = so.getMessage();//s.substring(s.indexOf("<File>") + 6, s.indexOf("</File>"));
                        new Thread() 
                        {
                            public void run() 
                            {
                                setName("calling sendAudioStream");
                                sendAudioStream(address, audioFile);
                            }
                        }.start();
                    }
                    if (so.getMessageType().equalsIgnoreCase("SendToAddress")/*s.contains("<SendToAddress>")*/) 
                    {
                        setObjectToPlay(so);
                    }
                    if (so.getMessageType().equalsIgnoreCase("Done")/*s.contains("<Done>")*/)
                    {
                        //System.out.println("received done");

                    }
                } 
                catch (IOException ex) 
                {
                    mbv.showErrorDialog(ex);
                    break;
                }
            }
            mbv.sharedLibraryTable.setModel(new CustomTableModel());
            mbv.sharedLibrary.clear();
            ((DefaultListModel)mbv.addressList.getModel()).removeAllElements();
            ((DefaultListModel)mbv.sharedPlayList.getModel()).removeAllElements();
        }   
    }
    
    private void sendAllPlaylistsTo(String source) 
        {
            try 
            {
                SocketObject so = new SocketObject(null, "PlayListName", "(All Music)", source);
                (new SocketWriteThread(oos, so, mbv)).start();
                if(!mbv.currentlyPlaying2.isEmpty())
                {
                    so = new SocketObject(null, "PlayListName", "Currently Playing", source);
                    (new SocketWriteThread(oos, so, mbv)).start();
                }
                for(int i = 0; i < mbv.jList3.getModel().getSize(); i++)
                {
                    so = new SocketObject(null, "PlayListName", (String)mbv.jList3.getModel().getElementAt(i), source);
                    (new SocketWriteThread(oos, so, mbv)).start();
                }
            } 
            catch (UnknownHostException ex) 
            {
                mbv.showErrorDialog(ex);
            }
        }

        private void updateSharedPlayList(SocketObject so) 
        {
            if(!mbv.jScrollPane12.isVisible())
            {
                mbv.jScrollPane12.setVisible(true);
            }
            ((DefaultListModel)mbv.sharedPlayList.getModel()).addElement(so.getMessage());
        }

        private void sendPlayList(SocketObject so) 
        {
            Vector<MusicOutputDesign> tmp = new Vector<MusicOutputDesign>();
            if(so.getMessage().equalsIgnoreCase("(All Music)"))
            {
                tmp.addAll(mbv.allMusic);
            }
            else if(so.getMessage().equalsIgnoreCase("Currently Playing"))
            {
                for(String id: mbv.currentlyPlaying2)
                {
                    tmp.add(mbv.back.findTrack(id));
                }
            }
            else
            {
                try 
                {
                    Vector<String> temp = mbv.db.listPlayList(mbv.db.getConnection().createStatement(), so.getMessage());
                    //Vector<MusicOutputDesign> tmp = new Vector<MusicOutputDesign>();
                    for(String id: temp)
                    {
                        tmp.add(mbv.back.findTrack(id));
                    }
                }
                catch (SQLException ex) 
                {
                    mbv.showErrorDialog(ex);
                }
            }
            SocketObject send = new SocketObject(new MusicOutputDesign(), "Library", "MusicOutputDesigns", socket.getLocalAddress().getHostAddress(), so.getSource(), new byte[0]);
            send.setList(tmp);
            new SocketWriteThread(oos,send,mbv).start();
        }

    public void getAudioStream(String address, String audioFile) {
        SocketObject so = null;
        try 
        {
            so = new SocketObject("GetStreamFrom", audioFile, address);
        } 
        catch (UnknownHostException ex) 
        {
            mbv.showErrorDialog(ex);//Logger.getLogger(SocketListener.class.getName()).log(Level.SEVERE, null, ex);
        }
        new SocketWriteThread(oos,so,mbv).start();
    }

    public void sendAudioStream(final String address, final String audioFile) 
    {
        new Thread() 
        {
            public void run() 
            {
                setName("running sendAudioStream");
                File file1 = new File(audioFile);
                //AudioInputStream in = null;
                byte[] buf = null;
                try 
                {
                    BufferedInputStream fis = new BufferedInputStream(new FileInputStream(file1));
                    //while (fis.available() < file1.length())
                   //     System.out.print("");
                    buf = new byte[fis.available()];
                    fis.read(buf, 0, buf.length);
                    fis.close();
                } 
                catch (Exception ex1) 
                {
                    mbv.showErrorDialog(ex1);
                }
                SocketObject so;
                if (buf != null) 
                {
                    so = new SocketObject(null, "SendToAddress", "", socket.getLocalAddress().getHostAddress(), address, buf);
                }
                else
                {
                    //System.out.println("File "+ audioFile + " was not found.");
                    so = new SocketObject(null, "FileNotFound", "", socket.getLocalAddress().getHostAddress(), address, null);
                }
                //System.out.println("destination: " + so.getDestination());
                new SocketWriteThread(oos,so,mbv).start();
                buf = null;
                stop();
            }
        }.start();
    }

    private void addToSharedLibrary(SocketObject so) 
    {
        //if (mbv.sharedLibrary == null) 
        //{
            mbv.sharedLibrary = (Vector<MusicOutputDesign>)so.getList().clone();//new Vector<MusicOutputDesign>();
        //}
        for(int i = 0; i < mbv.sharedLibrary.size(); i++)
        {
            MusicOutputDesign mod = mbv.sharedLibrary.elementAt(i);
            String rating = mod.getRating();
            ImageIcon ico = mbv.smileys[0];
            try 
            {
                ico = mbv.smileys[Integer.parseInt(rating)];
            } 
            catch (NumberFormatException ex) 
            {
                ico = mbv.smileys[0];
            }
            final Object[] o = new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(), mod.getTrackLength(), mod.getPlayCount(), ico};
            if (((CustomTableModel) mbv.sharedLibraryTable.getModel()).getColumnCount() == 0) 
            {
                try 
                {
                    SwingUtilities.invokeAndWait(new Runnable() 
                    {
                        public void run() 
                        {
                            CustomTableModel ctm = (CustomTableModel) mbv.sharedLibraryTable.getModel();
                            TableRowSorter<CustomTableModel> sorter3 = new TableRowSorter<CustomTableModel>(ctm);
                            if (ctm.getColumnCount() == 0) 
                            {
                                for (int i = 0; i < mbv.columnNames.length; i++) 
                                {
                                    ctm.addColumn(mbv.columnNames[i]);
                                }
                                sorter3.setComparator(ctm.findColumn("Track Number"), new TrackNumberStringComparator());
                                sorter3.setComparator(ctm.findColumn("Artist"), new ArtistStringComparator());
                                sorter3.setComparator(ctm.findColumn("Album"), new AlbumStringComparator());
                                sorter3.setComparator(ctm.findColumn("Play Count"), new PlayCountStringComparator());
                                mbv.sharedLibraryTable.setRowSorter(sorter3);
                            }
                            ctm.addRow(o);
                        }
                    });
                } 
                catch (InterruptedException ex) 
                {
                    mbv.showErrorDialog(ex);
                } 
                catch (InvocationTargetException ex)
                {
                    mbv.showErrorDialog(ex);
                }
            } 
            else 
            {
                CustomTableModel ctm = (CustomTableModel) mbv.sharedLibraryTable.getModel();
                ctm.addRow(o);
            }
        }
        mbv.sharedLibraryTable.setRowHeight(16);
        //mbv.sharedLibrary.add(so.getMOD());
    }

    private void updateSharedLibraryTable() 
    {
        if (!SwingUtilities.isEventDispatchThread()) 
        {
            try 
            {
                SwingUtilities.invokeAndWait(new Runnable() 
                {
                    public void run() 
                    {
                        //System.out.println("Running new event in EDT");
                        CustomTableModel ctm = (CustomTableModel) mbv.sharedLibraryTable.getModel();
                        if (ctm.getColumnCount() == 0) 
                        {
                            for (int i = 0; i < mbv.columnNames.length; i++) 
                            {
                                ctm.addColumn(mbv.columnNames[i]);
                            }
                        }
                        //ctm.addRow(o);
                        ctm.fireTableDataChanged();
                        //((CustomTableModel) MusicBoxView.sharedLibraryTable.getModel()).setNumRows(((CustomTableModel) MusicBoxView.sharedLibraryTable.getModel()).getRowCount() +1);
                        //((CustomTableModel) MusicBoxView.sharedLibraryTable.getModel()).addRow(o);
                        //((CustomTableModel) mbv.sharedLibraryTable.getModel()).fireTableDataChanged();
                    }
                });
            } 
            catch (InterruptedException ex) 
            {
                mbv.showErrorDialog(ex);
            } 
            catch (InvocationTargetException ex) 
            {
                mbv.showErrorDialog(ex);
            }
        } 
        else 
        {
            ((CustomTableModel) mbv.sharedLibraryTable.getModel()).fireTableDataChanged();
        }

    }

    private void sendLibraryTo(final String sendTo) 
    {
        SocketObject so = new SocketObject(new MusicOutputDesign(), "Library", "MusicOutputDesigns", socket.getLocalAddress().getHostAddress(), sendTo, new byte[0]);
        so.setList(mbv.allMusic);
        new SocketWriteThread(oos,so,mbv).start();
    }

    private void addAddressToList(String address) 
    {
        try 
        {
            InetAddress addr = InetAddress.getByName(address);
            //System.out.println(addr.getHostName());
            String hostName = addr.getHostName();
            //System.out.println(!((DefaultListModel) mbv.addressList.getModel()).contains(hostName));
            //System.out.println("Host name: " + hostName);
            if (!((DefaultListModel) mbv.addressList.getModel()).contains(hostName))
            {
                ((DefaultListModel) mbv.addressList.getModel()).addElement(hostName);
            }
        }
        catch (UnknownHostException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }
    public void setObjectToPlay(SocketObject objectToPlay)
    {
        this.objectToPlay = objectToPlay;
    }
    
    public SocketObject getObjectToPlay()
    {
        return objectToPlay;
    }
    
    public void disconnect()
    {
        rec.cancel();
        rec.purge();
        try 
        {
            socket.close();
        } 
        catch (IOException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }
}
