/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package musicbox.database;

import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import java.util.ArrayList;
import java.util.Map;
import java.util.StringTokenizer;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioSystem;
import javax.swing.DefaultListModel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import musicbox.backend.filefilters.AudioFileFilter;
import musicbox.backend.filefilters.DirListFilter;
import musicbox.gui.MusicBoxView;
import noTalent.MusicOutputDesign;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.AudioHeader;
import org.jaudiotagger.audio.mp3.MP3AudioHeader;
import org.jaudiotagger.audio.mp3.MP3File;

/**
 *
 * @author Isaac Hammon
 */
public class Database 
{

    private Connection con;
    private Connection customConnection;
    private Vector<File> playlistFiles;
    private Vector<File> audio;
    private boolean stillInEDT = false;
    private Connection connection;
    private MusicBoxView mbv;

    /**
     *
     */
    public Database(MusicBoxView mbv) 
    {
        this.mbv = mbv;
        playlistFiles = new Vector<File>();
        audio = new Vector<File>();
        try 
        {
            File f = new File("resources/MDB.odb");
            Class.forName("org.hsqldb.jdbcDriver");
            con = DriverManager.getConnection("jdbc:hsqldb:file:" + f.getAbsolutePath(), "SA", "");
            con.setAutoCommit(true);
            connection = con;
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        } 
        catch (ClassNotFoundException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    /**
     *
     * @param stmt
     * @param colName The column to be returned from the database.
     * @return list A list of everything in the requested column.
     */
    public ResultSet list(java.sql.Statement stmt, String colName) 
    {
        ResultSet rs = null;
        try 
        {
            String sel = "SELECT " + colName + " FROM MusicLibrary ORDER BY artist, album, trackNumber asc";
            rs = stmt.executeQuery(sel);
        } 
        catch (Exception e)
        {
            mbv.showErrorDialog(e);
            //e.printStackTrace();
        }
        return rs;
    }

    /**
     *
     * @param stmt
     * @param m
     * @throws SQLException
     */
    public void addToBetterMusicLibrary(Statement stmt, MusicOutputDesign m) 
    {
        String trackTitle = " ";
        String artist = " ";
        String album = " ";
        int trackNumber;
        if (m.getTrackNumber().length() <= 0) 
        {
            trackNumber = 0;
        } 
        else 
        {
            if (m.getTrackNumber().contains("/")) 
            {
                trackNumber = Integer.parseInt(m.getTrackNumber().substring(0, m.getTrackNumber().indexOf("/")));
            } 
            else if (m.getTrackNumber().contains("\\")) 
            {
                trackNumber = Integer.parseInt(m.getTrackNumber().substring(0, m.getTrackNumber().indexOf("\\")));
            }
            else 
            {
                try 
                {
                    trackNumber = Integer.parseInt(m.getTrackNumber());
                } 
                catch (NumberFormatException ex) 
                {
                    System.out.println("Track number improperly formatted.\nSetting to zero.");
                    trackNumber = 0;
                }
            }
        }
        if (m.getTrackTitle().length() > 0) 
        {
            trackTitle = replaceApostrophe(m.getTrackTitle());
        } 
        else 
        {
            trackTitle = "N/A";
        }
        if (m.getArtist().length() > 0) 
        {
            artist = replaceApostrophe(m.getArtist());
        } 
        else 
        {
            artist = "N/A";
        }
        if (m.getAlbum().length() > 0) 
        {
            album = replaceApostrophe(m.getAlbum());
        } 
        else 
        {
            album = "N/A";
        }
        String filePath = replaceApostrophe(m.getFilePath());
        String file = replaceApostrophe(m.getFile());
        if (m.getTrackNumber().compareTo(" ") == 0 || m.getTrackNumber() == null) 
        {
            m.setTrackNumber("" + 1);
        }
        if (m.getTrackNumber().contains("/")) 
        {
            m.setTrackNumber(m.getTrackNumber().substring(0, m.getTrackNumber().indexOf("/")));
        }
        String id = replaceApostrophe(m.getId());
        String check = "Select * From MusicLibrary WHERE ID = '" + id + "'";//filePath = \'" + filePath + "\' AND file = \'" + file + "\'";
        try 
        {
            ResultSet rs = stmt.executeQuery(check);
            boolean next = rs.next();
            if (!next) 
            {
                try 
                {
                    String insert = "INSERT INTO MusicLibrary VALUES ( \'" + filePath + "\', \'" + file + "\', " + trackNumber + ", \'" + trackTitle + "\', \'" + artist + "\', \'" + album + "\', \'" + m.getTrackLength() + "\', 0, 0, \'NoImage\', \'" + id + "\')";
                    stmt.executeUpdate(insert);
                } 
                catch (SQLException ex)
                {
                    mbv.showErrorDialog(ex);
                }
            } 
            else 
            {
                System.out.print("");
            }
        } 
        catch (SQLException ex)
        {
            mbv.showErrorDialog(ex);
        }

    }

    /**
     *
     * @param stmt
     * @param m
     * @throws SQLException
     */
    public void deleteFromMusicLibrary(Statement stmt, MusicOutputDesign m) throws SQLException
    {
        String id = replaceApostrophe(m.getId());
        String delete = "DELETE FROM MusicLibrary WHERE ID = \'" + id + "\'";
        int retVal = stmt.executeUpdate(delete);
        stmt.close();
        
    }

    /**
     *
     * @param stmt
     * @return rs ResultSet of the SQL query.
     * @throws SQLException
     */
    public ResultSet listDuplicateRecords(Statement stmt) throws SQLException 
    {
        String delete = "select TRACKTITLE, "
                + "COUNT(TRACKTITLE) AS NumTracks "
                + "FROM \"PUBLIC\".MUSICLIBRARY "
                + "WHERE ARTIST = ARTIST AND ALBUM = ALBUM AND TRACKTITLE = TRACKTITLE "
                + "GROUP BY TRACKTITLE "
                + "HAVING (COUNT(TRACKTITLE) > 1)";
        ResultSet rs = stmt.executeQuery(delete);
        return rs;
    }

    /**
     *
     * @param stmt
     * @return ResultSet ResultSet from SQL query.
     * @throws SQLException
     */
    public ResultSet selectAll(Statement stmt) throws SQLException 
    {
        String select = "SELECT * FROM MUSICLIBRARY ORDER BY artist, album, trackNumber";
        return stmt.executeQuery(select);
    }

    public Object[] selectDistinct(Statement stmt, String column) throws SQLException 
    {
        //String[] values = null;
        String select = "SELECT DISTINCT " + column + " FROM MUSICLIBRARY ORDER BY " + column;
        ResultSet rs = stmt.executeQuery(select);
        ArrayList<String> temp = new ArrayList<String>();
        while (rs.next())
        {
            temp.add(rs.getString(1));
        }
        Object[] array = temp.toArray();
        //values = (String[]) temp.toArray();
        return array;
    }

    public MusicOutputDesign selectTrack(String ID) 
    {
        MusicOutputDesign mod = new MusicOutputDesign();
        mod.setId(ID);
        try 
        {
            Statement stmt = con.createStatement();
            String query = "SELECT * FROM MUSICLIBRARY WHERE ID = '" + replaceApostrophe(ID) + "'";
            ResultSet rs = stmt.executeQuery(query);
            if (rs.next()) 
            {
                mod = new MusicOutputDesign(rs.getString("filePath"), rs.getString("file"), rs.getString("trackNumber"), rs.getString("trackTitle"), rs.getString("artist"), rs.getString("album"), rs.getString("trackLength"), rs.getString("playCount"), rs.getString("rating"), rs.getString("imagePath"), rs.getString("id"));
            }
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }

        return mod;
    }

    /**
     *
     * @param stmt
     * @param m
     * @throws SQLException
     */
    public void updatePlayCount(Statement stmt, MusicOutputDesign m) throws SQLException 
    {
        String id = replaceApostrophe(m.getId());
        String update = "UPDATE MusicLibrary SET PlayCount = " + m.getPlayCount() + " WHERE ID = \'" + id + "\'";
        stmt.executeUpdate(update);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param m
     * @throws SQLException
     */
    public void updateRating(Statement stmt, MusicOutputDesign m) throws SQLException 
    {
        String id = replaceApostrophe(m.getId());
        String update = "UPDATE MusicLibrary SET rating = " + m.getRating() + " WHERE ID = \'" + id + "\'";
        stmt.executeUpdate(update);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param oldM
     * @param newM
     * @throws SQLException
     */
    public void updateTags(Statement stmt, MusicOutputDesign oldM, MusicOutputDesign newM) throws SQLException 
    {
        String newfilePath = replaceApostrophe(newM.getFilePath());
        String newfile = replaceApostrophe(newM.getFile());
        String newalbum = replaceApostrophe(newM.getAlbum());
        String newartist = replaceApostrophe(newM.getArtist());
        String newtrackTitle = replaceApostrophe(newM.getTrackTitle());
        String imagePath = replaceApostrophe(newM.getImagePath());
        String id = replaceApostrophe(oldM.getId());
        if (newM.getTrackNumber().compareTo(" ") == 0 || newM.getTrackNumber() == null) 
        {
            newM.setTrackNumber("" + 1);
        }
        String update = "UPDATE MusicLibrary SET FilePath = \'" + newfilePath + "\', File = \'" + newfile + "\', album = \'" + newalbum + "\', artist = \'" + newartist + "\', trackTitle = \'" + newtrackTitle + "\', trackNumber = " + newM.getTrackNumber() + ", rating = " + newM.getRating()
                + ", IMAGEPATH = \'" + imagePath + "\', trackLength = \'" + newM.getTrackLength() + "\' WHERE ID = \'" + id + "\'";
        int r = stmt.executeUpdate(update);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param m
     * @throws SQLException
     */
    public void updateImagePath(Statement stmt, MusicOutputDesign m) throws SQLException 
    {
        String newPath = replaceApostrophe(m.getImagePath());
        String id = replaceApostrophe(m.getId());
        String update = "UPDATE MusicLibrary SET IMAGEPATH = \'" + newPath + "\' WHERE ID = \'" + id + "\'";
        int r = stmt.executeUpdate(update);
        //stmt.execute("COMMIT");
    }

    /**
     *
     * @param results
     * @return columnNames Vector of columnNames from ResultSet results.
     * @throws SQLException
     */
    public Vector<String> getColumnNames(ResultSet results) throws SQLException 
    {
        Vector<String> columnNames = new Vector<String>();
        ResultSetMetaData metaData = results.getMetaData();
        int columnCount = metaData.getColumnCount();
        for (int i = 1; i <= columnCount; i++)
        {
            columnNames.add(metaData.getColumnName(i));
        }
        return columnNames;
    }

    /**
     *
     * @param stmt
     */
    public void deleteAll(Statement stmt) 
    {
        //TODO add gui thread thingy
        try 
        {
            //JOptionPane.showMessageDialog(mbv.mainPanel, "Deleting all files from library", "Delete All", JOptionPane.PLAIN_MESSAGE);
            mbv.currentTrackInfoBox.setText("Deleting all files from Library");
            String exec = "DELETE FROM MusicLibrary";
            stmt.executeUpdate(exec);
            //mbv.jTextArea2.append("\nDone");
        } 
        catch (Exception e) 
        {
            //e.printStackTrace();
            mbv.showErrorDialog(e);
        }
    }

    /**
     *
     * @param stmt
     * @param name
     * @throws SQLException
     */
    public void deleteFromPlaylistTable(Statement stmt, String name) throws SQLException
    {
        String n = replaceApostrophe(name);
        if (n.contains(" ")) 
        {
            n = n.replace(" ", "_");
        }
        String delete = "DELETE FROM Playlists WHERE name = \'" + n + "\'";
        stmt.executeUpdate(delete);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param name
     * @param filePath
     * @param file
     * @throws SQLException
     */
    public void deleteSongFromPlaylist(Statement stmt, String name, String id) throws SQLException 
    {
        String n = replaceApostrophe(name);
        if (n.contains(" ")) 
        {
            n = n.replace(" ", "_");
        }
        String delete = "DELETE FROM " + n + " WHERE id = \'" + replaceApostrophe(id) + "\'";
        stmt.executeUpdate(delete);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param name
     * @throws SQLException
     */
    public void deleteEntirePlaylist(Statement stmt, String name) throws SQLException 
    {
        String n = replaceApostrophe(name);
        if (n.contains(" ")) 
        {
            n = n.replace(" ", "_");
        }
        String delete = "DROP TABLE " + n;
        stmt.executeUpdate(delete);
        stmt.execute("COMMIT");
    }

    /**
     *
     * @param stmt
     * @param name
     */
    public void createTable(Statement stmt, String name) 
    {
        try 
        {
            String n = replaceApostrophe(name);
            PreparedStatement ps = connection.prepareStatement("CREATE TABLE " + name + " (ID  VARCHAR(255))");
            boolean b = ps.execute();
        } 
        catch (Exception e)
        {
            //e.printStackTrace();
            mbv.showErrorDialog(e);
        }
    }
    
    public void createCurrentlyPlayingTable()
    {
        String sql = " CREATE TABLE CurrentlyPlaying (PLAYORDER INTEGER, ID VARCHAR(255), CURRENTTRACK INTEGER)";
        try 
        {
            connection.createStatement().execute(sql);
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    /**
     * 
     * @param stmt
     * @param name
     * @param filePath
     * @param file
     */
    public void addToPlayList(Statement stmt, String name, String id) 
    {
        /*String fPath = replaceApostrophe(filePath);
        String f = replaceApostrophe(file);
        String n = replaceApostrophe(name);
        if (n.contains(" ")) {
        n = n.replace(" ", "_");
        }*/

        //String insert = "INSERT INTO " + n + " VALUES ( \'" + fPath + "\', \'" + f + "\')";
        try 
        {
            PreparedStatement ps = connection.prepareStatement("INSERT INTO " + name + " VALUES (?)");
            ps.setString(1, id);
            boolean b = ps.execute();
            //stmt.executeUpdate(insert);
            //stmt.execute("COMMIT");
        } 
        catch (SQLException ex) 
        {
            //ex.printStackTrace();
            mbv.showErrorDialog(ex);
        }
    }

    /**
     *
     * @param stmt
     * @param name
     * @return list
     */
    public Vector<String> listPlayList(java.sql.Statement stmt, String name) 
    {
        Vector<String> list = new Vector();
        try 
        {
            String n = replaceApostrophe(name);
            if (n.contains(" ")) 
            {
                n = n.replace(" ", "_");
            }
            String sel = "SELECT * FROM " + n + " ORDER BY ID";

            ResultSet rs = stmt.executeQuery(sel);
            Vector<String> colNames = getColumnNames(rs);
            int leng = colNames.size();
            while (rs.next()) 
            {
                for (int i = 1; i <= leng; i++) 
                {
                    list.add(rs.getString(i));
                }
            }
        }
        catch (Exception e) 
        {
            mbv.showErrorDialog(e);
            //e.printStackTrace();
        }
        return list;
    }

    public void updatePlayOrder(Vector<String> v) 
    {
        for (int i = 0; i < v.size(); i++) 
        {
            String update = "Update CurrentlyPlaying Set PlayOrder = " + i + " Where ID = \'" + replaceApostrophe(v.elementAt(i)) + "\'";
            try 
            {
                Statement stmt = con.createStatement();
                stmt.executeUpdate(update);
                stmt.close();
            } 
            catch (SQLException ex)
            {
                mbv.showErrorDialog(ex);
            }
        }
    }

    public void updateCurrentTrack(String id) 
    {
        try 
        {
            Statement stmt = con.createStatement();
            stmt.executeUpdate("Update CurrentlyPlaying Set CurrentTrack = 0 Where CurrentTrack = 1");
            stmt.executeUpdate("Update CurrentlyPlaying Set CurrentTrack = 1 Where ID = \'" + replaceApostrophe(id) + "\'");
            stmt.close();
        }
        catch (SQLException ex) 
        {
            ex.printStackTrace();
            //mbv.showErrorDialog(ex);
        }
    }

    public void deleteFromCurrentlyPlaying(String id) 
    {
        try 
        {
            String query = "Delete from CurrentlyPlaying Where ID = \'" + replaceApostrophe(id) + "\'";
            Statement stmt = con.createStatement();
            stmt.executeUpdate(query);
            stmt.close();
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    public void insertIntoCurrentlyPlaying(int playOrder, String id) 
    {
        try 
        {
            String query = "Insert Into CurrentlyPlaying (PlayOrder, ID, CurrentTrack) Values (" + playOrder + ",\'" + this.replaceApostrophe(id) + "\'," + ((playOrder == mbv.back.currentLocation) ? (1) : (0)) + ");";
            Statement stmt = con.createStatement();
            stmt.executeUpdate(query);
            stmt.close();
        }
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    /**
     *
     * @param stmt
     * @param name
     */
    public void addToPlayListTable(String name, String file) 
    {
        String n = replaceApostrophe(name);
        String insert = "INSERT INTO PlayLists VALUES (?,?)";
        try 
        {
            PreparedStatement ps = connection.prepareStatement(insert);
            ps.setString(1, name);
            ps.setString(2, file);
            ps.executeUpdate();
        } 
        catch (SQLException ex) 
        {
            //ex.printStackTrace();
            mbv.showErrorDialog(ex);
        }
    }

    /**
     *
     * @param s
     * @return s
     */
    public String replaceApostrophe(String s) {
        String str = "";
        if (s == null) {
            return " ";
        }
        if (s.length() == 0) {
            str = " ";
            return str;
        }
        if (s.contains("\'")) {
            String str1 = s.replace("\'", "\'\'");
            return str1;
        }
        return s;
    }

    public String replaceApostropheAndSlash(String s) 
    {
        String str = "";
        if (s == null) 
        {
            return " ";
        }
        if (s.length() == 0) 
        {
            str = " ";
            return str;
        }
        if (s.contains("\'")) 
        {
            s = s.replace("\'", "\'\'");
        }
        if (s.contains("\\")) 
        {
            s = s.replace("\\", "_");
        }
        if (s.contains("/")) 
        {
            s = s.replace("/", "_");
        }

        return s;
    }

    /**
     *
     * @param myFile
     * @param stmt
     */
    public MusicOutputDesign insertMusicFile(final java.io.File myFile, final Statement stmt) 
    {
        try 
        {
            if (myFile.getName().toLowerCase().endsWith(".m3u")) 
            {
                playlistFiles.add(myFile);
                return null;
            } 
            else 
            {
                MusicOutputDesign m = new MusicOutputDesign();
                AudioFile file = null;
                AudioHeader audioHeader = null;
                String trackLength = null;
                if (!myFile.getName().toLowerCase().endsWith(".ape") && !myFile.getName().toLowerCase().endsWith(".au")
                        && !myFile.getName().toLowerCase().endsWith(".aif")) 
                {
                    file = AudioFileIO.read(myFile);
                    audioHeader = file.getAudioHeader();
                    trackLength = "" + audioHeader.getTrackLength();
                    int d = audioHeader.getTrackLength();
                    double e = d;
                    e = e / 60;
                    int f1 = (int) e;
                    e = e - f1;
                    e = e * 60;
                    if (e < 10)
                    {
                        if (f1 < 10) 
                        {
                            trackLength = "0" + f1 + ":0" + (int) e;
                        } 
                        else
                        {
                            trackLength = f1 + ":0" + (int) e;
                        }
                    } 
                    else 
                    {
                        if (f1 < 10) 
                        {
                            trackLength = "0" + f1 + ":" + (int) e;
                        } else 
                        {
                            trackLength = f1 + ":" + (int) e;
                        }
                    }
                }
                if (myFile.getName().toLowerCase().endsWith(".ogg")) 
                {
                    m = new MusicOutputDesign(myFile.getParent(), myFile.getName(), file.getTag().getFirstTrack(), file.getTag().getFirstTitle(), file.getTag().getFirstArtist(), file.getTag().getFirstAlbum(), trackLength, "0");
                } 
                else if (myFile.getName().toLowerCase().endsWith(".flac") || myFile.getName().toLowerCase().endsWith(".wav")
                        || myFile.getName().toLowerCase().endsWith(".aifc") || myFile.getName().toLowerCase().endsWith(".aiff")) 
                {
                    m = new MusicOutputDesign(myFile.getParent(), myFile.getName(), file.getTag().getFirstTrack(), file.getTag().getFirstTitle(), file.getTag().getFirstArtist(), file.getTag().getFirstAlbum(), trackLength, "0");
                } 
                else if (myFile.getName().toLowerCase().endsWith(".mp3")) 
                {
                    MP3File f = (MP3File) AudioFileIO.read(myFile);
                    MP3AudioHeader aHeader = null;
                    if (f.getAudioHeader() != null) 
                    {
                        aHeader = f.getMP3AudioHeader();
                    }
                    String trackNumber = new String();
                    String trackTitle = new String();
                    String artist = new String();
                    String album = new String();
                    String tl = new String();
                    if (f.getTag() != null) 
                    {
                        if (f.getTag().getFirstTrack() != null) 
                        {
                            trackNumber = f.getTag().getFirstTrack();
                        } 
                        else
                        {
                            trackNumber = "0";
                        }
                        if (f.getTag().getFirstTitle() != null) {
                            trackTitle = f.getTag().getFirstTitle();
                        } 
                        else 
                        {
                            trackTitle = "N/A";
                        }
                        if (f.getTag().getFirstArtist() != null) 
                        {
                            artist = f.getTag().getFirstArtist();
                        } 
                        else 
                        {
                            artist = "N/A";
                        }
                        if (f.getTag().getFirstAlbum() != null) 
                        {
                            album = f.getTag().getFirstAlbum();
                        } 
                        else 
                        {
                            album = "N/A";
                        }
                        if (aHeader != null && aHeader.getTrackLengthAsString() != null) 
                        {
                            tl = aHeader.getTrackLengthAsString();
                        } 
                        else 
                        {
                            tl = "00:00";
                        }
                    } 
                    else 
                    {
                        trackNumber = "0";
                        trackTitle = "N/A";
                        artist = "N/A";
                        album = "N/A";
                        if (aHeader != null && aHeader.getTrackLengthAsString() != null)
                        {
                            tl = aHeader.getTrackLengthAsString();
                        }
                        else 
                        {
                            tl = "00:00";
                        }
                    }
                    m = new MusicOutputDesign(myFile.getParent(), myFile.getName(), trackNumber, trackTitle, artist, album, tl, "0");
                } 
                else if (myFile.getName().toLowerCase().endsWith(".ape") || myFile.getName().toLowerCase().endsWith(".au")
                        || myFile.getName().toLowerCase().endsWith(".aif")) 
                {
                    /*
                     *TODO:  need to work on this!
                     */
                    String author = " ";
                    String album = " ";
                    String track = " ";
                    String title = " ";
                    Long duration = new Long(0);
                    AudioFileFormat aFF = AudioSystem.getAudioFileFormat(myFile);
                    Map properties = aFF.properties();
                    if (properties.containsKey("author")) 
                    {
                        author = author + (String) properties.get("author");
                    }
                    if (properties.containsKey("album")) 
                    {
                        album = album + (String) properties.get("album");
                    }
                    if (properties.containsKey("track")) 
                    {
                        track = track + (String) properties.get("track");
                    }
                    if (properties.containsKey("duration")) 
                    {
                        duration = duration + (Long) properties.get("duration");
                    }
                    String tLength = new String();
                    int d = duration.intValue();
                    double e = d / 1000;
                    e = e / 60;
                    int f1 = (int) e;
                    e = e - f1;
                    e = e * 60;
                    if (e < 10) 
                    {
                        if (f1 < 10) 
                        {
                            tLength = "0" + f1 + ":0" + (int) e;
                        } 
                        else 
                        {
                            tLength = f1 + ":0" + (int) e;
                        }
                    }
                    else 
                    {
                        if (f1 < 10)
                        {
                            tLength = "0" + f1 + ":" + (int) e;
                        } 
                        else 
                        {
                            tLength = f1 + ":" + (int) e;
                        }

                    }
                    m = new MusicOutputDesign(myFile.getParent(), myFile.getName(), track, title, author, album, tLength, "0");
                }
                addToBetterMusicLibrary(stmt, m);
                return m;
            }
            //include(stmt, dir, mp3Files[i]);
        } 
        catch (Exception e) 
        {
            //e.printStackTrace();
            mbv.showErrorDialog(e);
        }
        return new MusicOutputDesign();
    }

    /**
     *
     * @param stmt
     * @return list
     */
    public Vector<String> playLists(java.sql.Statement stmt) 
    {
        Vector<String> list = new Vector<String>();
        try 
        {
            String sel = "SELECT Name FROM PlayLists ORDER BY Name";

            ResultSet rs = stmt.executeQuery(sel);
            Vector<String> colNames = getColumnNames(rs);
            int leng = colNames.size();
            while (rs.next()) 
            {
                for (int i = 1; i <= leng; i++) 
                {
                    list.add(rs.getString(i));
                }
            }
        } 
        catch (Exception e) 
        {
            mbv.showErrorDialog(e);
            //e.printStackTrace();
        }
        return list;
    }

    /**
     *
     * @param myDir
     * @param dir
     * @param stmt
     */
    public void findAudioFiles(java.io.File myDir, String dir, Statement stmt) 
    {
        String[] dirNames = myDir.list(new DirListFilter());
        for (int i = 0; i < dirNames.length; i++) 
        {
            if (dirNames[i] != null) 
            {
                findAudioFiles(new java.io.File(dir + "/" + dirNames[i]), dir + "/" + dirNames[i], stmt);
            }
        }
        String[] audioFiles = myDir.list(new AudioFileFilter());
        for (int i = 0; i < audioFiles.length; i++) 
        {
            if (audioFiles[i] != null) 
            {
                audio.add(new File(dir + "/" + audioFiles[i]));
            }
        }
    }

    public Connection getConnection() 
    {
        return con;
    }

    public void addPlaylistsToDatabase(final Statement stmt) 
    {
        stillInEDT = false;
        SwingUtilities.invokeLater(new Runnable() 
        {
            public void run() 
            {
                for (int j = 0; j < playlistFiles.size(); j++) 
                {
                    File myFile = playlistFiles.elementAt(j);
                    String name = myFile.getName();
                    int r = JOptionPane.showConfirmDialog(mbv.mainPanel, "Is " + name + " ok as the playlist name?", "Playlist Name", JOptionPane.YES_NO_CANCEL_OPTION);
                    if (r == JOptionPane.CANCEL_OPTION || r == JOptionPane.CLOSED_OPTION) 
                    {
                        continue;
                    }
                    if (r == JOptionPane.NO_OPTION) 
                    {
                        name = JOptionPane.showInputDialog("What would you like to name the playlist?", name);
                    }
                    if (name.contains(" ")) 
                    {
                        name = name.replace(" ", "_");
                    }
                    createTable(stmt, name);
                    addToPlayListTable(name, myFile.toString());
                    try 
                    {
                        BufferedReader fIn = new BufferedReader(new FileReader(myFile));
                        String s;
                        while ((s = fIn.readLine()) != null) 
                        {
                            StringTokenizer line = new StringTokenizer(s);
                            if (line.hasMoreTokens()) 
                            {
                                String current = line.nextToken();
                                if (!current.startsWith("#EXT")) 
                                {
                                    File cur = new File(current).getAbsoluteFile();
                                    String n = cur.getName();
                                    String parent = cur.getParent();
                                    if (parent.compareToIgnoreCase(myFile.getParent()) != 0) 
                                    {
                                        parent = myFile.getParent();
                                    }
                                    //System.out.println("parent: " + parent + "\n name: " + n);
                                    //System.out.println("ID: " + getID(parent, n));
                                    addToPlayList(stmt, name, getID(parent, n));
                                }
                            }
                        }
                        fIn.close();
                    } 
                    catch (FileNotFoundException e) 
                    {
                        //System.out.println("Problem opening file: " + myFile);
                        mbv.showErrorDialog(e);
                    } 
                    catch (IOException e) 
                    {
                        mbv.showErrorDialog(e);
                       // System.out.println("Error reading from: " + myFile);
                    }
                }
                Vector<String> pL = mbv.getAllPlayLists();
                ((DefaultListModel) mbv.jList3.getModel()).clear();
                DefaultListModel dlm2 = new DefaultListModel();
                mbv.jMenu2.removeAll();
                mbv.jMenu3.removeAll();
                for (int i = 0; i < pL.size(); i++) {
                    dlm2.addElement(pL.elementAt(i));
                    final JMenuItem jmi = new JMenuItem(pL.elementAt(i));
                    jmi.addActionListener(new java.awt.event.ActionListener() {

                        @Override
                        public void actionPerformed(ActionEvent evt) {
                            mbv.addSongsToPlaylist(jmi.getText(), mbv.currentlyShowingTable, mbv.allMusic);
                        }
                    });
                    mbv.jMenu2.add(jmi);
                    final JMenuItem jmi1 = new JMenuItem(pL.elementAt(i));
                    jmi1.addActionListener(new java.awt.event.ActionListener() {

                        @Override
                        public void actionPerformed(ActionEvent evt) {
                            //TODO
                            //mbv.addSongsToPlaylist(jmi1.getText(), mbv.currentlyPlayingTable, mbv.currentlyPlaying);
                        }
                    });
                    mbv.jMenu3.add(jmi1);
                }
                pL.clear();
                mbv.jList3.setModel(dlm2);
                playlistFiles.clear();
                stillInEDT = true;
            }
        });
        while (!stillInEDT) 
        {
            System.out.print("");
        }
    }

    public String getID(String parent, String name) 
    {
        String id = "";
        try 
        {
            Statement stmt = connection.createStatement();
            String query = "SELECT ID FROM MUSICLIBRARY WHERE FILEPATH like '" + replaceApostropheAndSlash(parent) + "' AND FILE like '" + replaceApostropheAndSlash(name) + "'";
            ResultSet rs = stmt.executeQuery(query);
            if (rs.next()) 
            {
                id = rs.getString("ID");
            }
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
        return id;
    }

    public Vector<File> getAudioVector() 
    {
        return audio;
    }

    public void getCustomConnection(File f) 
    {
        try 
        {
            Class.forName("org.hsqldb.jdbcDriver");
            customConnection = DriverManager.getConnection("jdbc:hsqldb:file:" + f.getAbsolutePath());
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        } 
        catch (ClassNotFoundException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    public ResultSet selectAllCustom() 
    {
        ResultSet rs = null;
        try 
        {
            Statement stmt = customConnection.createStatement();
            rs = stmt.executeQuery("select * from MusicLibrary ORDER BY artist, album, trackNumber");
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
        return rs;
    }

    public void insertOldIntoNew(ResultSet rs) 
    {
        try {
            while (rs.next()) 
            {
                Statement stmt = con.createStatement();
                this.replaceApostrophe(null);
                stmt.executeUpdate("insert into MusicLibrary (FilePath, File, TrackNumber, TrackTitle,"
                        + " Artist, Album, TrackLength, PlayCount, Rating, ImagePath, ID) VALUES ('" + replaceApostrophe(rs.getString("FilePath")) + "', "
                        + " '" + replaceApostrophe(rs.getString("File")) + "', '" + rs.getInt("TrackNumber") + "', '" + replaceApostrophe(rs.getString("TrackTitle")) + "', "
                        + "'" + replaceApostrophe(rs.getString("Artist")) + "', '" + replaceApostrophe(rs.getString("Album")) + "', '" + replaceApostrophe(rs.getString("TrackLength")) + "', "
                        + "'" + rs.getInt("PlayCount") + "', '" + rs.getInt("Rating") + "', '" + replaceApostrophe(rs.getString("ImagePath")) + "', "
                        + "'" + replaceApostrophe(rs.getString("Artist")) + "-" + replaceApostrophe(rs.getString("Album")) + "-" + replaceApostrophe(rs.getString("TrackNumber")) + "')");
                stmt.close();
            }
            rs.close();
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    public void closeCustomConnection() 
    {
        try 
        {
            customConnection.close();
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }
    
    public void deleteSongFromPlaylists(String id) 
    {
        String selectPlayLists = "Select Name From PLAYLISTS";
        id = replaceApostrophe(id);
        try 
        {
            Statement stmt = getConnection().createStatement();
            ResultSet rs = stmt.executeQuery(selectPlayLists);
            while(rs.next())
            {
                String name = rs.getString("Name");
                String selectID = "SELECT ID FROM " + name + " WHERE ID = '" + id + "'";
                try
                {
                    Statement statem = getConnection().createStatement();
                    ResultSet res = statem.executeQuery(selectID);
                    if(res.next())
                    {
                        try
                        {
                            getConnection().createStatement().executeUpdate("Delete FROM " + name + "Where ID = '" + id + "'");
                        }
                        catch(SQLException sqle)
                        {
                            mbv.showErrorDialog(sqle);
                        }
                    }
                    res.close();
                    statem.close();
                }
                catch(SQLException e)
                {
                    mbv.showErrorDialog(e);
                }
            }
            rs.close();
            stmt.close();
        } 
        catch (SQLException ex) 
        {
            mbv.showErrorDialog(ex);
        }
    }

    /**
     *
     * @param args
     * @throws IOException
     */
    public static void main(String[] args) 
    {
        
    }

    
}
