/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * AlbumArt.java
 *
 * Created on Dec 13, 2009, 8:06:13 PM
 */
package musicbox.gui;

import musicbox.backend.filefilters.ImageFilter;
import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.tag.datatype.Artwork;

/**
 *
 * @author Isaac Hammon
 */
public class AlbumArt extends javax.swing.JDialog 
{

    File f;
    Dimension d;
    private int h;
    private int w;
    private String artist;
    private String album;
    private Vector<String> googleImages;
    private int googleImageLocation = 0;
    private boolean googleSearched = false;
    private String imagePath;
    private MusicBoxView mbv;
    private JScrollPane scroll;

    /** Creates new form AlbumArt
     * @param parent
     * @param f
     */
    public AlbumArt(java.awt.Frame parent, File f, MusicBoxView mbv) 
    {
        super(parent);
        this.mbv = mbv;
        this.f = f;
        googleImages = new Vector<String>();
        googleSearched = false;
        getArtWork();
        initComponents();
        setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
        setIconImage(mbv.musicBoxIcon16.getImage());
        setLocationRelativeTo(parent);
        setVisible(true);
    }

    /**
     *
     * @param parent
     * @param imagePath
     * @param artist
     * @param album
     */
    public AlbumArt(java.awt.Frame parent, String imagePath, String artist, String album, MusicBoxView mbv) 
    {
        super(parent);
        this.imagePath = imagePath;
        this.artist = artist;
        this.album = album;
        this.mbv = mbv;
        googleSearched = false;
        jLabel = new JLabel();
        setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
        ImageIcon iI = new ImageIcon();
        if (imagePath.contains("http://")) 
        {
            try 
            {
                if (mbv.internetConnection) 
                {
                    iI = new ImageIcon(new URL(imagePath));
                }
            } 
            catch (MalformedURLException ex) 
            {
                mbv.showErrorDialog(ex);
            }
        } 
        else 
        {
            iI = new ImageIcon(imagePath);
        }
        Image i = iI.getImage();
        d = new Dimension(iI.getIconWidth(), iI.getIconHeight());
        Image j = i.getScaledInstance(iI.getIconWidth(), iI.getIconHeight(), Image.SCALE_SMOOTH);
        ImageIcon iJ = new ImageIcon(j);
        h = iI.getIconHeight();
        w = iI.getIconWidth();
        if (h < 1 || w < 1) 
        {
            iJ = mbv.noImage;
            h = iJ.getIconHeight();
            w = iJ.getIconWidth();
        }
        
        jLabel.setSize(w, h);
        jLabel.setIcon(iJ);
        if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
        {
            scroll = new JScrollPane();
            scroll.setViewportView(jLabel);
            //scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
            add(scroll, BorderLayout.CENTER);
            Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
            dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
            setPreferredSize(dim);

        }
        else
        {
            add(jLabel, BorderLayout.CENTER);
            setPreferredSize(getMinimumSize());
        }
        initComponents();
        setIconImage(mbv.musicBoxIcon16.getImage());
        setLocation((int)((Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2) - ((double)getWidth()/2)),(int)((Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2) - ((double)getHeight()/2)));
        setVisible(true);
        if(scroll != null)
        {
            getContentPane().remove(scroll);
            getContentPane().remove(jLabel);
            scroll = new JScrollPane();
            scroll.setViewportView(jLabel);
            add(scroll, BorderLayout.CENTER);
            Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
            dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
            setPreferredSize(dim);
            pack();
            setLocation((int)((Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2) - ((double)getWidth()/2)),(int)((Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2) - ((double)getHeight()/2)));
        }
    }

    /**
     *
     * @param parent
     * @param v
     * @param loc
     * @param artist
     * @param album
     */
    @SuppressWarnings({"static-access", "static-access"})
    public AlbumArt(java.awt.Frame parent, Vector<String> v, int loc, String artist, String album, MusicBoxView mbv) 
    {
        super(parent);
        this.artist = artist;
        this.album = album;
        setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
        googleImages = v;
        this.mbv = mbv;
        googleSearched = true;
        googleImageLocation = loc;
        setArtWork();
        initComponents();
        setIconImage(mbv.musicBoxIcon16.getImage());
        setLocationRelativeTo(parent);
        setVisible(true);
    }

    private void setArtWork() 
    {
        if (googleSearched) 
        {
            if (mbv.internetConnection)
            {
                jLabel = new JLabel();
                ImageIcon icon;
                try 
                {
                    icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
                    h = icon.getIconHeight();
                    w = icon.getIconWidth();
                    if (h < 1 || w < 1) 
                    {
                        icon = mbv.noImage;
                        h = icon.getIconHeight();
                        w = icon.getIconWidth();
                    }
                    jLabel.setSize(w, h);
                    jLabel.setIcon(icon);
                    if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
                    {
                        scroll = new JScrollPane();
                        scroll.setViewportView(jLabel);
                        scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                        add(scroll, BorderLayout.CENTER);
                        Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                        dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                        setPreferredSize(dim);
                        
                    }
                    else
                    {
                        add(jLabel, BorderLayout.CENTER);
                        setPreferredSize(getMinimumSize());
                    }
                } 
                catch (MalformedURLException ex1) 
                {
                    //ex1.printStackTrace();
                    mbv.showErrorDialog(ex1);
                }
            }
        }
    }

    private void getArtWork() 
    {
        jLabel = new JLabel();
        boolean foundArt = false;
        if (!f.getName().endsWith(".ape")) 
        {
            try 
            {
                AudioFile file = AudioFileIO.read(f);
                Artwork albumArt = file.getTag().getFirstArtwork();
                artist = file.getTag().getFirstArtist();
                album = file.getTag().getFirstAlbum();
                try 
                {
                    byte[] b = albumArt.getBinaryData();
                    ImageIcon iI = new ImageIcon(b);
                    Image i = iI.getImage();
                    d = new Dimension(iI.getIconWidth(), iI.getIconHeight());
                    Image j = i.getScaledInstance(iI.getIconWidth(), iI.getIconHeight(), Image.SCALE_SMOOTH);
                    ImageIcon iJ = new ImageIcon(j);
                    jLabel.setSize(iI.getIconWidth(), iI.getIconHeight());
                    h = iI.getIconHeight();
                    w = iI.getIconWidth();
                    jLabel.setIcon(iJ);
                    if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
                    {
                        scroll = new JScrollPane();
                        scroll.setViewportView(jLabel);
                        scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                        add(scroll, BorderLayout.CENTER);
                        Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                        dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                        setPreferredSize(dim);
                        
                    }
                    else
                    {
                        add(jLabel, BorderLayout.CENTER);
                        setPreferredSize(getMinimumSize());
                    }
                    foundArt = true;
                    if (h < 0 && w < 0) 
                    {
                        foundArt = false;
                    }
                } 
                catch (Exception e) 
                {
                    if (f.getParentFile() != null) 
                    {
                        File myDir = f.getParentFile();
                        String[] dirNames = myDir.list(new ImageFilter());
                        if (dirNames.length > 0) 
                        {
                            ImageIcon max = new ImageIcon(myDir + "/" + dirNames[0]);
                            for (int p1 = 0; p1 < dirNames.length; p1++) 
                            {
                                ImageIcon iK = new ImageIcon(myDir + "/" + dirNames[p1]);
                                if (iK.getIconHeight() + iK.getIconWidth() > max.getIconHeight() + iK.getIconWidth()) 
                                {
                                    max = iK;
                                }
                            }
                            Image k = max.getImage();
                            d = new Dimension(max.getIconWidth(), max.getIconHeight());
                            h = max.getIconHeight();
                            w = max.getIconWidth();
                            Image l = k.getScaledInstance(max.getIconWidth(), max.getIconHeight(), Image.SCALE_SMOOTH);
                            ImageIcon iL = new ImageIcon(l);
                            jLabel.setSize(iL.getIconWidth(), iL.getIconHeight());
                            jLabel.setIcon(iL);
                            if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
                            {
                                scroll = new JScrollPane();
                                scroll.setViewportView(jLabel);
                                scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                                add(scroll, BorderLayout.CENTER);
                                Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                                dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                                setPreferredSize(dim);

                            }
                            else
                            {
                                add(jLabel, BorderLayout.CENTER);
                                setPreferredSize(getMinimumSize());
                            }
                            foundArt = true;
                        }
                    }
                }
            } 
            catch (Exception p) 
            {
                //p.printStackTrace();
                mbv.showErrorDialog(p);
            }
        }
        if (foundArt == false) 
        {
            if (mbv.internetConnection) 
            {
                googleImageSearch();
                ImageIcon icon;
                try 
                {
                    icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
                    h = icon.getIconHeight();
                    w = icon.getIconWidth();
                    if (h < 1 || w < 1) 
                    {
                        icon = mbv.noImage;
                        h = icon.getIconHeight();
                        w = icon.getIconWidth();
                    }
                    jLabel.setSize(w, h);
                    jLabel.setIcon(icon);
                    if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
                    {
                        scroll = new JScrollPane();
                        scroll.setViewportView(jLabel);
                        scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                        add(scroll, BorderLayout.CENTER);
                        Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                        dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                        setPreferredSize(dim);
                        
                    }
                    else
                    {
                        add(jLabel, BorderLayout.CENTER);
                        setPreferredSize(getMinimumSize());
                    }
                } 
                catch (MalformedURLException ex1) 
                {
                    //ex1.printStackTrace();
                    mbv.showErrorDialog(ex1);
                    icon = mbv.noImage;
                    h = icon.getIconHeight();
                    w = icon.getIconWidth();
                    jLabel.setSize(w, h);
                    jLabel.setIcon(icon);
                    add(jLabel, BorderLayout.CENTER);
                }
            }
        }
    }

    /**
     *
     */
    public void googleImageSearch() 
    {
        googleSearched = true;
        googleImageLocation = 0;
        try 
        {
            String u = "http://images.google.com/images?q=" + artist + " - " + album;
            if (u.contains(" ")) 
            {
                u = u.replace(" ", "+");
            }
            URL url = new URL(u);

            HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
            httpcon.addRequestProperty("User-Agent", "Mozilla/4.76");

            BufferedReader readIn = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));
            googleImages.clear();
            
            String text = "";
            String lin = "";
            while((lin = readIn.readLine()) != null)
            {
                text += lin;
            }
            readIn.close();
            if(text.contains("\n"))
            {
                text = text.replace("\n", "");
            }
            String[] array = text.split("\\Qhref=\"/imgres?imgurl=\\E");
            for(String s: array)
            {
                if(s.startsWith("http://") || s.startsWith("https://") && s.contains("&amp;"))
                {
                    String s1 = s.substring(0, s.indexOf("&amp;"));
                    googleImages.add(s1);
                }
            }
        } 
        catch (Exception ex4) 
        {
            mbv.showErrorDialog(ex4);
            //ex4.printStackTrace();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel = jLabel;
        jPanel2 = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getResourceMap(AlbumArt.class);
        setTitle(resourceMap.getString("Form.title")); // NOI18N
        setName("Form"); // NOI18N

        jLabel.setName("jLabel"); // NOI18N
        jLabel.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                jLabelMouseMoved(evt);
            }
        });
        getContentPane().add(jLabel, java.awt.BorderLayout.CENTER);
        setTitle(artist +  " - " + album);

        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setPreferredSize(new java.awt.Dimension(105, 33));
        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 20, 5));

        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel2.add(jButton1);
        if(googleImageLocation > 0)
        {
            jButton1.setEnabled(true);
        }
        else
        {
            jButton1.setEnabled(false);
        }
        jButton1.setCursor(new Cursor(Cursor.HAND_CURSOR));

        jButton2.setText(resourceMap.getString("jButton2.text")); // NOI18N
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });
        jPanel2.add(jButton2);
        if (googleSearched)
        {
            jButton2.setEnabled(true);
        } else {
            jButton2.setEnabled(false);
        }
        jButton2.setCursor(new Cursor(Cursor.HAND_CURSOR));

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_END);
        if(googleSearched == false)
        {
            getContentPane().remove(jPanel2);
        }
        else
        {
            bottomShowing = true;
        }

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jLabelMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jLabelMouseMoved
        if (bottomShowing == true && ((double) evt.getY() / (double) jLabel.getHeight()) * 100 < 50) 
        {
            bottomShowing = false;
            topShowing = true;
            getContentPane().remove(jPanel2);
            getContentPane().add(jPanel2, BorderLayout.PAGE_START);
            pack();
        } 
        else if (topShowing == true && ((double) evt.getY() / (double) jLabel.getHeight()) * 100 >= 50) 
        {
            bottomShowing = true;
            topShowing = false;
            getContentPane().remove(jPanel2);
            getContentPane().add(jPanel2, BorderLayout.PAGE_END);
            pack();
        }
    }//GEN-LAST:event_jLabelMouseMoved

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        googleImageLocation += 1;
        if(scroll != null && scroll.isVisible())
        {
            getContentPane().remove(scroll);
        }
        getContentPane().remove(jLabel);
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
            h = icon.getIconHeight();
            w = icon.getIconWidth();
            if (h < 1 || w < 1) 
            {
                icon = mbv.noImage;
                h = icon.getIconHeight();
                w = icon.getIconWidth();
            }
            jLabel.setSize(w, h);
            jLabel.setIcon(icon);
            if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
            {
                scroll = new JScrollPane();
                scroll.setViewportView(jLabel);
                scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                add(scroll, BorderLayout.CENTER);
                Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                setPreferredSize(dim);
            }
            else
            {
                add(jLabel, BorderLayout.CENTER);
                setPreferredSize(getMinimumSize());
            }
            pack();
            setLocation((int)((Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2) - ((double)getWidth()/2)),(int)((Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2) - ((double)getHeight()/2)));
        } 
        catch (MalformedURLException ex1) 
        {
            //ex1.printStackTrace();
            mbv.showErrorDialog(ex1);
        }
        if (googleImageLocation == 0) 
        {
            jButton1.setEnabled(false);
        } 
        else 
        {
            jButton1.setEnabled(true);
        }
        if(googleImageLocation < googleImages.size() - 1)
        {
            jButton2.setEnabled(true);
        } 
        else 
        {
            jButton2.setEnabled(false);
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        googleImageLocation -= 1;
                ImageIcon icon;
                getContentPane().remove(jLabel);
                try 
                {
                    icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
                    h = icon.getIconHeight();
                    w = icon.getIconWidth();
                    if (h < 1 || w < 1) 
                    {
                        icon = mbv.noImage;
                        h = icon.getIconHeight();
                        w = icon.getIconWidth();
                    }
                    jLabel.setSize(w, h);
                    jLabel.setIcon(icon);
                    if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
                    {
                        scroll = new JScrollPane();
                        scroll.setViewportView(jLabel);
                        scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                        add(scroll, BorderLayout.CENTER);
                        Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                        dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                        setPreferredSize(dim);
                    }
                    else
                    {
                        add(jLabel, BorderLayout.CENTER);
                        setPreferredSize(getMinimumSize());
                    }
                    pack();
                    setLocation((int)((Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2) - ((double)getWidth()/2)),(int)((Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2) - ((double)getHeight()/2)));
                } 
                catch (MalformedURLException ex) 
                {
                    //ex1.printStackTrace();
                    mbv.showErrorDialog(ex);
                }
                if (googleImageLocation == 0) 
                {
                    jButton1.setEnabled(false);
                } 
                else 
                {
                    jButton1.setEnabled(true);
                }
                if (googleImageLocation < googleImages.size() - 1) 
                {
                    jButton2.setEnabled(true);
                } 
                else 
                {
                    jButton2.setEnabled(false);
                }
    }//GEN-LAST:event_jButton1ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JLabel jLabel;
    private javax.swing.JPanel jPanel2;
    // End of variables declaration//GEN-END:variables
    boolean bottomShowing = false;
    boolean topShowing = false;
}
