/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package musicbox.gui;

import java.awt.Cursor;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.sql.Connection;
import java.sql.DriverManager;
import java.util.Collections;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JTable;
import musicbox.backend.comparators.IDComparator;
import noTalent.MusicOutputDesign;

/**
 *
 * @author Isaac Hammon
 */
public class CustomTableListener implements MouseListener, KeyListener, MouseMotionListener 
{

    private int previousY = -1;
    public MusicBoxView mbv;

    public CustomTableListener() 
    {
    }

    @Override
    public void mouseClicked(MouseEvent e) 
    {
        if (e.getSource() instanceof JTable) 
        {
            JTable jT = (JTable) e.getSource();
            CustomTableModel ctm = (CustomTableModel) jT.getModel();
            Vector<MusicOutputDesign> current = mbv.allMusic;
            String tableName = jT.getName();
            if (e.getButton() == MouseEvent.BUTTON3) 
            {
                if (tableName.equals("CurrentlyShowingTable")) 
                {
                    mbv.ShowingPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                } 
                else if (tableName.equals("CurrentlyPlayingTable")) 
                {
                    mbv.PlayingPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                }
                else if(tableName.equals("sharedLibraryTable"))
                {
                    mbv.SharedLibraryPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                }
                else 
                {
                    mbv.PlaylistPopupMenu.show(e.getComponent(), e.getX(), e.getY());
                }
            } 
            else if (e.getButton() == MouseEvent.BUTTON1) 
            {
                int col = jT.getTableHeader().columnAtPoint(e.getPoint());
                if (col == 6) 
                {
                    double theRest = (double) jT.getWidth() - (double) jT.getColumn("Rating").getWidth();
                    double place = (double) e.getX() - theRest;
                    double x = place / (double) jT.getColumn("Rating").getWidth();
                    int y = jT.rowAtPoint(e.getPoint());
                    MusicOutputDesign mod = new MusicOutputDesign();
                    if (jT.getName().equals("CurrentlyShowingTable")) 
                    {
                        //current = mbv.allMusic;
                        mod = mbv.allMusic.elementAt(jT.convertRowIndexToModel(y));
                    } 
                    else if (jT.getName().equals("CurrentlyPlayingTable")) {
                        //current = mbv.currentlyPlaying;
                        mod = mbv.back.findTrack(mbv.currentlyPlaying2.elementAt(jT.convertRowIndexToModel(y)));//mbv.allMusic.elementAt(mbv.currentlyPlaying.elementAt(jT.convertRowIndexToModel(y)));
                    } 
                    else 
                    {
                        //current = mbv.playList;
                        mod = mbv.back.findTrack(mbv.playList.elementAt(jT.convertRowIndexToModel(y)));
                    }
                    boolean sameTrack = false;
                    if (y < current.size()) 
                    {
                        if (mod.equals(mbv.back.currentTrack)) 
                        {
                            sameTrack = true;
                        }
                        if (y < 0) {
                            y = 0;
                        }

                        if (x > 0 && x <= 0.2) {
                            ctm.setValueAt(mbv.oneSmile, y, jT.getColumnModel().getColumnIndex("Rating"));
                            mbv.back.updateRating(mod, 1, "");
                            if (sameTrack) {
                                mbv.ratingLabel.setIcon(mbv.oneSmile);
                            }
                        } else if (x > 0.2 && x <= 0.4) {
                            ctm.setValueAt(mbv.twoSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                            mbv.back.updateRating(mod, 2, "");
                            if (sameTrack) {
                                mbv.ratingLabel.setIcon(mbv.twoSmiles);
                            }
                        } else if (x > 0.4 && x <= 0.6) {
                            ctm.setValueAt(mbv.threeSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                            mbv.back.updateRating(mod, 3, "");
                            if (sameTrack) {
                                mbv.ratingLabel.setIcon(mbv.threeSmiles);
                            }
                        } else if (x > 0.6 && x <= 0.8) {
                            ctm.setValueAt(mbv.fourSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                            mbv.back.updateRating(mod, 4, "");
                            if (sameTrack) {
                                mbv.ratingLabel.setIcon(mbv.fourSmiles);
                            }
                        } else if (x > 0.8 && x < 1.0) {
                            ctm.setValueAt(mbv.fiveSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                            mbv.back.updateRating(mod, 5, "");
                            if (sameTrack) {
                                mbv.ratingLabel.setIcon(mbv.fiveSmiles);
                            }
                        }
                    }

                }
            }
        }
    }

    @Override
    public void mousePressed(MouseEvent e) 
    {
        JTable jT = (JTable) e.getSource();
        if (e.getClickCount() == 2) 
        {
            if (jT.getSelectedRowCount() == 1) 
            {
                if (!jT.getName().equals("CurrentlyPlayingTable")) 
                {
                    int location = jT.getSelectedRow();
                    location = jT.convertRowIndexToModel(location);
                    int locations[];
                    String id = "";
                    MusicOutputDesign mod = null;
                    if (jT.getName().equalsIgnoreCase("playlistTable")) 
                    {
                        id = mbv.playList.elementAt(location);
                        locations = new int[]{location};
                        mod = mbv.back.findTrack(mbv.playList.elementAt(location));
                    }
                    else if(jT.getName().equals("sharedLibraryTable"))
                    {
                        id = mbv.sharedLibrary.elementAt(location).getId();
                        locations = new int[]{location};
                        mod = mbv.sharedLibrary.elementAt(location);
                    }
                    else
                    {
                        id = mbv.allMusic.elementAt(location).getId();
                        locations = new int[]{location};
                        mod = mbv.allMusic.elementAt(location);
                    }
                    mbv.currentlyPlaying2.add(id);
                    //mbv.populateTableModel(mbv.currentlyPlayingTable, locations);
                    CustomTableModel ctm = ((CustomTableModel)mbv.currentlyPlayingTable.getModel());
                    if(ctm.getColumnCount() == 0)
                    {
                        for(String column: mbv.columnNames)
                        {
                            ctm.addColumn(column);
                        }
                    }
                    ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), mbv.smileys[Integer.parseInt(mod.getRating())]});
                    //mbv.currentlyPlayingTable.setModel(ctm);
                    //mbv.currentlyPlayingTable.doLayout();
                    //mbv.updateTable(mbv.currentlyPlaying, mbv.currentlyPlayingTable, "");
                    mbv.back.calculatePlaylistTime();
                    if (mbv.playing == false) 
                    {
                        mbv.playButton.doClick();
                    } 
                    else if (mbv.playing == true) 
                    {
                        mbv.stop();
                        mbv.back.currentLocation = mbv.currentlyPlaying2.size() - 1;
                        mbv.playButton.doClick();
                    }
                } 
                else 
                {
                    int location = jT.getSelectedRow();
                    mbv.back.currentLocation = location;
                    if (mbv.playing == false) 
                    {
                        mbv.playButton.doClick();
                    } 
                    else if (mbv.playing == true) 
                    {
                        mbv.stop();
                        //mbv.playButton.doClick();
                        mbv.play();
                    }
                }
            }
        }
        else if(e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1)
        {
            
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
    }

    @Override
    public void mouseEntered(MouseEvent e) 
    {
        JTable jT = (JTable) e.getSource();
        DefaultListSelectionModel dlsm = (DefaultListSelectionModel)jT.getSelectionModel();
        jT.setSelectionModel(dlsm);
    }
    //TODO: Fix this method

    @Override
    public void mouseExited(MouseEvent e) 
    {
        JTable jT = (JTable) e.getSource();
        DefaultListSelectionModel dlsm = (DefaultListSelectionModel)jT.getSelectionModel();
        dlsm.setValueIsAdjusting(false);
        jT.setSelectionModel(dlsm);
        MusicOutputDesign mod;
        if (previousY != -1 && previousY < jT.getRowCount()) 
        {
            previousY = jT.convertRowIndexToModel(previousY);
            if (jT.getName().equals("CurrentlyShowingTable")) 
            {
                mod = mbv.allMusic.elementAt(previousY);
            } 
            else if (jT.getName().equals("CurrentlyPlayingTable")) 
            {
                if(!mbv.currentlyPlaying2.elementAt(previousY).startsWith("sharedLibraryTable"))
                {
                    mod = mbv.back.findTrack(mbv.currentlyPlaying2.elementAt(previousY));
                }
                else
                {
                    String id = mbv.currentlyPlaying2.elementAt(previousY);
                    if(id.indexOf("sharedLibraryTable-") > -1)
                    {
                        id = id.replace("sharedLibraryTable-", "");
                    }
                    mod = mbv.back.findTrack(mbv.sharedLibrary,id);
                }
            } 
            else 
            {
                mod = mbv.back.findTrack(mbv.playList.elementAt(previousY));
            }
            if (previousY != -1) 
            {
                mbv.updateTableAtLocation(mod, previousY, jT);
            }
        }
        previousY = -1;
    }

    @Override
    public void keyTyped(KeyEvent e) 
    {
    }

    @Override
    public void keyPressed(KeyEvent e) 
    {
        JTable jT = (JTable) e.getSource();
        Vector<MusicOutputDesign> current = mbv.allMusic;
        String tableName = jT.getName();
        if (e.getKeyCode() == KeyEvent.VK_ENTER) 
        {
            if (jT.getSelectedRowCount() >= 1) 
            {
                if (!tableName.equals("CurrentlyPlayingTable")) 
                {
                    int[] location = jT.getSelectedRows();
                    for (int i = 0; i < location.length; i++) 
                    {
                        location[i] = jT.convertRowIndexToModel(location[i]);
                        /*
                         * Do nothing for now...
                         */
                        /*if (jT.getName().equalsIgnoreCase("playlistTable")) 
                        {
                            location[i] = mbv.playList.elementAt(location[i]);
                        }*/
                    }
                    mbv.back.currentLocation = mbv.currentlyPlaying2.size();
                    CustomTableModel ctm = (CustomTableModel)mbv.currentlyPlayingTable.getModel();
                    if(ctm.getColumnCount() == 0)
                    {
                        for(String column: mbv.columnNames)
                        {
                            ctm.addColumn(column);
                        }
                    }
                    Vector<MusicOutputDesign> temp1 = (Vector<MusicOutputDesign>)mbv.allMusic.clone();
                    Collections.sort(temp1, new IDComparator());
                    for (int i = 0; i < location.length; i++) 
                    {
                        MusicOutputDesign mod = null;
                        if(jT.getName().equalsIgnoreCase("currentlyShowingTable"))
                        {
                            mod = mbv.allMusic.elementAt(location[i]);
                        }
                        else if(jT.getName().equalsIgnoreCase("playlistTable"))
                        {
                            mod = mbv.back.findTrack(temp1, mbv.playList.elementAt(location[i]));
                        }
                        else
                        {
                            mod = mbv.sharedLibrary.elementAt(location[i]);
                        }
                        mbv.currentlyPlaying2.add(mod.getId());
                        ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), mbv.smileys[Integer.parseInt(mod.getRating())]});
                    }
                    temp1.clear();
                    temp1 = null;
                    //mbv.populateTableModel(mbv.currentlyPlayingTable, location);
                    //mbv.updateTable(mbv.currentlyPlaying, mbv.currentlyPlayingTable, "");
                    mbv.back.calculatePlaylistTime();
                    if (mbv.playing == false) 
                    {
                        mbv.playButton.doClick();
                    } 
                    else if (mbv.playing == true) 
                    {
                        mbv.stop();
                        mbv.playButton.doClick();
                    }
                } 
                else 
                {
                    int[] location = jT.getSelectedRows();
                    for (int i = 0; i < location.length; i++) 
                    {
                        location[i] = jT.convertRowIndexToModel(location[i]);
                    }
                    mbv.back.currentLocation = location[0];
                    if (mbv.playing == false) 
                    {
                        mbv.playButton.doClick();
                    } 
                    else if (mbv.playing == true) 
                    {
                        mbv.stop();
                        mbv.playButton.doClick();
                    }
                }
            }
        } 
        else if (e.getKeyCode() == KeyEvent.VK_DELETE) 
        {
            if (jT.getName().equals("CurrentlyShowingTable")) 
            {
                try 
                {
                    String db_file_name_prefix = "resources/MDB.odb";
                    Class.forName("org.hsqldb.jdbcDriver");
                    Connection con = DriverManager.getConnection("jdbc:hsqldb:file:" + db_file_name_prefix);
                    java.sql.Statement stmt = con.createStatement();
                    int[] location = jT.getSelectedRows();
                    for (int i = 0; i < location.length; i++) 
                    {
                        location[i] = jT.convertRowIndexToModel(location[i]);
                    }
                    MusicOutputDesign[] delete = new MusicOutputDesign[location.length];
                    for (int i = 0; i < location.length; i++) 
                    {
                        delete[i] = current.elementAt(location[i]);
                    }
                    for (int jl = 0; jl < delete.length; jl++) 
                    {
                        mbv.db.deleteFromMusicLibrary(stmt, delete[jl]);
                    }
                    jT.clearSelection();
                    mbv.getAllMusic();
                    mbv.populateTableModel((CustomTableModel) jT.getModel());
                    //mbv.updateCurrentlyShowing("");

                    //mbv.updateTable(current, jT, "");

                    DefaultListModel dlm = new DefaultListModel();
                    dlm.addElement("(All Artists)");
                    String previousArtist = "awesome";
                    for (int i = 0; i < mbv.allMusic.size(); i++) 
                    {
                        if (i - 1 >= 0) 
                        {
                            previousArtist = mbv.allMusic.elementAt(i - 1).getArtist();
                        }
                        if (previousArtist.compareToIgnoreCase(mbv.allMusic.elementAt(i).getArtist()) != 0) 
                        {
                            dlm.addElement(mbv.allMusic.elementAt(i).getArtist());
                        }
                    }
                    dlm = mbv.sortListModel(dlm);
                    mbv.jList1.setModel(dlm);

                    DefaultListModel dlm1 = new DefaultListModel();
                    dlm1.addElement("(All Albums)");
                    String previousAlbum = "awesome";
                    for (int i = 0; i < mbv.allMusic.size(); i++) 
                    {
                        if (i - 1 >= 0) 
                        {
                            previousAlbum = mbv.allMusic.elementAt(i - 1).getAlbum();
                        }
                        if (previousAlbum.compareToIgnoreCase(mbv.allMusic.elementAt(i).getAlbum()) != 0) 
                        {
                            dlm1.addElement(mbv.allMusic.elementAt(i).getAlbum());
                        }
                    }
                    dlm1 = mbv.sortListModel(dlm1);
                    mbv.jList2.setModel(dlm1);
                } 
                catch (Exception f) 
                {//f.printStackTrace();
                    mbv.showErrorDialog(f);
                }
            } 
            else if (jT.getName().equals("CurrentlyPlayingTable")) 
            {
                mbv.removeFromCurrentlyPlaying();
            }
        }
    }

    @Override
    public void keyReleased(KeyEvent e) 
    {
    }

    @Override
    public void mouseDragged(MouseEvent e) 
    {
    }

    @Override
    public void mouseMoved(MouseEvent e) 
    {
        JTable jT = (JTable) e.getSource();
        DefaultListSelectionModel dlsm = (DefaultListSelectionModel)jT.getSelectionModel();
        CustomTableModel ctm = (CustomTableModel) jT.getModel();
        Vector<MusicOutputDesign> current = mbv.allMusic;
        MusicOutputDesign previousMOD = new MusicOutputDesign();
        if (jT.getName().equals("CurrentlyShowingTable")) 
        {
            if (previousY != -1) 
            {
                previousMOD = mbv.allMusic.elementAt(previousY/*jT.convertRowIndexToView(previousY)*/);
            }
        }
        else if (jT.getName().equals("CurrentlyPlayingTable")) 
        {
            if (previousY != -1) 
            {
                if(!mbv.currentlyPlaying2.elementAt(previousY).startsWith("sharedLibraryTable"))
                {
                    previousMOD = mbv.back.findTrack(mbv.currentlyPlaying2.elementAt(previousY));
                }
                else
                {
                    previousMOD = mbv.back.findTrack(mbv.sharedLibrary, mbv.currentlyPlaying2.elementAt(previousY).replace("sharedLibraryTable-", ""));
                }
            }
        }
        else if(jT.getName().equals("sharedLibraryTable"))
        {
            return;
        }
        else 
        {
            if (previousY != -1) 
            {
                previousMOD = mbv.back.findTrack(mbv.playList.elementAt(previousY));
            }
        }
        int col = jT.convertColumnIndexToView(jT.getTableHeader().columnAtPoint(e.getPoint()));
        if (col == jT.convertColumnIndexToView(jT.getColumnModel().getColumnIndex("Rating"))) 
        {
            jT.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            double t = e.getX();
            while (t > 0) 
            {
                t = t - (double) jT.getColumn("Rating").getWidth();
            }
            double theRest = e.getX() + t;
            double place = ((double) e.getX() - theRest);
            double x = 1 - (place / (double) jT.getColumn("Rating").getWidth());
            int y = jT.rowAtPoint(e.getPoint());
            MusicOutputDesign mod = new MusicOutputDesign();
            if (y == -1) 
            {
                if (previousY != -1) 
                {
                    if (previousY < current.size()) 
                    {
                        mbv.updateTableAtLocation(previousMOD, previousY, jT);
                        jT.setSelectionModel(dlsm);
                    }
                }
                previousY = -1;
                return;
            }
            if (jT.getName().equals("CurrentlyShowingTable")) 
            {
                mod = mbv.allMusic.elementAt(jT.convertRowIndexToModel(y));
            } 
            else if (jT.getName().equals("CurrentlyPlayingTable")) 
            {
                if(mbv.currentlyPlaying2.elementAt(jT.convertRowIndexToModel(y)).startsWith("sharedLibraryTable"))
                {
                    return;
                }
                mod = mbv.back.findTrack(mbv.currentlyPlaying2.elementAt(jT.convertRowIndexToModel(y)));
            } 
            else 
            {
                mod = mbv.back.findTrack(mbv.playList.elementAt(jT.convertRowIndexToModel(y)));
            }
            if (y < 0) 
            {
                y = 0;
            }
            y = jT.convertRowIndexToModel(y);
            if (previousY != y) 
            {
                if (previousY != -1) 
                {
                    if (previousY < current.size()) 
                    {
                        mbv.updateTableAtLocation(previousMOD, previousY, jT);
                        jT.setSelectionModel(dlsm);
                    }
                }
                previousY = y;
            }
            boolean sameTrack = false;
            if (y < current.size()) 
            {
                if (mod.equals(mbv.back.currentTrack)) 
                {
                    sameTrack = true;
                }
                if (x >= 0 && x <= 0.2) 
                {
                    ctm.setValueAt(mbv.oneSmile, y, jT.getColumnModel().getColumnIndex("Rating"));
                    jT.setSelectionModel(dlsm);
                    if (sameTrack) 
                    {
                        mbv.ratingLabel.setIcon(mbv.oneSmile);
                    }
                } 
                else if (x > 0.2 && x <= 0.4) 
                {
                    ctm.setValueAt(mbv.twoSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                    jT.setSelectionModel(dlsm);
                    if (sameTrack) 
                    {
                        mbv.ratingLabel.setIcon(mbv.twoSmiles);
                    }
                } 
                else if (x > 0.4 && x <= 0.6) 
                {
                    ctm.setValueAt(mbv.threeSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                    jT.setSelectionModel(dlsm);
                    if (sameTrack) 
                    {
                        mbv.ratingLabel.setIcon(mbv.threeSmiles);
                    }
                } 
                else if (x > 0.6 && x <= 0.8) 
                {
                    ctm.setValueAt(mbv.fourSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                    jT.setSelectionModel(dlsm);
                    if (sameTrack) 
                    {
                        mbv.ratingLabel.setIcon(mbv.fourSmiles);
                    }
                } 
                else if (x > 0.8 && x < 1.0) 
                {
                    ctm.setValueAt(mbv.fiveSmiles, y, jT.getColumnModel().getColumnIndex("Rating"));
                    jT.setSelectionModel(dlsm);
                    if (sameTrack) 
                    {
                        mbv.ratingLabel.setIcon(mbv.fiveSmiles);
                    }
                }
            }
        } 
        else 
        {
            jT.setCursor(Cursor.getDefaultCursor());
            if (previousY != -1) 
            {
                if (previousY < current.size()) 
                {
                    jT.setSelectionModel(dlsm);
                    mbv.updateTableAtLocation(previousMOD, previousY, jT);
                    jT.setSelectionModel(dlsm);
                }
            }
            previousY = -1;
        }
    }
    public void setMusicBoxView(MusicBoxView mbv)
    {
        this.mbv = mbv;
    }
}
