/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * FileChooser.java
 *
 * Created on Oct 11, 2009, 7:32:02 PM
 */
package musicbox.gui;

import java.io.File;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Timer;
import java.util.TimerTask;
import javax.swing.DefaultListModel;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileNameExtensionFilter;
import noTalent.MusicOutputDesign;

/**
 *
 * @author Isaac Hammon
 */
public class FileChooser extends javax.swing.JDialog 
{

    /**
     *
     */
    protected boolean wait;
    private MusicBoxView mbv;

    /** Creates new form FileChooser
     * @param parent
     */
    public FileChooser(java.awt.Frame parent,MusicBoxView mbv) 
    {
        super(parent);
        this.mbv = mbv;
        initComponents();
        this.setIconImage(mbv.musicBoxIcon16.getImage());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     * @param evt
     */
    public void importMusic(java.awt.event.ActionEvent evt) 
    {
        String action = evt.getActionCommand();
        if (action.compareTo("ApproveSelection") == 0) 
        {
            try {
                wait = true;
                File[] files = jFileChooser1.getSelectedFiles();
                closeFileChooser();
                Import im = new Import(files,mbv);
                im.start();

            } 
            catch (Exception e) {//e.printStackTrace();
                mbv.showErrorDialog(e);
            }
        } 
        else 
        {
            closeFileChooser();
        }
    }

    /**
     *
     */
    public void closeFileChooser() 
    {
        dispose();
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jFileChooser1 = new javax.swing.JFileChooser();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N
        setResizable(false);

        jFileChooser1.setAcceptAllFileFilterUsed(false);
        jFileChooser1.setMultiSelectionEnabled(true);
        jFileChooser1.setName("jFileChooser1"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jFileChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jFileChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        FileNameExtensionFilter filter = new FileNameExtensionFilter("audio file", "mp3", "wav", "au", "aif", "ogg", "ape", "flac", "m3u");
        jFileChooser1.setFileFilter(filter);
        jFileChooser1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                importMusic(evt);
            }
        });

        pack();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JFileChooser jFileChooser1;
    // End of variables declaration//GEN-END:variables
}

class Import extends Thread 
{
    File[] files;
    private boolean wait;
    private AddFiles addFiles;
    private MusicBoxView mbv;

    public Import(File[] files, MusicBoxView mbv) 
    {
        this.files = files;
        this.mbv = mbv;
    }

    @Override
    @SuppressWarnings({"empty-statement", "empty-statement", "empty-statement"})
    public void run() 
    {
        try 
        {
            mbv.busyIconTimer.start();
            final Statement stmt = mbv.db.getConnection().createStatement();
            wait = true;
            SwingUtilities.invokeLater(new Runnable() 
            {
                public void run() 
                {
                    addFiles = new AddFiles(files.length,mbv);
                    addFiles.setLocationRelativeTo(null);
                    addFiles.setVisible(true);
                    wait = false;
                }
            });
            while(wait)
            {
                System.out.print("");
            }
            for (int i = 0; i < files.length; i++) 
            {
                MusicOutputDesign mod = mbv.db.insertMusicFile(files[i], stmt);
                mbv.allMusic.add(mod);
                if (((CustomTableModel)mbv.currentlyShowingTable.getModel()).getColumnCount() == 0) 
                {
                    for (String columnName: mbv.columnNames) 
                    {
                        ((CustomTableModel)mbv.currentlyShowingTable.getModel()).addColumn(columnName);
                    }
                }
                
                ((CustomTableModel)mbv.currentlyShowingTable.getModel()).addRow(mod, mbv);
                if (addFiles != null && addFiles.isVisible()) 
                {
                    addFiles.getJTextArea().setText("Adding file: " + files[i]);
                    wait = true;
                    SwingUtilities.invokeLater(new Runnable()
                    {
                       public void run()
                       {
                           addFiles.getProgressBar().setValue(addFiles.getProgressBar().getValue() + 1);
                           wait = false;
                       }
                    });
                    while(wait)
                    {
                        System.out.print("");
                    }
                }
            }
            if(addFiles != null && addFiles.isVisible())
            {
                addFiles.getJButton().setText("Close");
            }
            mbv.db.addPlaylistsToDatabase(stmt);
            stmt.close();
            
            if(SwingUtilities.isEventDispatchThread())
            {
                ((CustomTableModel)mbv.currentlyShowingTable.getModel()).fireTableDataChanged();
                updateLists();
            }
            else
            {
                SwingUtilities.invokeLater(new Thread()
                {
                    public void run()
                    {
                        ((CustomTableModel)mbv.currentlyShowingTable.getModel()).fireTableDataChanged();
                        updateLists();
                    }
                });
            }
            
            mbv.busyIconTimer.stop();
        } 
        catch (Exception e) 
        {//e.printStackTrace();
            mbv.showErrorDialog(e);
        }
        while (wait)
        {
            System.out.print("");
        }
        new Timer().schedule(new SetLibraryImages(), 100);
    }
    
    public void updateLists()
    {
        DefaultListModel dlm = new DefaultListModel();
            dlm.addElement("(All Artists)");
            try
            {
                Object[] array = mbv.db.selectDistinct(mbv.db.getConnection().createStatement(), "artist");
                for(Object o: array)
                {
                    dlm.addElement(o);
                }
            }
            catch(SQLException ex)
            {
                mbv.showErrorDialog(ex);
            }
            dlm = mbv.sortListModel(dlm);
            mbv.jList1.setModel(dlm);

            DefaultListModel dlm1 = new DefaultListModel();
            dlm1.addElement("(All Albums)");
            try
            {
                Object[] array = mbv.db.selectDistinct(mbv.db.getConnection().createStatement(), "album");
                for(Object o: array)
                {
                    dlm1.addElement(o);
                }
            }
            catch(SQLException ex)
            {
                mbv.showErrorDialog(ex);
            }
            dlm1 = mbv.sortListModel(dlm1);
            mbv.jList2.setModel(dlm1);
    }

    class SetLibraryImages extends TimerTask 
    {
        @Override
        public void run() 
        {
            mbv.createAllLibraryArtistPanels();
            mbv.createAllLibraryAlbumPanels();
            try 
            {
                java.sql.Statement stmt = mbv.db.getConnection().createStatement();
                mbv.setAllLibraryArtistImages(stmt);
                mbv.setAllLibraryAlbumImages(stmt);
                stmt.close();
            }
            catch (SQLException ex) 
            {
                //ex.printStackTrace();
                mbv.showErrorDialog(ex);
            }
        }
    }
}
