/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR v PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * MusicBoxView.java
 */
package musicbox.gui;

import java.lang.reflect.InvocationTargetException;
import java.net.UnknownHostException;
import musicbox.backend.Backend;
import musicbox.backend.comparators.ArtistComparator;
import musicbox.backend.filefilters.ImageFilter;
import musicbox.database.Database;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.SplashScreen;
import java.awt.Toolkit;
import javax.swing.event.HyperlinkListener;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.*;
import javax.swing.SwingWorker;
import java.net.URL;
import java.util.Vector;
import java.sql.*;
import java.io.*;

import java.net.MalformedURLException;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.LogManager;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.CompoundControl;
import javax.sound.sampled.Control;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.Port;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.TableRowSorter;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;
import musicbox.backend.FindFileRunnable;
import musicbox.backend.MusicBoxApp;
import noTalent.MusicOutputDesign;
import musicbox.backend.SocketListener;
import musicbox.backend.SocketWriteThread;
import noTalent.SocketObject;
import musicbox.backend.comparators.AlbumStringComparator;
import musicbox.backend.comparators.ArtistStringComparator;
import musicbox.backend.comparators.IDComparator;
import musicbox.backend.comparators.PlayCountStringComparator;
import musicbox.backend.comparators.TrackNumberStringComparator;
import org.jaudiotagger.audio.AudioFile;
import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.tag.datatype.Artwork;
import org.jdesktop.application.LocalStorage;

/**
 * The application's main frame.
 *
 * @author Isaac Hammon
 */
public class MusicBoxView extends FrameView implements ActionListener, PropertyChangeListener, HyperlinkListener {

    /**
     * Constructor
     * @param app
     */
    @SuppressWarnings("static-access")
    public MusicBoxView(SingleFrameApplication app) {
        super(app);
        try 
        {
            LogManager.getLogManager().readConfiguration(new StringBufferInputStream("org.jaudiotagger.level = OFF"));
        }
        catch (IOException ex) 
        {
            //ex.printStackTrace();
            showErrorDialog(ex);
        } 
        catch (SecurityException ex) 
        {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
        Splash s = new Splash();
        s.start();
        ResourceMap resourceMap = getResourceMap();
        zeroSmiles = resourceMap.getImageIcon("zeroSmiles");
        oneSmile = resourceMap.getImageIcon("oneSmile");
        twoSmiles = resourceMap.getImageIcon("twoSmiles");
        threeSmiles = resourceMap.getImageIcon("threeSmiles");
        fourSmiles = resourceMap.getImageIcon("fourSmiles");
        fiveSmiles = resourceMap.getImageIcon("fiveSmiles");
        noImage = resourceMap.getImageIcon("noImage");
        musicBoxIcon16 = resourceMap.getImageIcon("musicBoxIcon16");
        getFrame().setIconImage(musicBoxIcon16.getImage());
        smileys = new ImageIcon[]{zeroSmiles, oneSmile, twoSmiles, threeSmiles, fourSmiles, fiveSmiles};
        ctl = new CustomTableListener();
        cbh = new CustomButtonHandler(this);
        task = new Task();
        p = this;
        db = new Database(p);
        back = new Backend(0);
        back.setMusicBoxView(p);
        ctl.setMusicBoxView(p);
        getAllMusic();
        ls = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getLocalStorage();
        ls.setDirectory(new File("resources/session"));
        try 
        {
            projectFont = (Font) ls.load("preferences.xml");
            fontColor = (Color) ls.load("fontcolor.xml");
            internetConnection = (Boolean) ls.load("internetconnection.xml");
            errorMessage = (Boolean) ls.load("errormessages.xml");
            progressBarColor = (Color) ls.load("progressbarcolor.xml");
            laf = (LookAndFeel) ls.load("lookandfeel.xml");
            serverAddress = (String) ls.load("serveraddress.xml");
            java.awt.EventQueue.invokeLater(new Runnable() 
            {

                @Override
                public void run() 
                {
                    try 
                    {
                        JDialog.setDefaultLookAndFeelDecorated(true);
                        JFrame.setDefaultLookAndFeelDecorated(true);
                        MusicBoxApp.getApplication().getMainFrame().setDefaultLookAndFeelDecorated(true);
                        UIManager.setLookAndFeel(laf);
                        SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                        SwingUtilities.updateComponentTreeUI(PlayingPopupMenu);
                        SwingUtilities.updateComponentTreeUI(ShowingPopupMenu);
                        SwingUtilities.updateComponentTreeUI(jPopupMenu3);
                        SwingUtilities.updateComponentTreeUI(PlaylistPopupMenu);
                        if (laf.getSupportsWindowDecorations()) 
                        {
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.dispose();
                            mainFrame.setUndecorated(true);
                            mainFrame.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
                            mainFrame.setVisible(true);
                            playlistTable.setRowHeight(16);
                            currentlyPlayingTable.setRowHeight(16);
                            currentlyShowingTable.setRowHeight(16);
                        } 
                        else 
                        {
                            JDialog.setDefaultLookAndFeelDecorated(false);
                            JFrame.setDefaultLookAndFeelDecorated(false);
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.dispose();
                            mainFrame.setUndecorated(false);
                            mainFrame.setVisible(true);
                            playlistTable.setRowHeight(16);
                            currentlyPlayingTable.setRowHeight(16);
                            currentlyShowingTable.setRowHeight(16);
                        }
                    } 
                    catch (UnsupportedLookAndFeelException ex) 
                    {
                        showErrorDialog(ex);
                    }
                }
            });
            UIManager.put("nimbusOrange", progressBarColor);
        } 
        catch (IOException ex) 
        {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
        cma = new CustomMouseAdapter(p);
        initComponents();
        
        playlistTable.setRowHeight(16);
        currentlyPlayingTable.setRowHeight(16);
        currentlyShowingTable.setRowHeight(16);
        jProgressBar1.setBackground(progressBarColor);
        //jProgressBar1.setForeground(progressBarColor);
        setFont();
        setFontColor();
        // status bar initialization - message timeout, idle icon and busy animation, etc

        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new javax.swing.Timer(messageTimeout, new ActionListener() 
        {
            @Override
            public void actionPerformed(ActionEvent e) 
            {
                statusMessageLabel.setText("Command: " + e.getActionCommand() + ", Param: " + e.paramString());
            }
        });
        messageTimer.setRepeats(false);

        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) 
        {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new javax.swing.Timer(busyAnimationRate, new ActionListener() 
        {
            @Override
            public void actionPerformed(ActionEvent e) 
            {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        screenHeight = (int) Toolkit.getDefaultToolkit().getScreenSize().getHeight();
        screenWidth = (int) Toolkit.getDefaultToolkit().getScreenSize().getWidth();
        sSXLocation = (screenWidth / 2) / 2;
        sSYLocation = screenHeight - ((screenHeight / 2) / 2);

        s.cont = false;
        getCurrentlyPlaying();
    }

    public void executeTask() 
    {
        if (!isPlaying()) 
        {
            task.execute();
        }
    }

    public MusicBoxView getInstance() 
    {
        return p;
    }

    public void connectToServer() 
    {
        try 
        {
            if(serverAddress == null || serverAddress.equals(""))
            {
                serverAddress = "";
                try 
                {
                    SwingUtilities.invokeAndWait(new Thread()
                    {
                        public void run()
                        {
                            String address = JOptionPane.showInputDialog(null, "Please input the IP address or the name \nof the computer the MusicBox Server is running on:", "Server Address", JOptionPane.PLAIN_MESSAGE);
                            if(address != null && !address.equals(""))
                            {
                                int retVal = JOptionPane.showConfirmDialog(null, "Do you want to save this address?", "Save Address", JOptionPane.YES_NO_OPTION);
                                if(retVal == JOptionPane.YES_OPTION)
                                {
                                    try 
                                    {
                                        ls.save(address, "serveraddress.xml");
                                    } 
                                    catch (IOException ex) 
                                    {
                                        showErrorDialog(ex);
                                    }
                                }
                            }
                            serverAddress = address;
                            
                        }
                    });
                } 
                catch (InterruptedException ex) 
                {
                    showErrorDialog(ex);
                } 
                catch (InvocationTargetException ex) 
                {
                    showErrorDialog(ex);
                }
            }
            while(serverAddress != null && serverAddress.equals(""))
                System.out.print("");
            if(serverAddress == null)
                return;
            socket = new Socket(serverAddress, 17372);
            sl = new SocketListener(socket, p);
            jMenuItem7.setText("Disconnect");
        } 
        catch (UnknownHostException ex) 
        {
            showErrorDialog(ex);
        } 
        catch (IOException ex) 
        {
            showErrorDialog(ex);
        }
    }

    /**
     *
     * @param m
     * @return r
     */
    protected ImageIcon getAlbumArt(MusicOutputDesign m) 
    {
        //boolean foundArt = false;
        String file = m.getFilePath() + "/" + m.getFile();
        ImageIcon r = new ImageIcon();

        try 
        {
            AudioFile f = AudioFileIO.read(new File(file));
            Artwork albumArt = f.getTag().getFirstArtwork();
            byte[] b1 = albumArt.getBinaryData();
            ImageIcon iI = new ImageIcon(b1);
            int h = iI.getIconHeight();
            int w = iI.getIconWidth();
            Image i = iI.getImage();
            Image j = i.getScaledInstance(100, 100, Image.SCALE_SMOOTH);
            r = new ImageIcon(j);
            foundArt = true;
            m.setImagePath("embedded");
            previousLoc = new File(file).getParent();
            if (h <= 0 || w <= 0) 
            {
                foundArt = false;
                m.setImagePath("NoImage");
                r = null;
            }
        } 
        catch (Exception ex) 
        {
            if (new File(file).getParent().compareTo(previousLoc) != 0) 
            {
                previousLoc = new File(file).getParent();
                File foo = new File(file);
                if (foo.getParentFile() != null) 
                {
                    File myDir = foo.getParentFile();
                    String[] dirNames = myDir.list(new ImageFilter());
                    if (dirNames != null && dirNames.length > 0) 
                    {
                        ImageIcon max = new ImageIcon(myDir.getAbsolutePath() + "/" + dirNames[0]);
                        for (int p1 = 0; p1 < dirNames.length; p1++) 
                        {
                            ImageIcon iK = new ImageIcon(myDir.getAbsolutePath() + "/" + dirNames[p1]);
                            if (iK.getIconHeight() + iK.getIconWidth() > max.getIconHeight() + iK.getIconWidth()) 
                            {
                                max = iK;
                                m.setImagePath(myDir.getAbsolutePath() + "/" + dirNames[p1]);
                            }
                        }
                        Image k = max.getImage();
                        Image l = k.getScaledInstance(100, 100, Image.SCALE_SMOOTH);
                        r = new ImageIcon(l);
                        foundArt = true;
                        if (max.getIconHeight() <= 0 || max.getIconWidth() <= 0) 
                        {
                            foundArt = false;
                            m.setImagePath("NoImage");
                            r = null;
                        }
                    }
                }
            }
        }
        return r;
    }

    private void removeNonDuplicates(Vector<MusicOutputDesign> currentlyShowing) 
    {
        Collections.sort(currentlyShowing, new ArtistComparator());
        for (int i = 0; i < currentlyShowing.size() - 1; i++) 
        {
            if (currentlyShowing.elementAt(i).getTrackTitle().compareToIgnoreCase(currentlyShowing.elementAt(i + 1).getTrackTitle()) == 0
                    && currentlyShowing.elementAt(i).getArtist().compareToIgnoreCase(currentlyShowing.elementAt(i + 1).getArtist()) != 0) 
            {
                if (i == 0) 
                {
                    currentlyShowing.remove(i);
                    i -= 1;
                } 
                else 
                {
                    if (currentlyShowing.elementAt(i).getTrackTitle().compareToIgnoreCase(currentlyShowing.elementAt(i - 1).getTrackTitle()) == 0
                            && currentlyShowing.elementAt(i).getArtist().compareToIgnoreCase(currentlyShowing.elementAt(i - 1).getArtist()) != 0) 
                    {
                        currentlyShowing.remove(i);
                        i -= 1;
                    }
                }
            } 
            else if (currentlyShowing.elementAt(i).getTrackTitle().compareToIgnoreCase(currentlyShowing.elementAt(i + 1).getTrackTitle()) != 0) 
            {
                if (i == 0) 
                {
                    currentlyShowing.remove(i);
                    i -= 1;
                } 
                else 
                {
                    if (currentlyShowing.elementAt(i).getTrackTitle().compareToIgnoreCase(currentlyShowing.elementAt(i - 1).getTrackTitle()) != 0) 
                    {
                        currentlyShowing.remove(i);
                        i -= 1;
                    }
                }
            }
        }
    }

    class SetLibraryImages extends TimerTask 
    {
        @Override
        public void run() 
        {
            createAllLibraryAlbumPanels();
            createAllLibraryArtistPanels();
            int numRows = ((CustomTableModel) currentlyShowingTable.getModel()).getRowCount();
            for (int i = 0; i < numRows; i++) 
            {
                ((CustomTableModel) currentlyShowingTable.getModel()).removeRow(0);
            }
            populateTableModel((CustomTableModel) currentlyShowingTable.getModel());
            try 
            {
                java.sql.Statement stmt = back.db.getConnection().createStatement();
                setAllLibraryAlbumImages(stmt);
                setAllLibraryArtistImages(stmt);
                stmt.close();
            } 
            catch (SQLException ex) 
            {
                //ex.printStackTrace();
                showErrorDialog(ex);
            }
        }
    }

    /**
     *
     * @param stmt
     */
    protected void setAllLibraryArtistImages(Statement stmt) 
    {
        Component[] panels = jPanel3.getComponents();
        while (panels.length != jList1.getModel().getSize() - 1) 
        {
            panels = jPanel3.getComponents();
        }
        for (int i = 0; i < panels.length; i++) 
        {
            JLabel l = (JLabel) panels[i].getComponentAt(0, 0);
            String artist = l.getToolTipText();
            Vector<MusicOutputDesign> m = getAllInstancesOfArtist(artist);
            foundArt = false;
            for (int k = 0; k < m.size(); k++) 
            {
                if (m.elementAt(k).getImagePath().compareToIgnoreCase("NoImage") != 0 && m.elementAt(k).getImagePath().compareToIgnoreCase("embedded") != 0) 
                {
                    if (!m.elementAt(k).getImagePath().toLowerCase().contains("http://")) 
                    {
                            try
                            {
                                l.setIcon(new ImageIcon(new ImageIcon(m.elementAt(k).getImagePath()).getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                            }
                            catch(Exception ex)
                            {
                                showErrorDialog(ex);
                            }
                    } 
                    else
                    {
                        if(internetConnection)
                        {
                            try
                            {
                                l.setIcon(new ImageIcon(new ImageIcon(new URL(m.elementAt(k).getImagePath())).getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                            }
                            catch (MalformedURLException ex) {
                                showErrorDialog(ex);
                            }
                            catch(Exception ex)
                            {
                                showErrorDialog(ex);
                            }
                        }
                    }
                    foundArt = true;
                    break;
                }
                ImageIcon ii = getAlbumArt(m.elementAt(k));
                if (foundArt == true) 
                {
                    l.setIcon(ii);
                    /*if (m.elementAt(k).getImagePath().compareTo("embedded") != 0) 
                    {
                        try 
                        {
                            db.updateImagePath(stmt, m.elementAt(k));
                        } 
                        catch (SQLException ex) 
                        {
                            showErrorDialog(ex);
                        }
                    }*/
                    break;
                }
            }
            if (foundArt == false || l.getIcon().getIconHeight() <= 0 || l.getIcon().getIconWidth() <= 0) 
            {
                l.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
            }
        }
    }

    /**
     *
     * @param stmt
     */
    protected void setAllLibraryAlbumImages(Statement stmt) 
    {
        Component[] panels = jPanel4.getComponents();
        while (panels.length != jList2.getModel().getSize() - 1) 
        {
            panels = jPanel4.getComponents();
        }
        for (int i = 0; i < panels.length; i++) 
        {
            JLabel l = (JLabel) panels[i].getComponentAt(0, 0);
            String album = l.getToolTipText();
            Vector<MusicOutputDesign> m = getAllInstancesOfAlbum(album);
            foundArt = false;
            for (int k = 0; k < m.size(); k++) 
            {
                if (m.elementAt(k).getImagePath().compareToIgnoreCase("NoImage") != 0 && m.elementAt(k).getImagePath().compareToIgnoreCase("embedded") != 0) 
                {
                    if (!m.elementAt(k).getImagePath().toLowerCase().contains("http://")) 
                    {
                        try
                        {
                            l.setIcon(new ImageIcon(new ImageIcon(m.elementAt(k).getImagePath()).getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                            foundArt = true;
                            break;
                        }
                        catch(Exception ex)
                        {
                            showErrorDialog(ex);
                        }
                    } 
                    else 
                    {
                        if(internetConnection)
                        {
                            try 
                            {
                                l.setIcon(new ImageIcon(new ImageIcon(new URL(m.elementAt(k).getImagePath())).getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                                foundArt = true;
                                break;
                            } 
                            catch (MalformedURLException ex) 
                            {
                                showErrorDialog(ex);
                            }
                            catch(Exception ex){
                                showErrorDialog(ex);
                            }
                        }
                    }
                }
                ImageIcon ii = getAlbumArt(m.elementAt(k));
                if (foundArt == true) 
                {
                    l.setIcon(ii);
                    /*if (m.elementAt(k).getImagePath().compareTo("embedded") != 0) 
                    {
                        try 
                        {
                            db.updateImagePath(stmt, m.elementAt(k));
                        } 
                        catch (SQLException ex) 
                        {
                            showErrorDialog(ex);
                        }
                    }*/
                    break;
                }
            }
            if (foundArt == false || l.getIcon().getIconHeight() <= 0 || l.getIcon().getIconWidth() <= 0) 
            {
                l.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
            }
        }
    }

    /**
     *
     */
    protected void createAllLibraryArtistPanels() 
    {
        jPanel3.removeAll();
        previousLoc = "awesome";
        //Collections.sort(allMusic, new ArtistComparator());

        java.awt.EventQueue.invokeLater(new Runnable() 
        {
            @Override
            public void run() 
            {
                for (int i = 1; i < jList1.getModel().getSize(); i++) 
                {
                    String s = (String) jList1.getModel().getElementAt(i);
                    JLabel j = new JLabel();
                    j.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    j.setToolTipText(s);
                    j.setSize(100, 100);
                    j.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                    j.addMouseListener(cma);
                    JPanel jp = new JPanel();
                    jp.setLayout(new BorderLayout());
                    jp.add(j, BorderLayout.PAGE_START);
                    jp.setSize(100, 100);
                    jPanel3.add(jp);
                }
                jScrollPane9.setViewportView(jPanel3);
            }
        });
    }

    /**
     *
     */
    protected void createAllLibraryAlbumPanels() 
    {
        previousLoc = "awesome";
        jPanel4.removeAll();
        //Collections.sort(allMusic, new AlbumComparator());
        java.awt.EventQueue.invokeLater(new Runnable() 
        {
            @Override
            public void run() 
            {
                for (int i = 1; i < jList2.getModel().getSize(); i++) 
                {
                    String s = (String) jList2.getModel().getElementAt(i);
                    JLabel j = new JLabel();
                    j.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    j.setSize(100, 100);
                    j.setToolTipText(s);
                    j.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                    j.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    j.addMouseListener(cma);
                    JPanel jp = new JPanel();
                    jp.setSize(100, 100);
                    jp.setLayout(new BorderLayout());
                    jp.add(j, BorderLayout.PAGE_START);
                    jPanel4.add(jp);
                }
                jScrollPane11.setViewportView(jPanel4);
            }
        });
    }

    /**
     *
     */
    public void setFont() 
    {
        playButton.setFont(projectFont);
        pauseButton.setFont(projectFont);
        stopButton.setFont(projectFont);
        shuffleButton.setFont(projectFont);
        previousTrackButton.setFont(projectFont);
        nextTrackButton.setFont(projectFont);
        jButton7.setFont(projectFont);
        jButton8.setFont(projectFont);
        currentTrackInfoBox.setFont(projectFont);
        jTabbedPane1.setFont(projectFont);
        jLabel1.setFont(projectFont);
        jLabel2.setFont(projectFont);
        jList1.setFont(projectFont);
        jList2.setFont(projectFont);
        jList3.setFont(projectFont);
        playlistTable.setFont(projectFont);
        currentlyPlayingTable.setFont(projectFont);
        currentlyShowingTable.setFont(projectFont);
        sharedLibraryTable.setFont(projectFont);
        addressList.setFont(projectFont);
        jLabel5.setFont(projectFont);
        addressList.setFont(projectFont);
        sharedPlayList.setFont(projectFont);
    }

    /**
     *
     */
    public void setFontColor()
    {
        playButton.setForeground(fontColor);
        pauseButton.setForeground(fontColor);
        stopButton.setForeground(fontColor);
        shuffleButton.setForeground(fontColor);
        previousTrackButton.setForeground(fontColor);
        nextTrackButton.setForeground(fontColor);
        jButton7.setForeground(fontColor);
        jButton8.setForeground(fontColor);
        currentTrackInfoBox.setForeground(fontColor);
        jTabbedPane1.setForeground(fontColor);
        jLabel1.setForeground(fontColor);
        jLabel2.setForeground(fontColor);
        jList1.setForeground(fontColor);
        jList2.setForeground(fontColor);
        jList3.setForeground(fontColor);
        playlistTable.setForeground(fontColor);
        currentlyPlayingTable.setForeground(fontColor);
        currentlyShowingTable.setForeground(fontColor);
        sharedLibraryTable.setForeground(fontColor);
        addressList.setForeground(fontColor);
        jLabel5.setForeground(fontColor);
        addressList.setForeground(fontColor);
        sharedPlayList.setForeground(fontColor);
    }

    /**
     *
     */
    @SuppressWarnings("static-access")
    public void showRating() {
        if (back.currentTrack != null) {
            ratingLabel.setIcon(smileys[Integer.parseInt(back.currentTrack.getRating())]);
            if (mini != null) {
                mini.jLabel1.setIcon(smileys[Integer.parseInt(back.currentTrack.getRating())]);
            }
        }
    }

    /**
     *  Displays the about box.
     */
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(true);
            aboutBox = new MusicBoxAboutBox(mainFrame,p);
            //aboutBox.createBufferStrategy(1);
            aboutBox.setLocationRelativeTo(mainFrame);
        }

        MusicBoxApp.getApplication().show(aboutBox);
    }

    /**
     *
     */
    public void showMiniPlayer() 
    {
        final JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
        JFrame.setDefaultLookAndFeelDecorated(true);
        if (mini == null) 
        {
            mini = new MiniPlayer(p);
            mini.setLocationRelativeTo(mainFrame);
        }
        if (!mini.isVisible()) 
        {
            if (songSplash != null && songSplash.isVisible()) 
            {
                songSplash.dispose();
            }
            MusicBoxApp.getApplication().show(mini);
            mainFrame.setVisible(false);
            mini.initGraphics();
        } 
        else 
        {
            mini.setVisible(false);
            mainFrame.setVisible(true);
        }
    }

    /**
     * Displays the Directory Chooser.
     */
    @Action
    public void showDirChooser() 
    {
        if (dirChooser == null) 
        {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(true);
            dirChooser = new DirChooser(mainFrame,this.p);
            dirChooser.setLocationRelativeTo(mainFrame);
        }
        MusicBoxApp.getApplication().show(dirChooser);
    }

    /**
     * Displays the File Chooser
     */
    @Action
    public void showFileChooser() 
    {
        if (fileChooser == null) 
        {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(true);
            fileChooser = new FileChooser(mainFrame,p);
            fileChooser.setLocationRelativeTo(mainFrame);
        }

        MusicBoxApp.getApplication().show(fileChooser);
    }

    /**
     *
     * @param current
     */
    public MusicOutputDesign showFileUnavailable(MusicOutputDesign current) 
    {
        final FindFileRunnable ffr = new FindFileRunnable(p,current);
        if(EventQueue.isDispatchThread())
        {
                    ffr.run();
        }
        else
        {
            EventQueue.invokeLater(ffr);
        }
        return ffr.getNewTrack();
        /*JDialog fileUnavailable = new FileUnavailable(mainFrame, current);
        JDialog.setDefaultLookAndFeelDecorated(true);
        fileUnavailable.setLocationRelativeTo(mainFrame);
        MusicBoxApp.getApplication().show(fileUnavailable);*/
    }

    /**
     * Displays AlbumArt dialog.
     */
    @Action
    @SuppressWarnings("static-access")
    public void showLargeAlbumArtwork() 
    {
        JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
        AlbumArt.setDefaultLookAndFeelDecorated(true);
        if (back.currentTrack.getImagePath().compareTo("NoImage") != 0 && back.currentTrack.getImagePath().compareTo("embedded") != 0) 
        {
            new AlbumArt(mainFrame, back.currentTrack.getImagePath(), back.currentTrack.getArtist(), back.currentTrack.getAlbum(),p);
        } 
        else if (back.googleImages.size() == 0) 
        {
            new AlbumArt(mainFrame, new File(back.fFilename),p);
        } 
        else 
        {
            new AlbumArt(mainFrame, back.googleImages, googleImageLocation, back.currentTrack.getArtist(), back.currentTrack.getAlbum(),p);
        }
    }

    /**
     *
     * @param i
     * @param s
     * @param p
     * @param j
     */
    @Action
    @SuppressWarnings("static-access")
    public void showSmallAlbumArt(ImageIcon i, String s, Point p, JLabel j) {
        if (smallArt != null && smallArt.isVisible() && smallArt.googleSearched == false) 
        {
            smallArt.dispose();
        }
        if (smallArt == null || smallArt.googleSearched == false) 
        {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            smallArt = new SmallAlbumArt(mainFrame, i, s, j, this.p);
            smallArt.setTitle(s);
            smallArt.setLocation(p);
            smallArt.setDefaultLookAndFeelDecorated(false);
            //JDialog.setDefaultLookAndFeelDecorated(false);
            MusicBoxApp.getApplication().show(smallArt);
        }
    }

    /**
     *
     */
    public void showPreferences() 
    {
        JFrame.setDefaultLookAndFeelDecorated(true);
        if (pref != null && pref.isVisible()) 
        {
            pref.dispose();
        }
        pref = new Preferences(p);
        pref.setVisible(true);
    }

    /**
     *  Displays the SongSplash dialog.
     * @param m information used to display in the dialog.
     */
    @Action
    public void showSongSplash(final MusicOutputDesign m) 
    {
        if (songSplash != null && songSplash.isVisible()) 
        {
            songSplash.dispose();
        }
        java.awt.EventQueue.invokeLater(new Runnable() 
        {
            @Override
            @SuppressWarnings("static-access")
            public void run() 
            {
                songSplash = new SongSplash(MusicBoxApp.getApplication().getMainFrame(), m, p);
                songSplash.setLocation(sSXLocation, sSYLocation);
                SongSplash.setDefaultLookAndFeelDecorated(false);
                songSplash.setVisible(true);
                while(!songSplash.isVisible())
                {
                    System.out.print("");
                }
                songSplash.disposeSongSplash();
            }
        });

    }

    /**
     * Displays SongInfo Dialog
     * @param jT The JTable that is selected
     * @param v The vector that corresponds with the JTable
     */
    @Action
    public void showSongInfo(JTable jT, Vector<MusicOutputDesign> v) 
    {
        int[] location = jT.getSelectedRows();
        for (int i = 0; i < location.length; i++) 
        {
            location[i] = jT.convertRowIndexToModel(location[i]);
        }
        JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
        JDialog.setDefaultLookAndFeelDecorated(true);
        songInfo = new SongInfo(mainFrame, v, 0, location, p);
        songInfo.setLocationRelativeTo(mainFrame);
        if(songInfo.isInitialized())
        {
            MusicBoxApp.getApplication().show(songInfo);
        }
    }

    @Action
    public void showSongInfo(JTable jT) 
    {
        int[] locations = jT.getSelectedRows();
        for (int i = 0; i < locations.length; i++) 
        {
            if (jT.getName().equalsIgnoreCase("CurrentlyShowingTable")) 
            {
                locations[i] = jT.convertRowIndexToView(locations[i]);
            }
            if (jT.getName().equalsIgnoreCase("playlistTable")) 
            {
                String id = playList.elementAt(jT.convertRowIndexToView(locations[i]));
                for(MusicOutputDesign mod: allMusic)
                {
                    if(mod.getId().equals(id))
                    {
                        locations[i] = allMusic.indexOf(mod);
                        break;
                    }
                }
            } 
            else if (jT.getName().equalsIgnoreCase("CurrentlyPlayingTable")) 
            {
                String id = currentlyPlaying2.elementAt(jT.convertRowIndexToView(locations[i]));
                for(MusicOutputDesign mod: allMusic)
                {
                    if(mod.getId().equals(id))
                    {
                        locations[i] = allMusic.indexOf(mod);
                        break;
                    }
                }
            }
        }
        if(locations[0] >= 0)
        {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(true);
            songInfo = new SongInfo(mainFrame, allMusic, 0, locations, p);
            songInfo.setLocationRelativeTo(mainFrame);
            if(songInfo.isInitialized())
            {
                MusicBoxApp.getApplication().show(songInfo);
            }
        }
    }

    /**
     *  Displays the ErrorDialog
     * @param ex Exception to be displayed.
     */
    @Action
    public void showErrorDialog(final Exception ex) 
    {
        if (errorMessage) 
        {
            final JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            java.awt.EventQueue.invokeLater(new Runnable() 
            {

                @Override
                public void run() 
                {
                    JDialog.setDefaultLookAndFeelDecorated(true);
                    errorDialog = new ErrorDialog(mainFrame, ex, p);
                    errorDialog.setLocationRelativeTo(mainFrame);
                    MusicBoxApp.getApplication().show(errorDialog);
                }
            });

        }
    }

    /**
     * Updates an element in a vector with the latest information.
     * @param v1 Vector to be updated.
     * @param m1 New information.
     * @param jT JTable that is associated with the vector.
     */
    public void updateVector(Vector<MusicOutputDesign> v1, MusicOutputDesign m1, JTable jT) 
    {
        for (int i = 0; i < v1.size(); i++) 
        {
            
            if (v1.elementAt(i).getFilePath().compareToIgnoreCase(m1.getFilePath()) == 0 && v1.elementAt(i).getFile().compareToIgnoreCase(m1.getFile()) == 0) 
            {
                v1.setElementAt(m1, i);
                //TODO: i is vector location, not table location in all circumstances.
                //Need to find table location
                if (jT.isShowing()) 
                {
                    updateTableAtLocation(m1, jT.convertRowIndexToView(i), jT);
                }
                return;
            } 
            else if (v1.elementAt(i).getAlbum().compareToIgnoreCase(m1.getAlbum()) == 0 && v1.elementAt(i).getArtist().compareToIgnoreCase(m1.getArtist()) == 0 && v1.elementAt(i).getTrackTitle().compareToIgnoreCase(m1.getTrackTitle()) == 0) 
            {
                v1.setElementAt(m1, i);
                if (jT.isShowing()) 
                {
                    updateTableAtLocation(m1, jT.rowAtPoint(jT.getMousePosition()), jT);
                }
                return;
            }
        }
    }

    public void updateVectorAtLocation(Vector<Integer> v1, int location, MusicOutputDesign m1, JTable jT) 
    {
        allMusic.setElementAt(m1, v1.elementAt(location));
        if (jT.isShowing()) 
        {
            updateTableAtLocation(m1, location, jT);
        }
    }
    
    public void updateVectorAtLocation(Vector<String> v1, int location, String ID, MusicOutputDesign m1, JTable jT)
    {
        Vector<MusicOutputDesign> temp = (Vector<MusicOutputDesign>)allMusic.clone();
        Collections.sort(temp, new IDComparator());
        allMusic.setElementAt(m1, allMusic.indexOf(back.findTrack(temp,m1.getId())));
        if (jT.isShowing()) 
        {
            updateTableAtLocation(m1, location, jT);
        }
        temp.clear();
        temp = null;
    }
    
    public RowFilter<CustomTableModel, Object> regexSearch(String filterText, int index) 
    {
        //([{\^-$|]})?*+
        filterText = "\\Q" + filterText + "\\E";
        RowFilter<CustomTableModel, Object> rf = null;
        //If current expression doesn't parse, don't update.
        try 
        {
            rf = CustomRowFilter.regexFilter(filterText,index);
        } 
        catch (java.util.regex.PatternSyntaxException pse) 
        {
            showErrorDialog(pse);
        }
        return rf;
    }

    public RowFilter<CustomTableModel, Object> regexSearch(String filterText) 
    {
        //([{\^-$|]})?*+
        filterText = "\\Q" + filterText + "\\E";
        RowFilter<CustomTableModel, Object> rf = null;
        //If current expression doesn't parse, don't update.
        try 
        {
            rf = CustomRowFilter.regexFilter(filterText);
        } 
        catch (java.util.regex.PatternSyntaxException pse) 
        {
            showErrorDialog(pse);
        }
        return rf;
    }

    /**
     * Updates a table at one specific location.
     * @param m New information
     * @param location Location in the table to be updated
     * @param jT JTable to be updated.
     */
    public void updateTableAtLocation(MusicOutputDesign m, int location, final JTable jT) 
    {
        //location = jT.convertRowIndexToView(location);
        final CustomTableModel ctm = (CustomTableModel) jT.getModel();
        final DefaultListSelectionModel dlsm = (DefaultListSelectionModel) jT.getSelectionModel();
        final int anchor = dlsm.getAnchorSelectionIndex();
        final int lead = dlsm.getLeadSelectionIndex();
        //System.out.println(ctm.getValueAt(location, 1));
        ctm.setValueAt(m.getTrackNumber(), location, jT.getColumnModel().getColumnIndex(columnNames[0]));
        ctm.setValueAt(m.getTrackTitle(), location, jT.getColumnModel().getColumnIndex(columnNames[1]));
        ctm.setValueAt(m.getArtist(), location, jT.getColumnModel().getColumnIndex(columnNames[2]));
        ctm.setValueAt(m.getAlbum(), location, jT.getColumnModel().getColumnIndex(columnNames[3]));
        ctm.setValueAt(m.getTrackLength(), location, jT.getColumnModel().getColumnIndex(columnNames[4]));
        ctm.setValueAt(m.getPlayCount(), location, jT.getColumnModel().getColumnIndex(columnNames[5]));
        ctm.setValueAt(smileys[Integer.parseInt(m.getRating())], location, jT.getColumnModel().getColumnIndex(columnNames[6]));
        if (!SwingUtilities.isEventDispatchThread()) {
            try 
            {
                SwingUtilities.invokeAndWait(new Runnable() 
                {

                    public void run() 
                    {
                        ctm.fireTableDataChanged();
                        dlsm.setAnchorSelectionIndex(anchor);
                        dlsm.setLeadSelectionIndex(lead);
                        jT.setSelectionModel(dlsm);
                        if(anchor != -1 && lead != -1)
                        {
                            jT.addRowSelectionInterval(anchor, lead);
                        }
                    }
                });
            } 
            catch (InterruptedException ex) 
            {
                showErrorDialog(ex);
            } 
            catch (InvocationTargetException ex) 
            {
                showErrorDialog(ex);
            }
        } 
        else 
        {
            ctm.fireTableDataChanged();
            dlsm.setAnchorSelectionIndex(anchor);
            dlsm.setLeadSelectionIndex(lead);
            jT.setSelectionModel(dlsm);
            if(anchor != -1 && lead != -1)
            {
                jT.addRowSelectionInterval(anchor, lead);
            }
        }
    }

    /**
     * Plays the currently selected music, or everything in the library if
     * nothing is selected.
     */
    @SuppressWarnings({"static-access", "static-access"})
    public void play() 
    {
        playing = true;
        back.cont = true;
        if (/*currentlyPlaying.size()*/currentlyPlaying2.size() == 0) 
        {
            if (currentlyShowingTable.getSelectedRowCount() == 0) 
            {
                currentlyShowingTable.selectAll();
            }
            //if (currentlyShowingTable.getSelectedRowCount() > 0) {
            int[] locations = currentlyShowingTable.getSelectedRows();
            for (int i = 0; i < locations.length; i++) 
            {
                locations[i] = currentlyShowingTable.convertRowIndexToModel(locations[i]);
            }
            for (int i = 0; i < locations.length; i++) 
            {
                //currentlyPlaying.add(currentlyShowing.elementAt(location[i]));
                currentlyPlaying2.add(allMusic.elementAt(locations[i]).getId());
                //currentlyPlaying.add(locations[i]);
            }
            back.calculatePlaylistTime();
            currentlyShowingTable.clearSelection();
            /*} else {
            //currentlyShowingTable.selectAll();
            //currentlyPlaying = (Vector<MusicOutputDesign>) currentlyShowing.clone();
            //currentlyPlaying = (Vector<MusicOutputDesign>) allMusic.clone();
            //Backend.calculatePlaylistTime();
            //}*/

            //updateTable(currentlyPlaying, currentlyPlayingTable, "flag");
            populateTableModel((CustomTableModel)currentlyPlayingTable.getModel(), currentlyPlaying2, locations);
            //populateTableModel(currentlyPlayingTable, locations);
        }
        back = new Backend(back.currentLocation);
        back.setMusicBoxView(p);
        back.start();

        pauseButton.setEnabled(true);
        stopButton.setEnabled(true);
        shuffleButton.setEnabled(true);
        nextTrackButton.setEnabled(true);
        if (mini != null && mini.isVisible()) 
        {
            mini.nextButton.setEnabled(true);
        }
        playButton.setEnabled(false);
        task.addPropertyChangeListener(p);
    }

    /**
     * Pauses/resumes the current song being played.
     * @throws InterruptedException
     */
    public void pause() throws InterruptedException 
    {
        if (pause == false) 
        {
            back.suspend();
            pause = true;
            return;

        } 
        else if (pause == true) 
        {
            back.resume();
            pause = false;
            return;
        }
    }

    /**
     *  Stops the current song being played.
     */
    public void stop() 
    {
        playing = false;
        //task.setPlaying(false);
        back.sT.stop();
        back.sT = null;
        back.stop();
        if (back.line != null) 
        {
            back.line.drain();
            back.line.stop();
            back.line.close();
        }
        if(sl != null)
        {
            sl.setObjectToPlay(null);
        }
        task.removePropertyChangeListener(p);
        playButton.setEnabled(true);
        pauseButton.setEnabled(false);
        stopButton.setEnabled(false);
        shuffleButton.setEnabled(false);
        previousTrackButton.setEnabled(false);
        nextTrackButton.setEnabled(false);
        if (mini != null && mini.isVisible()) 
        {
            mini.previousButton.setEnabled(false);
            mini.nextButton.setEnabled(false);
        }
    }

    /**
     * Shuffles the current playlist.
     */
    public void shuffle() 
    {
        stop();
        back.shuffle();
        play();

    }

    public void nextGoogleImage() 
    {
        googleImageLocation += 1;
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(back.googleImages.elementAt(googleImageLocation)));
            ImageIcon ico = new ImageIcon(icon.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH));
            albumArtLabel.setIcon(ico);
        } 
        catch (MalformedURLException ex1) 
        {
            //ex1.printStackTrace();
            albumArtLabel.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
            showErrorDialog(ex1);
        }
        if (googleImageLocation == 0) 
        {
            jButton8.setEnabled(false);
        } 
        else 
        {
            jButton8.setEnabled(true);
        }
        if (googleImageLocation < back.googleImages.size() - 1) 
        {
            jButton7.setEnabled(true);
        } 
        else 
        {
            imageResults += 21;
            googleImageLocation = 0;
            String start = "&sa=N&start=" + imageResults + "&ndsp=21";
            back.googleImageSearch(googleSearch, start);
        }
    }

    public void previousGoogleImage() 
    {
        googleImageLocation -= 1;
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(back.googleImages.elementAt(googleImageLocation)));
            ImageIcon ico = new ImageIcon(icon.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH));
            albumArtLabel.setIcon(ico);
        } 
        catch (MalformedURLException ex1) 
        {
            //ex1.printStackTrace();
            albumArtLabel.setIcon(new ImageIcon(noImage.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
            showErrorDialog(ex1);
        }
        if (googleImageLocation == 0) 
        {
            if (imageResults == 0) 
            {
                jButton8.setEnabled(false);
            } 
            else 
            {
                imageResults -= 21;
                googleImageLocation = 19;
                String start = "&sa=N&start=" + imageResults + "&ndsp=21";
                back.googleImageSearch(googleSearch, start);
            }
        } 
        else 
        {
            jButton8.setEnabled(true);
        }
        if (googleImageLocation < back.googleImages.size() - 1) 
        {
            jButton7.setEnabled(true);
        } 
        else 
        {
            jButton7.setEnabled(false);
        }
    }

    /**
     * Stops the currently playing song and starts playing the next song in the
     * playlist.
     */
    @SuppressWarnings({"static-access", "static-access"})
    public void next() 
    {
        stop();
        back.currentLocation++;
        if (back.currentLocation == /*currentlyPlaying.size()*/currentlyPlaying2.size()) 
        {
            back.currentLocation = 0;
            return;
        }
        play();
    }

    /**
     * Stops the currently playing song and starts the previous song in the
     * playlist.
     */
    @SuppressWarnings("static-access")
    public void previous() 
    {
        stop();
        back.currentLocation--;
        play();
    }

    /**
     * way of removing a property for task.  (Used to finish the progress
     *  bar task.)
     */
    public void fin() 
    {
        //task.setProg(101);
    }

    /**
     *
     * @param dlm
     * @return dlm Returns a sorted list model.
     */
    public DefaultListModel sortListModel(DefaultListModel dlm) 
    {
        ArrayList<String> temp = new ArrayList<String>(dlm.size());
        String first = (String) dlm.remove(0);
        for (int i = 0; i < dlm.size(); i++) 
        {
            temp.add((String) dlm.elementAt(i));
        }
        Collections.sort(temp, String.CASE_INSENSITIVE_ORDER);
        dlm.clear();
        dlm.addElement(first);
        for (int i = 0; i < temp.size(); i++) 
        {
            dlm.addElement(temp.get(i));
        }
        return dlm;
    }

    /**
     *
     * @return temp
     */
    public Vector<String> getAllPlayLists() 
    {
        Vector<String> temp = new Vector<String>();
        try 
        {
            java.sql.Statement stmt = back.db.getConnection().createStatement();
            temp = db.playLists(stmt);
            stmt.close();
        } 
        catch (SQLException ex) 
        {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
        for (int i = 0; i < temp.size(); i++) 
        {
            if (temp.elementAt(i).contains("_")) 
            {
                String s = temp.elementAt(i).replace("_", " ");
                temp.setElementAt(s, i);
            }
        }
        return temp;
    }

    private void findAllTracks(Vector<String> temp) 
    {
        //Vector<MusicOutputDesign> all = (Vector<MusicOutputDesign>) allMusic.clone();
        //Collections.sort(allMusic, new IDComparator());
        //Collections.sort(all, new FullFileNameComparator());
        while (!temp.isEmpty()) 
        {
            String id = temp.remove(0);
            MusicOutputDesign mod = back.findTrack(id);
            populateTableModel((CustomTableModel) playlistTable.getModel(), mod);
            playList.add(id);
        }
    }

    private int binarySearchFullFileName(Vector<MusicOutputDesign> v, String value, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        if (mid >= v.size()) 
        {
            return -1;
        }
        String filePath = v.elementAt(mid).getFilePath().replace("\\", "/") + "/" + v.elementAt(mid).getFile();
        if (filePath.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchFullFileName(v, value, low, mid - 1);
        }
        else if (filePath.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchFullFileName(v, value, mid + 1, high);
        } 
        else 
        {
            return mid; // found
        }
    }

    private int binarySearchFilePath(Vector<MusicOutputDesign> v, String value, int low, int high, String flag) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        if (mid >= v.size()) 
        {
            return -1;
        }
        String filePath = v.elementAt(mid).getFilePath().replace("\\", "/");
        if (filePath.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchFilePath(v, value, low, mid - 1, flag);
        } 
        else if (filePath.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchFilePath(v, value, mid + 1, high, flag);
        } 
        else 
        {
            return mid; // found
        }
    }

    private int binarySearchFile(Vector<MusicOutputDesign> v, String value, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        if (mid >= v.size()) 
        {
            return -1;
        }
        String file = v.elementAt(mid).getFile();
        if (file.compareToIgnoreCase(value) > 0) 
        {
            return binarySearchFile(v, value, low, mid - 1);
        } 
        else if (file.compareToIgnoreCase(value) < 0) 
        {
            return binarySearchFile(v, value, mid + 1, high);
        } 
        else 
        {
            return mid; // found
        }
    }

    private void removeSongFromPlayList() 
    {
        int[] location = playlistTable.getSelectedRows();
        for (int i = 0; i < location.length; i++) 
        {
            location[i] = playlistTable.convertRowIndexToModel(location[i]);
        }
        String name = (String) jList3.getSelectedValue();
        Vector<String> temp = new Vector<String>();
        try 
        {
            java.sql.Statement stmt = back.db.getConnection().createStatement();
            for (int i = 0; i < location.length; i++) 
            {
                db.deleteSongFromPlaylist(stmt, name, playList.elementAt(location[i]));
            }
            temp = db.listPlayList(stmt, name);
            stmt.close();
        } 
        catch (SQLException ex) 
        {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
        playList.clear();
        findAllTracks(temp);
        //TODO:
        //updateTable(playList, playlistTable);
        //populateTableModel((CustomTableModel)playlistTable.getModel(), playList);
    }

    /**
     *
     * @param s
     * @return m
     */
    protected Vector<MusicOutputDesign> getAllInstancesOfArtist(String s) 
    {
        Vector<MusicOutputDesign> m = new Vector<MusicOutputDesign>();
        for (int i = 0; i < allMusic.size(); i++) 
        {
            if (s.compareToIgnoreCase(allMusic.elementAt(i).getArtist()) == 0) 
            {
                m.add(allMusic.elementAt(i));
            }
        }
        return m;
    }

    private Vector<MusicOutputDesign> getAllInstancesOfAlbum(String s) 
    {
        Vector<MusicOutputDesign> m = new Vector<MusicOutputDesign>();
        for (int i = 0; i < allMusic.size(); i++) 
        {
            if (s.compareToIgnoreCase(allMusic.elementAt(i).getAlbum()) == 0) 
            {
                m.add(allMusic.elementAt(i));
            }
        }
        return m;
    }

    class Task extends SwingWorker<String, String> 
    {
        boolean playing = false;
        /*
         * Main task. Executed in background thread.
         */

        @SuppressWarnings({"static-access", "static-access", "static-access"})
        @Override
        public String doInBackground() 
        {
            int progress = 0;
            //Initialize progress property.
            setProgress(progress);
            //while (progress < (int)trackLength) {
            playing = true;
            int i = 0;
            while (progress < 100) 
            {
                //Sleep for up to one second.
                try 
                {
                    Thread.sleep(1000);
                } 
                catch (InterruptedException ignore) 
                {
                }
                //Need to work on this
                double d = 0.0;
                if (back.fFilename.toLowerCase().endsWith(".ogg") || back.fFilename.toLowerCase().endsWith(".mp3") 
                        || back.fFilename.toLowerCase().endsWith(".wav") || back.fFilename.toLowerCase().endsWith(".aifc") 
                        || back.fFilename.toLowerCase().endsWith(".aiff") || back.fFilename.toLowerCase().endsWith(".aif") 
                        || back.fFilename.toLowerCase().endsWith(".au") || back.fFilename.toLowerCase().endsWith(".ape")
                        || back.fFilename.toLowerCase().endsWith(".flac")) 
                {
                    try 
                    {
                        DatatypeFactory df = DatatypeFactory.newInstance();
                        d = (back.line.getMicrosecondPosition() / 1000000) + timeSkipped;
                        SimpleDateFormat sdf = new SimpleDateFormat("hh:mm:ss");
                        Duration d1 = df.newDuration((back.line.getMicrosecondPosition() / 1000) + ((int) timeSkipped) * 1000);
                        String totalTime = new String();
                        if (d1.getDays() > 0) 
                        {
                            totalTime = totalTime + d1.getDays() + ":";
                        }
                        if (d1.getHours() > 0) 
                        {
                            if (d1.getHours() % 24 > 9) 
                            {
                                totalTime = totalTime + d1.getHours() % 24 + ":";
                            } 
                            else 
                            {
                                totalTime = totalTime + "0" + d1.getHours() % 24 + ":";
                            }
                        }
                        if (d1.getMinutes() > 0) 
                        {
                            if (d1.getMinutes() % 60 > 9) 
                            {
                                totalTime = totalTime + d1.getMinutes() % 60 + ":";
                            } 
                            else 
                            {
                                totalTime = totalTime + "0" + d1.getMinutes() % 60 + ":";
                            }
                        } 
                        else 
                        {
                            totalTime = totalTime + "00:";
                        }
                        if (d1.getSeconds() > 0) 
                        {
                            if (d1.getSeconds() % 60 > 9) 
                            {
                                totalTime = totalTime + d1.getSeconds() % 60;
                            } 
                            else 
                            {
                                totalTime = totalTime + "0" + d1.getSeconds() % 60;
                            }
                        } 
                        else 
                        {
                            totalTime = totalTime + "00";
                        }
                        jLabel1.setText(totalTime);
                        if (mini != null && mini.isVisible()) 
                        {
                            mini.jSlider1.setToolTipText(totalTime);
                            mini.jLabel2.setText(totalTime);
                        }
                    } 
                    catch (DatatypeConfigurationException ex) 
                    {
                        showErrorDialog(ex);
                    }
                } 
                else 
                {
                    i++;
                    d = i + timeSkipped;
                    double e = d;
                    e = e / 60;
                    int f1 = (int) e;
                    e = e - f1;
                    e = e * 60;
                    String tt = new String();
                    if (e < 10) 
                    {
                        if (f1 < 10) 
                        {
                            tt = "0" + f1 + ":0" + (int) e;
                        } 
                        else 
                        {
                            tt = f1 + ":0" + (int) e;
                        }
                    } 
                    else 
                    {
                        if (f1 < 10) 
                        {
                            tt = "0" + f1 + ":" + (int) e;
                        } 
                        else 
                        {
                            tt = f1 + ":" + (int) e;
                        }
                    }
                    jLabel1.setText(tt);
                    if (mini != null && mini.isVisible()) 
                    {
                        mini.jSlider1.setToolTipText(tt);
                        mini.jLabel2.setText(tt);
                    }
                }
                d = d / trackLength;
                d = d * 100;
                progress = (int) d;
                /*if (playing == false) {
                break;
                }*/
                if (progress > 100) 
                {
                    progress = 100;
                }
                if(!isDone())
                {    
                    setProgress(Math.min(progress, 100));
                }
                else
                {
                    jProgressBar1.setValue(Math.min(progress, 100));
                    if (mini != null && mini.isVisible()) 
                    {
                        mini.jSlider1.setValue(progress);
                    }
                }
            }
            //playing = false;
            return "Finished";
        }
        public void setProg(int progress)
        {
            setProgress(progress);
        }
        public boolean isPlaying()
        {
            return playing;
        }
        private void setPlaying(boolean playing)
        {
            this.playing = playing;
        }
        public void finished()
        {
            try
            {
                super.finalize();
            }
            catch(Throwable ex)
            {
                ex.printStackTrace();
            }
        }
    }

    @Override
    public void actionPerformed(ActionEvent evt) 
    {
    }

    public boolean isPlaying() 
    {
        if(task.isDone())
        {
            task.finished();
            task = new Task();
            task.execute();
        }
        if (playing && task.isPlaying()) 
        {
            return true;
        }
        else 
        {
            return false;
        }
    }

    /**
     * Updates the page according to the link the user clicks on.
     * @param e
     */
    @Override
    public void hyperlinkUpdate(HyperlinkEvent e) 
    {
        try 
        {
            if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) 
            {
                jEditorPane1.setPage(e.getURL());
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("BODY { font-size: 12pt; text-align: left;}");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("P { font-size: 12pt; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("A { font-size: 12pt;"
                        + "text-decoration:underline; color:blue; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("LI { font-size: 12pt; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("DIV { font-size: 12pt; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("SPAN { font-size: 12pt; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("TD { font-size: 12pt; }");
                ((HTMLDocument) jEditorPane1.getDocument()).getStyleSheet().addRule("TH { font-size: 12pt; }");
            }

        } 
        catch (IOException ex) 
        {
            if(doc != null)
            {
                jEditorPane1.setDocument(doc);
            }
            else
            {
                try 
                {
                    jEditorPane1.setPage("http://www.wikipedia.org");
                } 
                catch (IOException ex1) 
                {
                    showErrorDialog(ex1);
                }
            }
            //ex.printStackTrace();
            showErrorDialog(ex);
        }

    }

    /**
     * Invoked when task's progress property changes.
     */
    @Override
    public void propertyChange(PropertyChangeEvent evt) 
    {
        if ("progress".equals(evt.getPropertyName())) 
        {
            int progress = task.getProgress();
            jProgressBar1.setValue(progress);
            if (mini != null && mini.isVisible()) 
            {
                mini.jSlider1.setValue(progress);
            }
        }
    }

    @SuppressWarnings("static-access")
    public void removeFromCurrentlyPlaying() 
    {
        //Vector<Integer> temp = new Vector<Integer>();
        Vector<String> temp = new Vector<String>();
        int[] locations = currentlyPlayingTable.getSelectedRows();
        for (int i = 0; i < locations.length; i++) 
        {
            locations[i] = currentlyPlayingTable.convertRowIndexToModel(locations[i]);
        }
        for (int i = 0; i < locations.length; i++) 
        {
            if(locations[i] < /*currentlyPlaying.size()*/currentlyPlaying2.size())
            {
                temp.add(/*currentlyPlaying*/currentlyPlaying2.elementAt(locations[i]));
                back.db.deleteFromCurrentlyPlaying(currentlyPlaying2.elementAt(locations[i]));
            }
            ((CustomTableModel) currentlyPlayingTable.getModel()).removeRow(locations[i] - i);
            //currentlyPlaying.remove(locations[i]-i);
        }
        //currentlyPlaying.removeAll(temp);
        currentlyPlaying2.removeAll(temp);
        temp.clear();
        for (int j = 0; j < /*currentlyPlaying.size()*/currentlyPlaying2.size(); j++) 
        {
            if (back.currentTrack.getId().equals(/*currentlyPlaying*/currentlyPlaying2.elementAt(j))) 
            {
                back.currentLocation = j;
                break;
            }
        }
        //updateTable(currentlyPlaying, currentlyPlayingTable, "flag");
        back.calculatePlaylistTime();
    }
    
    public void getCurrentlyPlaying()
    {
        String query = "Select * from CurrentlyPlaying Order By PlayOrder Asc";
        Vector<MusicOutputDesign> temp = (Vector<MusicOutputDesign>)allMusic.clone();
        Collections.sort(temp, new IDComparator());
        CustomTableModel ctm = (CustomTableModel)currentlyPlayingTable.getModel();
        if(ctm.getColumnCount() == 0)
        {
            for(String column: columnNames)
            {
                ctm.addColumn(column);
            }
        }
        try 
        {
            ResultSet rs = back.db.getConnection().createStatement().executeQuery(query);
            while(rs.next())
            {
                int current = rs.getInt("CurrentTrack");
                String id = rs.getString("ID");
                currentlyPlaying2.add(id);
                if(current == 1)
                {
                    back.currentLocation = currentlyPlaying2.size()-1;
                }
                MusicOutputDesign mod = back.findTrack(id);
                ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                    mod.getTrackLength(), mod.getPlayCount(), smileys[Integer.parseInt(mod.getRating())]});
            }
            //TEMPORARY
            back.calculatePlaylistTime();
        } 
        catch (SQLException ex) 
        {
            if(ex.getMessage().indexOf("Table not found in statement") > -1)
            {
                db.createCurrentlyPlayingTable();
            }
            showErrorDialog(ex);
        }
    }
    /**
     * Retrieves all the music from the db.
     */
    public void getAllMusic() 
    {
        allMusic.clear();
        ResultSet rs = null;
        try 
        {
            java.sql.Statement stmt = back.db.getConnection().createStatement();
            rs = db.list(stmt, "*");
            while (rs.next()) 
            {
                MusicOutputDesign m = new MusicOutputDesign();
                m.setFilePath(rs.getString("filePath"));
                m.setFile(rs.getString("file"));
                m.setTrackNumber("" + rs.getInt("trackNumber"));
                m.setTrackTitle(rs.getString("trackTitle"));
                m.setArtist(rs.getString("artist"));
                m.setAlbum(rs.getString("album"));
                m.setTrackLength(rs.getString("trackLength"));
                m.setPlayCount("" + rs.getInt("playCount"));
                m.setRating("" + rs.getInt("rating"));
                m.setImagePath(rs.getString("imagePath"));
                m.setId(rs.getString("ID"));
                if(m.getId() == null)
                {
                    m.setId(m.getArtist()+"-"+m.getAlbum()+"-"+m.getTrackNumber());
                }
                allMusic.add(m);
            }
            stmt.close();
        } 
        catch (SQLException ex) 
        {
            showErrorDialog(ex);
        }
        //Collections.sort(allMusic, new IDComparator());
    }

    public void populateTableModel(CustomTableModel ctm, MusicOutputDesign m) 
    {
        if (ctm.getColumnCount() == 0) 
        {
            for (int i = 0; i < columnNames.length; i++) 
            {
                ctm.addColumn(columnNames[i]);
            }
        }
        ctm.addRow(new Object[]{m.getTrackNumber(), m.getTrackTitle(), m.getArtist(), m.getAlbum(),
                    m.getTrackLength(), m.getPlayCount(), smileys[Integer.parseInt(m.getRating())]});
    }

    public CustomTableModel populateTableModel(CustomTableModel ctm) 
    {
        if (ctm.getColumnCount() == 0) 
        {
            for (int i = 0; i < columnNames.length; i++)
            {
                ctm.addColumn(columnNames[i]);
            }
        }
        for (MusicOutputDesign mod: allMusic) 
        {
            ctm.addRow(mod,this);
        }
        return ctm;
    }
    
    public void populateTableModel2(CustomTableModel ctm, Vector<String> v)
    {
        if (ctm.getColumnCount() == 0) 
        {
            for (String column: columnNames) 
            {
                ctm.addColumn(column);
            }
        }
        boolean shared = false;
        boolean local = false;
        Vector<MusicOutputDesign> temp1 = null;
        Vector<MusicOutputDesign> temp2 = null;
        for(String ID: v)
        {
            MusicOutputDesign mod;
            if(!ID.startsWith("sharedLibraryTable"))
            {
                if(!local)
                {
                    temp1 = (Vector<MusicOutputDesign>)allMusic.clone();
                    Collections.sort(temp1, new IDComparator());
                    local = true;
                }
                mod = back.findTrack(temp1, ID);
            }
            else
            {
                if(!shared)
                {
                    temp2 = (Vector<MusicOutputDesign>)sharedLibrary.clone();
                    Collections.sort(temp2, new IDComparator());
                    shared = true;
                }
                mod = back.findTrack(temp2, ID.replace("sharedLibraryTable-", ""));
            }
            ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), smileys[Integer.parseInt(mod.getRating())]});
        }
        if(temp1 != null)
        {
            temp1.clear();
            temp1 = null;
        }
        if(temp2 != null)
        {
            temp2.clear();
            temp2 = null;
        }
    }

    public void populateTableModel(CustomTableModel ctm, Vector<Integer> v) 
    {
        if (ctm.getColumnCount() == 0) 
        {
            for (int i = 0; i < columnNames.length; i++) 
            {
                ctm.addColumn(columnNames[i]);
            }
        }
        for (int i = 0; i < v.size(); i++) 
        {
            MusicOutputDesign mod;
            if(v.elementAt(i) >= 0)
            {
                mod = allMusic.elementAt(v.elementAt(i));
            }
            else
            {
                mod = sharedLibrary.elementAt((v.elementAt(i)+1)*-1);
            }
            ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), smileys[Integer.parseInt(mod.getRating())]});
        }
    }

    public void populateTableModel(CustomTableModel ctm, Vector<String> v, int[] locations) 
    {
        if (ctm.getColumnCount() == 0) 
        {
            for (int i = 0; i < columnNames.length; i++) 
            {
                ctm.addColumn(columnNames[i]);
            }
        }
        Vector<MusicOutputDesign> temp = null;
        if(!v.elementAt(0).startsWith("sharedLibraryTable-"))
        {
            temp = (Vector<MusicOutputDesign>) allMusic.clone();
        }
        else
        {
            temp = (Vector<MusicOutputDesign>) sharedLibrary.clone();
        }
        Collections.sort(temp, new IDComparator());
        for (int i = 0; i < locations.length; i++) 
        {
            MusicOutputDesign mod = back.findTrack(temp,v.elementAt(locations[i]));
            ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), smileys[Integer.parseInt(mod.getRating())]});
        }
        temp.clear();
        temp = null;
    }

    /*public CustomTableModel populateTableModel(JTable jT, int[] locations) 
    {
        CustomTableModel ctm = (CustomTableModel) jT.getModel();
        /*for(int i = 0; i < locations.length; i++){
        locations[i] = jT.convertRowIndexToModel(locations[i]);
        }
        if (ctm.getColumnCount() == 0) 
        {
            for (int i = 0; i < columnNames.length; i++) 
            {
                ctm.addColumn(columnNames[i]);
            }
        }
        for (int i = 0; i < locations.length; i++) 
        {
            MusicOutputDesign mod;
            if (jT.getName().equalsIgnoreCase("playlistTable")) 
            {
                mod = back.findTrack(playList.elementAt(locations[i]));
            } 
            else if (jT.getName().equalsIgnoreCase("CurrentlyPlayingTable")) 
            {
                if(locations[i] >= 0/*!currentlyPlaying2.elementAt(locations[i]).startsWith("sharedLibraryTable-"))
                {
                    mod = allMusic.elementAt(locations[i]);
                }
                else
                {
                    mod = sharedLibrary.elementAt((locations[i]));
                }
                System.out.println("Artist: " + mod.getAlbum() + "\tTrackTitle: " + mod.getTrackTitle());
            } 
            else if(jT.getName().equalsIgnoreCase("sharedLibraryTable"))
            {
                mod = sharedLibrary.elementAt((locations[i]));
            }
            else 
            {
                mod = allMusic.elementAt(locations[i]);
            }
            //System.out.println("Track title: " + mod.getTrackTitle());
            ctm.addRow(new Object[]{mod.getTrackNumber(), mod.getTrackTitle(), mod.getArtist(), mod.getAlbum(),
                        mod.getTrackLength(), mod.getPlayCount(), smileys[Integer.parseInt(mod.getRating())]});
        }
        return ctm;
    }*/

    /**
     * Sorts a table using a comparator.
     * @param jT
     * @param b
     * @param v
     * @param c
     * @return
     */
    /*public boolean tableSort(JTable jT, boolean b, Vector<MusicOutputDesign> v, Comparator c) {
    previousComparator = c;
    if (b == false) {
    Collections.sort(v, c);
    updateTable(v, jT);
    return true;
    } else {
    Comparator reverseC = Collections.reverseOrder(c);
    Collections.sort(v, reverseC);
    updateTable(v, jT);
    return false;
    }
    }

    public boolean tableSort(JTable jT, boolean b, Vector<MusicOutputDesign> v, Comparator c, String flag) {
    previousComparator = c;
    if (b == false) {
    Collections.sort(v, c);
    updateTable(v, jT, "");
    return true;
    } else {
    Comparator reverseC = Collections.reverseOrder(c);
    Collections.sort(v, reverseC);
    updateTable(v, jT, "");
    return false;
    }
    }*/
    /**
     * Sorts a table using the last comparator used.
     * @param jT
     * @param v
     * @return true Always returns true.
     */
    /* public boolean tableSort(JTable jT, Vector<MusicOutputDesign> v) {
    if (previousComparator == null) {
    previousComparator = new ArtistComparator();
    }

    Collections.sort(v, previousComparator);
    updateTable(v, jT, "flag");
    return true;
    }*/
    /**
     * Updates an entire table
     * @param v Vector to update the table.
     * @param jT Table to be updated.
     */
    /*public void updateTable(Vector<MusicOutputDesign> v, JTable jT) {
    DefaultTableModel tM = (DefaultTableModel) jT.getModel();
    tM.setRowCount(v.size());
    for (int i = 0; i < v.size(); i++) {
    jT.setValueAt(v.elementAt(i).trackNumber, i, jT.getColumnModel().getColumnIndex(columnNames[0]));
    jT.setValueAt(v.elementAt(i).trackTitle, i, jT.getColumnModel().getColumnIndex(columnNames[1]));
    jT.setValueAt(v.elementAt(i).artist, i, jT.getColumnModel().getColumnIndex(columnNames[2]));
    jT.setValueAt(v.elementAt(i).album, i, jT.getColumnModel().getColumnIndex(columnNames[3]));
    jT.setValueAt(v.elementAt(i).trackLength, i, jT.getColumnModel().getColumnIndex(columnNames[4]));
    jT.setValueAt(v.elementAt(i).playCount, i, jT.getColumnModel().getColumnIndex(columnNames[5]));
    jT.setValueAt(smileys[Integer.parseInt(v.elementAt(i).rating)], i, jT.getColumnModel().getColumnIndex(columnNames[6]));
    }
    jT.setModel(tM);
    }*/
    /**
     * version of updating a Table
     * @param v Vector to update table
     * @param jT Table to be updated
     */
    /*public void updateTable(Vector<MusicOutputDesign> v, JTable jT, String flag) {
    CustomTableModel tM = (CustomTableModel) jT.getModel();
    tM.setRowCount(v.size());
    for (int i = 0; i < v.size(); i++) {
    jT.setValueAt(v.elementAt(i).trackNumber, i, jT.getColumnModel().getColumnIndex(columnNames[0]));
    jT.setValueAt(v.elementAt(i).trackTitle, i, jT.getColumnModel().getColumnIndex(columnNames[1]));
    jT.setValueAt(v.elementAt(i).artist, i, jT.getColumnModel().getColumnIndex(columnNames[2]));
    jT.setValueAt(v.elementAt(i).album, i, jT.getColumnModel().getColumnIndex(columnNames[3]));
    jT.setValueAt(v.elementAt(i).trackLength, i, jT.getColumnModel().getColumnIndex(columnNames[4]));
    jT.setValueAt(v.elementAt(i).playCount, i, jT.getColumnModel().getColumnIndex(columnNames[5]));
    jT.setValueAt(smileys[Integer.parseInt(v.elementAt(i).rating)], i, jT.getColumnModel().getColumnIndex(columnNames[6]));
    }
    jT.setModel(tM);
    }*/
    /**
     * Appends the currently selected songs from a table to the playlist.
     * @param v Vector associated with the table.
     * @param jT Table that is currently selected.
     */
    @SuppressWarnings("static-access")
    public void appendToPlayList(Vector<MusicOutputDesign> v, JTable jT) 
    {
        int[] location = jT.getSelectedRows();
        String name = jT.getName();
        CustomTableModel ctm = (CustomTableModel)currentlyPlayingTable.getModel();
        for (int i = 0; i < location.length; i++) 
        {
            location[i] = jT.convertRowIndexToModel(location[i]);
            if(name.equals("sharedLibraryTable"))
            {
                currentlyPlaying2.add("sharedLibraryTable-"+v.elementAt(location[i]).getId());
                
                ctm.addRow(new Object[]{v.elementAt(location[i]).getTrackNumber(), v.elementAt(location[i]).getTrackTitle(), v.elementAt(location[i]).getArtist(), v.elementAt(location[i]).getAlbum(),
                        v.elementAt(location[i]).getTrackLength(), v.elementAt(location[i]).getPlayCount(), smileys[Integer.parseInt(v.elementAt(location[i]).getRating())]});
                //location[i] = (location[i] + 1)*-1;
                //currentlyPlaying.add(location[i]);
            }
            else
            {
                //currentlyPlaying.add(location[i]);
                currentlyPlaying2.add(v.elementAt(location[i]).getId());
                back.db.insertIntoCurrentlyPlaying(currentlyPlaying2.size()-1, v.elementAt(location[i]).getId());
                ctm.addRow(new Object[]{v.elementAt(location[i]).getTrackNumber(), v.elementAt(location[i]).getTrackTitle(), v.elementAt(location[i]).getArtist(), v.elementAt(location[i]).getAlbum(),
                        v.elementAt(location[i]).getTrackLength(), v.elementAt(location[i]).getPlayCount(), smileys[Integer.parseInt(v.elementAt(location[i]).getRating())]});
            }
            //location[i] = currentlyPlaying2.size()-1;
        }
        /*Vector<MusicOutputDesign> v1 = new Vector<MusicOutputDesign>();
        for (int i = 0; i < location.length; i++) {
        v1.add(v.elementAt(location[i]));
        }
        currentlyPlaying.addAll(v1);
        updateTable(currentlyPlaying, currentlyPlayingTable);*/
        //currentlyPlayingTable.setModel(populateTableModel(currentlyPlayingTable, location));
        back.calculatePlaylistTime();
        if (playing == false) 
        {
            play();
        }
        jT.clearSelection();
    }
    //TODO: FIXME

    public void appendToPlayList(Vector<String> v, JTable jT, boolean flag) 
    {
        int[] location = jT.getSelectedRows();
        for (int i = 0; i < location.length; i++) 
        {
            location[i] = jT.convertRowIndexToModel(location[i]);
            currentlyPlaying2.add(v.elementAt(location[i]));
            back.db.insertIntoCurrentlyPlaying(currentlyPlaying2.size()-1, v.elementAt(location[i]));
        }
        populateTableModel((CustomTableModel) currentlyPlayingTable.getModel(), v, location);
        back.calculatePlaylistTime();
        if (playing == false) 
        {
            play();
        }
        jT.clearSelection();
    }

    /**
     *
     * @param v
     */
    @SuppressWarnings("static-access")
    public void appendToPlayList(Vector<MusicOutputDesign> v) 
    {
        for (int i = 0; i < v.size(); i++) 
        {
            //currentlyPlaying.add(allMusic.indexOf(v.elementAt(i)));
            currentlyPlaying2.add(v.elementAt(i).getId());
            back.db.insertIntoCurrentlyPlaying(currentlyPlaying2.size(), v.elementAt(i).getId());
        }
        ((CustomTableModel) currentlyPlayingTable.getModel()).removeAllRows();
        //populateTableModel((CustomTableModel) currentlyPlayingTable.getModel(), currentlyPlaying);
        populateTableModel2((CustomTableModel) currentlyPlayingTable.getModel(), currentlyPlaying2);
        if (!SwingUtilities.isEventDispatchThread()) 
        {
            try 
            {
                SwingUtilities.invokeAndWait(new Runnable() 
                {
                    public void run() 
                    {
                        ((CustomTableModel) currentlyPlayingTable.getModel()).fireTableDataChanged();
                    }
                });
            } 
            catch (InterruptedException ex) 
            {
                showErrorDialog(ex);
            } 
            catch (InvocationTargetException ex) 
            {
                showErrorDialog(ex);
            }
        } 
        else 
        {
            ((CustomTableModel) currentlyPlayingTable.getModel()).fireTableDataChanged();
        }

        back.calculatePlaylistTime();
        if (playing == false) 
        {
            playButton.doClick();
        }
    }

    /**
     * Dumps all the available Mixers and whatnot to find just the right line
     * for volume control.  Needs work.
     */
    private boolean dumpMixers() 
    {
        if (AudioSystem.isLineSupported(Port.Info.SPEAKER)) 
        {
            try 
            {
                Port speakerLine = (Port) AudioSystem.getLine(Port.Info.SPEAKER);
                speakerLine.open();
                if (speakerLine.isControlSupported(FloatControl.Type.VOLUME)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.VOLUME);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;
                } 
                else if (speakerLine.isControlSupported(FloatControl.Type.MASTER_GAIN)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.MASTER_GAIN);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;
                }
            } 
            catch (LineUnavailableException ex) 
            {
                //ex.printStackTrace();
                showErrorDialog(ex);
            }
        } 
        else if (AudioSystem.isLineSupported(Port.Info.LINE_OUT)) 
        {
            try 
            {
                Port speakerLine = (Port) AudioSystem.getLine(Port.Info.LINE_OUT);
                speakerLine.open();
                if (speakerLine.isControlSupported(FloatControl.Type.VOLUME)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.VOLUME);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;

                } 
                if (speakerLine.isControlSupported(FloatControl.Type.MASTER_GAIN)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.MASTER_GAIN);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;
                }
            } 
            catch (LineUnavailableException ex) 
            {
                //ex.printStackTrace();
                showErrorDialog(ex);
            }
        }
        else if(AudioSystem.isLineSupported(Port.Info.HEADPHONE))
        {
            try 
            {
                Port speakerLine = (Port) AudioSystem.getLine(Port.Info.HEADPHONE);
                if (speakerLine.isControlSupported(FloatControl.Type.VOLUME)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.VOLUME);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;
                } 
                if (speakerLine.isControlSupported(FloatControl.Type.MASTER_GAIN)) 
                {
                    FloatControl fC = (FloatControl) speakerLine.getControl(FloatControl.Type.MASTER_GAIN);
                    FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                    jSlider1.setModel(model);
                    jSlider1.setOrientation(JSlider.HORIZONTAL);
                    jSlider1.setName("slider");
                    jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                    setTickSpacings(jSlider1);
                    jScrollPane7.setViewportView(jSlider1);
                    return true;
                }
            } 
            catch (LineUnavailableException ex) 
            {
                //ex.printStackTrace();
                showErrorDialog(ex);
            }
        }
        for(Mixer.Info mi: AudioSystem.getMixerInfo())
        {
            //System.out.println("mixer name: " + mi.getName());  
            Mixer mixer = AudioSystem.getMixer(mi);
            try 
            {
                mixer.open();
                Line.Info [] lineInfos = mixer.getTargetLineInfo();
                for(Line.Info lineInfo : lineInfos)  
                {
                    if(AudioSystem.isLineSupported(lineInfo))
                    {
                        try 
                        {
                            Line l = AudioSystem.getLine(lineInfo);
                            l.open();

                            if(l.isControlSupported(FloatControl.Type.VOLUME))
                            {
                                FloatControl fC = (FloatControl) l.getControl(FloatControl.Type.VOLUME);
                                FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                                jSlider1.setModel(model);
                                jSlider1.setOrientation(JSlider.HORIZONTAL);
                                jSlider1.setName("slider");
                                jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                                setTickSpacings(jSlider1);
                                jScrollPane7.setViewportView(jSlider1);
                                return true;
                            }
                            else if(l.isControlSupported(FloatControl.Type.MASTER_GAIN))
                            {
                                FloatControl fC = (FloatControl) l.getControl(FloatControl.Type.MASTER_GAIN);
                                FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel(fC);
                                jSlider1.setModel(model);
                                jSlider1.setOrientation(JSlider.HORIZONTAL);
                                jSlider1.setName("slider");
                                jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                                setTickSpacings(jSlider1);
                                jScrollPane7.setViewportView(jSlider1);
                                return true;
                            }

                            else
                            {
                                for(Control c: l.getControls())
                                {
                                    if(c instanceof CompoundControl)
                                    {
                                        CompoundControl cc = (CompoundControl)c;
                                        for(Control mc: cc.getMemberControls())
                                        {
                                            if(mc instanceof FloatControl)
                                            {
                                                FloatControlBoundedRangeModel model = new FloatControlBoundedRangeModel((FloatControl)mc);
                                                jSlider1.setModel(model);
                                                jSlider1.setOrientation(JSlider.HORIZONTAL);
                                                jSlider1.setName("slider");
                                                jSlider1.setToolTipText("Volume: " + jSlider1.getValue() + "%");
                                                setTickSpacings(jSlider1);
                                                jScrollPane7.setViewportView(jSlider1);
                                                return true;
                                            }
                                        }
                                    }
                                }
                            }
                            l.close();
                        }
                        catch (LineUnavailableException ex) 
                        {
                            showErrorDialog(ex);
                        }
                    }
                }
                mixer.close();
            } 
            catch (LineUnavailableException ex) 
            {
                showErrorDialog(ex);
            }
        }
        return false;
    }

    /**
     *
     * @param slider
     */
    public void setTickSpacings(JSlider slider) 
    {
        int nRange = slider.getMaximum() - slider.getMinimum();
        if (nRange < 10) 
        {
            slider.setMajorTickSpacing(1);
        } 
        else if (nRange < 50) 
        {
            slider.setMajorTickSpacing(5);
            slider.setMinorTickSpacing(1);
        } 
        else if (nRange < 100) 
        {
            slider.setMajorTickSpacing(10);
            slider.setMinorTickSpacing(5);
        } 
        else if (nRange < 200) 
        {
            slider.setMajorTickSpacing(20);
            slider.setMinorTickSpacing(5);
        } 
        else if (nRange < 500) 
        {
            slider.setMajorTickSpacing(50);
            slider.setMinorTickSpacing(10);
        } 
        else 
        {
            slider.setMajorTickSpacing(100);
            // slider.setMinorTickSpacing(100);
        }
    }

    /**
     * Finds all the albums by an artist and sets the list model for jList2
     * @param artist Artist to search
     */
    private void getAlbumsByArtist(String artist) 
    {
        Vector<String> aList = new Vector<String>();
        aList.add("(All Albums)");
        for (int i = 0; i < allMusic.size(); i++) 
        {
            if (allMusic.elementAt(i).getArtist().equalsIgnoreCase(artist) || artist.equalsIgnoreCase("(All Artists)")) 
            {
                if (!aList.contains(allMusic.elementAt(i).getAlbum())) 
                {
                    aList.add(allMusic.elementAt(i).getAlbum());
                }
            }
        }

        Collections.sort(aList);
        DefaultListModel dlm = new DefaultListModel();
        for (String s: aList) 
        {
            dlm.addElement(s);
        }
        jList2.setModel(dlm);
    }

    /**
     * Finds the artist associated with an album and sets the list model for jList1
     * @param album
     */
    private void getArtistByAlbum(String album) 
    {
        Vector<String> aList = new Vector<String>();
        aList.add("(All Artists)");
        for (int i = 0; i < allMusic.size(); i++) 
        {
            if (allMusic.elementAt(i).getAlbum().equalsIgnoreCase(album) || album.equalsIgnoreCase("(All Albums)")) 
            {
                if (!aList.contains(allMusic.elementAt(i).getArtist())) 
                {
                    aList.add(allMusic.elementAt(i).getArtist());
                }
            }
        }

        Collections.sort(aList);
        DefaultListModel dlm = new DefaultListModel();
        for (String s: aList) 
        {
            dlm.addElement(s);
        }
        jList1.setModel(dlm);
    }

    /**
     * Finds all the songs in a specific albums
     * @param album Album to search
     * @return temp Vector of all the songs in a specific album
     */
    private Vector<MusicOutputDesign> getTracksFromAlbum(String album) 
    {
        Vector<MusicOutputDesign> temp = new Vector<MusicOutputDesign>();
        for (int i = 0; i < allMusic.size(); i++) 
        {
            if (allMusic.elementAt(i).getAlbum().compareToIgnoreCase(album) == 0) 
            {
                temp.add(allMusic.elementAt(i));
            }
        }
        return temp;
    }

    private void createNewPlayList() 
    {
        String name = JOptionPane.showInputDialog("What would you like to name your new playlist?", "");
        if (name != null && name.length() != 0) 
        {
            try 
            {
                if (name.contains(" ")) 
                {
                    name = name.replace(" ", "_");
                }
                java.sql.Statement stmt = back.db.getConnection().createStatement();
                db.addToPlayListTable(name,"NoFile");
                db.createTable(stmt, name);
                stmt.close();
            } 
            catch (SQLException ex) 
            {
                //ex.printStackTrace();
                showErrorDialog(ex);
            }
            Vector<String> pL = getAllPlayLists();
            DefaultListModel dlm2 = new DefaultListModel();
            jMenu2.removeAll();
            jMenu3.removeAll();
            for (int i = 0; i < pL.size(); i++) {
                dlm2.addElement(pL.elementAt(i));
                final JMenuItem jmi = new JMenuItem(pL.elementAt(i));
                jmi.addActionListener(new java.awt.event.ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent evt) {
                        addSongsToPlaylist(jmi.getText(), currentlyShowingTable, allMusic/*currentlyShowing*/);
                    }
                });
                jMenu2.add(jmi);
                final JMenuItem jmi1 = new JMenuItem(pL.elementAt(i));
                jmi1.addActionListener(new java.awt.event.ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent evt) {
                        //addSongsToPlaylist(jmi1.getText(), currentlyPlayingTable, currentlyPlaying);
                    }
                });
                jMenu3.add(jmi1);
            }
            pL.clear();
            jList3.setModel(dlm2);
        }
    }

    private void deletePlaylist(String name) {
        try {
            if (name.contains(" ")) {
                name.replace(" ", "_");
            }
            java.sql.Statement stmt = back.db.getConnection().createStatement();
            db.deleteFromPlaylistTable(stmt, name);
            db.deleteEntirePlaylist(stmt, name);
            jMenu2.remove(jList3.getSelectedIndex());
            jMenu3.remove(jList3.getSelectedIndex());
            stmt.close();
        } catch (SQLException ex) {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
        Vector<String> pL = getAllPlayLists();
        DefaultListModel dlm2 = new DefaultListModel();
        for (int i = 0; i < pL.size(); i++) {
            dlm2.addElement(pL.elementAt(i));
        }
        pL.clear();
        jList3.setModel(dlm2);
    }

    /**
     *
     * @param name
     * @param jT
     * @param v
     */
    public void addSongsToPlaylist(String name, JTable jT, Vector<MusicOutputDesign> v) 
    {
        try 
        {
            if (name.contains(" ")) 
            {
                name.replace(" ", "_");
            }
            int[] locations = jT.getSelectedRows();
            for (int i = 0; i < locations.length; i++) 
            {
                locations[i] = jT.convertRowIndexToModel(locations[i]);
            }
            java.sql.Statement stmt = back.db.getConnection().createStatement();
            for (int i = 0; i < locations.length; i++) 
            {
                db.addToPlayList(stmt, name, v.elementAt(locations[i]).getId());
            }
            stmt.close();
        } catch (SQLException ex) {
            //ex.printStackTrace();
            showErrorDialog(ex);
        }
    }
    
    public SocketListener getSocketListener()
    {
        return sl;
    }
    
    public int getJList4PreviousSelection()
    {
        return previousSelection;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        playButton = new JButton();
        pauseButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        jProgressBar1 = new javax.swing.JProgressBar();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane8 = new javax.swing.JScrollPane();
        dlm1 = new DefaultListModel();
        dlm1.addElement("(All Albums)");
        try
        {
            Object[] albums = db.selectDistinct(db.getConnection().createStatement(), "album");
            for(Object o: albums)
            {
                dlm1.addElement(o);
            }
        }catch(SQLException ex)
        {
            showErrorDialog(ex);
        }
        /*String previousAlbum = "awesome";
        for(int i = 0; i < allMusic.size(); i++){
            if(i - 1 >= 0)
            previousAlbum = allMusic.elementAt(i-1).getAlbum();
            if(previousAlbum.compareToIgnoreCase(allMusic.elementAt(i).getAlbum()) != 0 && !dlm1.contains(allMusic.elementAt(i).getAlbum()))
            dlm1.addElement(allMusic.elementAt(i).getAlbum());
        }*/
        jList2 = new javax.swing.JList();
        jScrollPane10 = new javax.swing.JScrollPane();
        DefaultListModel dlm = new DefaultListModel();

        dlm.addElement("(All Artists)");
        try
        {
            Object[] array = db.selectDistinct(db.getConnection().createStatement(), "artist");
            for(Object o: array)
            {
                dlm.addElement(o);
            }
        }catch(SQLException ex)
        {
            showErrorDialog(ex);
        }
        //String previousArtist = "awesome";
        //for(int i = 0; i < allMusic.size(); i++){
            //    if(i - 1 >= 0)
            //        previousArtist = allMusic.elementAt(i-1).getArtist();
            //    if(previousArtist.compareToIgnoreCase(allMusic.elementAt(i).getArtist()) != 0 && !dlm.contains(allMusic.elementAt(i).getArtist()))
            //        dlm.addElement(allMusic.elementAt(i).getArtist());
            //}
        jList1 = new javax.swing.JList();
        jScrollPane6 = new javax.swing.JScrollPane();
        CustomTableModel ctm = new CustomTableModel();
        currentlyShowingTable = new musicbox.gui.CustomTable(populateTableModel(ctm));
        jScrollPane3 = new javax.swing.JScrollPane();
        CustomTableModel ctm1 = new CustomTableModel();
        currentlyPlayingTable = new musicbox.gui.CustomTable(ctm1);
        jScrollPane5 = new javax.swing.JScrollPane();
        jEditorPane1 = new javax.swing.JEditorPane();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jMenu2 = new JMenu("Add song to playlist");
        jMenu3 = new JMenu("Add song to playlist");
        Vector<String> pL = getAllPlayLists();
        DefaultListModel dlm2 = new DefaultListModel();
        for(int i = 0; i < pL.size(); i++){
            dlm2.addElement(pL.elementAt(i));
            final JMenuItem jmi = new JMenuItem(pL.elementAt(i));
            jmi.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    addSongsToPlaylist(jmi.getText(), currentlyShowingTable, allMusic);
                }
            });
            jMenu2.add(jmi);
            final JMenuItem jmi1 = new JMenuItem(pL.elementAt(i));
            jmi1.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    addSongsToPlaylist(jmi1.getText(), currentlyPlayingTable, allMusic);
                }
            });
            jMenu3.add(jmi1);
        }
        pL.clear();
        jList3 = new javax.swing.JList();
        jScrollPane4 = new javax.swing.JScrollPane();
        CustomTableModel ctm2 = new CustomTableModel();
        playlistTable = new CustomTable(ctm2);
        jTabbedPane2 = new javax.swing.JTabbedPane();
        jScrollPane9 = new javax.swing.JScrollPane();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane11 = new javax.swing.JScrollPane();
        jPanel4 = new javax.swing.JPanel();
        jPanel5 = new javax.swing.JPanel();
        jScrollPane13 = new javax.swing.JScrollPane();
        addressList = new javax.swing.JList();
        jScrollPane15 = new javax.swing.JScrollPane();
        CustomTableModel sharedLibraryModel = new CustomTableModel();
        sharedLibraryTable = new CustomTable(sharedLibraryModel);
        jScrollPane12 = new javax.swing.JScrollPane();
        sharedPlayList = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        albumArtLabel = new javax.swing.JLabel();
        shuffleButton = new javax.swing.JButton();
        jScrollPane7 = new javax.swing.JScrollPane();
        jSlider1 = new javax.swing.JSlider()
        {
            public String getToolTipText(MouseEvent event)
            {
                if(getValue() != 0 && getMaximum() != 0)
                return "Volume: " + getValue() + "%";
                else
                return "Volume: 0%";
            }
        };
        previousTrackButton = new javax.swing.JButton();
        nextTrackButton = new javax.swing.JButton();
        ratingLabel = new javax.swing.JLabel();
        jButton7 = new javax.swing.JButton();
        jButton8 = new javax.swing.JButton();
        searchTextField = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        currentTrackInfoBox = new javax.swing.JTextArea();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem8 = new javax.swing.JMenuItem();
        jMenuItem2 = new javax.swing.JMenuItem();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem6 = new javax.swing.JMenuItem();
        jMenuItem4 = new javax.swing.JMenuItem();
        jMenuItem9 = new javax.swing.JMenuItem();
        jMenuItem7 = new javax.swing.JMenuItem();
        jMenuItem5 = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        PlayingPopupMenu = new javax.swing.JPopupMenu();
        ShowingPopupMenu = new javax.swing.JPopupMenu();
        jPopupMenu3 = new javax.swing.JPopupMenu();
        PlaylistPopupMenu = new javax.swing.JPopupMenu();
        SharedLibraryPopupMenu = new javax.swing.JPopupMenu();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                mainPanelComponentResized(evt);
            }
        });

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getResourceMap(MusicBoxView.class);
        playButton.setText(resourceMap.getString("playButton.text")); // NOI18N
        playButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        playButton.setName("playButton"); // NOI18N

        pauseButton.setText(resourceMap.getString("pauseButton.text")); // NOI18N
        pauseButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        pauseButton.setName("pauseButton"); // NOI18N

        stopButton.setText(resourceMap.getString("stopButton.text")); // NOI18N
        stopButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        stopButton.setName("stopButton"); // NOI18N

        jProgressBar1.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        jProgressBar1.setName("jProgressBar1"); // NOI18N
        jProgressBar1.setStringPainted(true);
        jProgressBar1.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                jProgressBar1MouseDragged(evt);
            }
        });

        jTabbedPane1.setTabLayoutPolicy(javax.swing.JTabbedPane.SCROLL_TAB_LAYOUT);
        jTabbedPane1.setAutoscrolls(true);
        jTabbedPane1.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        jTabbedPane1.setMinimumSize(new java.awt.Dimension(100, 100));
        jTabbedPane1.setName("jTabbedPane1"); // NOI18N
        jTabbedPane1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jTabbedPane1MouseClicked(evt);
            }
        });

        jPanel1.setName("jPanel1"); // NOI18N

        jScrollPane8.setName("jScrollPane8"); // NOI18N

        dlm1 = sortListModel(dlm1);
        jList2.setModel(dlm1);
        jList2.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        jList2.setName("jList2"); // NOI18N
        jScrollPane8.setViewportView(jList2);
        jList2.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent e){
                if(jList2.getSelectedIndex() != -1){
                    String album = (String)jList2.getSelectedValue();
                    if(album != null){
                        if(album.compareTo("(All Albums)") == 0){
                            //currentlyShowing = (Vector<MusicOutputDesign>)allMusic.clone();
                            //Collections.sort(currentlyShowing, new ArtistComparator());
                            //updateTable(currentlyShowing, currentlyShowingTable);
                            getArtistByAlbum(album);
                            getAlbumsByArtist("(All Artists)");
                            album = "";
                        }
                        if(!album.equals("")){
                            getArtistByAlbum(album);
                        }
                        RowFilter<CustomTableModel, Object> rf = regexSearch(album,3);
                        TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)currentlyShowingTable.getRowSorter();
                        sorter.setRowFilter(rf);
                        currentlyShowingTable.setRowSorter(sorter);
                    }

                }
            }
        });

        jScrollPane10.setName("jScrollPane10"); // NOI18N

        dlm = sortListModel(dlm);
        jList1.setModel(dlm);
        jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        jList1.setName("jList1"); // NOI18N
        jScrollPane10.setViewportView(jList1);
        jList1.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent e)
                {
                    String artist = (String)jList1.getSelectedValue();
                    if(artist != null && !artist.equals(""))
                    {
                        if(artist.compareToIgnoreCase("(All Artists)") == 0)
                        {
                            //currentlyShowing = (Vector<MusicOutputDesign>)allMusic.clone();
                            //Collections.sort(currentlyShowing, new ArtistComparator());
                            //updateTable(currentlyShowing, currentlyShowingTable);
                            //                if(!jList2.getModel().equals(dlm1))
                            //                jList2.setModel(dlm1);
                            getAlbumsByArtist(artist);
                            getArtistByAlbum("(All Albums)");
                            artist = "";
                        }
                        if(!artist.equals(""))
                        {
                            getAlbumsByArtist(artist);
                        }
                        //Collections.sort(currentlyShowing, new AlbumComparator());
                        //updateTable(currentlyShowing, currentlyShowingTable);
                        RowFilter<CustomTableModel, Object> rf = regexSearch(artist,2);
                        TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)currentlyShowingTable.getRowSorter();
                        sorter.setRowFilter(rf);
                        currentlyShowingTable.setRowSorter(sorter);
                    }
                }
            });

            jScrollPane6.setName("jScrollPane6"); // NOI18N

            //currentlyShowingTable.createDefaultColumnsFromModel();
            currentlyShowingTable.setName("CurrentlyShowing");
            currentlyShowingTable.setFillsViewportHeight(true);
            currentlyShowingTable.setName("CurrentlyShowingTable"); // NOI18N
            currentlyShowingTable.setRowMargin(0);
            currentlyShowingTable.setFont(projectFont);
            jScrollPane6.setViewportView(currentlyShowingTable);
            //currentlyShowing = (Vector<MusicOutputDesign>)allMusic.clone();
            //updateTable(currentlyShowing, currentlyShowingTable);
            currentlyShowingTable.setRowHeight(16);
            TableRowSorter<CustomTableModel> sorter = new TableRowSorter<CustomTableModel>(ctm);
            sorter.setComparator(ctm.findColumn("Track Number"), new TrackNumberStringComparator());
            sorter.setComparator(ctm.findColumn("Artist"), new ArtistStringComparator());
            sorter.setComparator(ctm.findColumn("Album"), new AlbumStringComparator());
            sorter.setComparator(ctm.findColumn("Play Count"), new PlayCountStringComparator());
            currentlyShowingTable.setRowSorter(sorter);
            currentlyShowingTable.addKeyListener(ctl);
            currentlyShowingTable.addMouseListener(ctl);
            currentlyShowingTable.addMouseMotionListener(ctl);

            javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
            jPanel1.setLayout(jPanel1Layout);
            jPanel1Layout.setHorizontalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel1Layout.createSequentialGroup()
                    .addComponent(jScrollPane10, javax.swing.GroupLayout.DEFAULT_SIZE, 308, Short.MAX_VALUE)
                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                    .addComponent(jScrollPane8, javax.swing.GroupLayout.DEFAULT_SIZE, 325, Short.MAX_VALUE))
                .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 639, Short.MAX_VALUE)
            );
            jPanel1Layout.setVerticalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel1Layout.createSequentialGroup()
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jScrollPane8, javax.swing.GroupLayout.DEFAULT_SIZE, 156, Short.MAX_VALUE)
                        .addComponent(jScrollPane10, javax.swing.GroupLayout.DEFAULT_SIZE, 156, Short.MAX_VALUE))
                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                    .addComponent(jScrollPane6, javax.swing.GroupLayout.DEFAULT_SIZE, 355, Short.MAX_VALUE))
            );

            jTabbedPane1.addTab(resourceMap.getString("jPanel1.TabConstraints.tabTitle"), jPanel1); // NOI18N

            jScrollPane3.setName("jScrollPane3"); // NOI18N

            currentlyPlayingTable.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
            currentlyPlayingTable.setName("CurrentlyPlayingTable"); // NOI18N
            jScrollPane3.setViewportView(currentlyPlayingTable);
            currentlyPlayingTable.setRowHeight(16);
            TableRowSorter<CustomTableModel> sorter1 = new TableRowSorter<CustomTableModel>(ctm1);
            /*sorter1.setComparator(ctm.findColumn("Track Number"), new TrackNumberStringComparator());
            sorter1.setComparator(ctm.findColumn("Artist"), new ArtistStringComparator());
            sorter1.setComparator(ctm.findColumn("Album"), new AlbumStringComparator());
            */
            currentlyPlayingTable.setRowSorter(sorter1);
            currentlyPlayingTable.addKeyListener(ctl);
            currentlyPlayingTable.addMouseListener(ctl);
            currentlyPlayingTable.addMouseMotionListener(ctl);

            jTabbedPane1.addTab(resourceMap.getString("jScrollPane3.TabConstraints.tabTitle"), jScrollPane3); // NOI18N

            jScrollPane5.setAutoscrolls(true);
            jScrollPane5.setName("jScrollPane5"); // NOI18N

            jEditorPane1.setContentType(resourceMap.getString("jEditorPane1.contentType")); // NOI18N
            jEditorPane1.setEditable(false);
            jEditorPane1.setFont(resourceMap.getFont("jEditorPane1.font")); // NOI18N
            jEditorPane1.setName("jEditorPane1"); // NOI18N
            jScrollPane5.setViewportView(jEditorPane1);
            jEditorPane1.addHyperlinkListener(this);

            if(internetConnection){
                java.awt.EventQueue.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        try{
                            jEditorPane1.setPage("http://www.wikipedia.org");
                        }catch(Exception ex4){//ex4.printStackTrace();
                            showErrorDialog(ex4);
                        }
                    }
                });

            }

            jTabbedPane1.addTab(resourceMap.getString("jScrollPane5.TabConstraints.tabTitle"), jScrollPane5); // NOI18N

            jPanel2.setName("jPanel2"); // NOI18N

            jScrollPane1.setName("jScrollPane1"); // NOI18N

            jList3.setModel(dlm2);
            jList3.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
            jList3.setName("jList3"); // NOI18N
            jScrollPane1.setViewportView(jList3);
            jList3.addListSelectionListener(new ListSelectionListener(){
                public void valueChanged(ListSelectionEvent e){
                    if(jList3.getSelectedIndex() >= 0){

                        String name = (String)jList3.getSelectedValue();
                        if(previousList.compareToIgnoreCase(name) != 0){
                            ((CustomTableModel)playlistTable.getModel()).setRowCount(0);
                            playList.clear();
                            previousList = name;
                            Vector<String> temp = new Vector<String>();
                            try {
                                java.sql.Statement stmt = back.db.getConnection().createStatement();
                                temp =  db.listPlayList(stmt, name);
                                stmt.close();
                            } catch(SQLException ex){
                                //ex.printStackTrace();
                                showErrorDialog(ex);
                            }
                            findAllTracks(temp);
                            temp = null;
                            //TODO:
                            //Collections.sort(playList, new ArtistComparator());
                            //updateTable(playList, playlistTable);
                            //populateTableModel((CustomTableModel)playlistTable.getModel(), playList);
                        }

                    }
                }
            });
            jList3.addMouseListener(new MouseAdapter(){
                public void mouseClicked(MouseEvent e){
                    if(e.getButton() == MouseEvent.BUTTON3){
                        jPopupMenu3.show(e.getComponent(), e.getX(), e.getY());
                    }
                }
            });

            jScrollPane4.setName("jScrollPane4"); // NOI18N

            playlistTable.setName("playlistTable"); // NOI18N
            jScrollPane4.setViewportView(playlistTable);
            playlistTable.setRowHeight(16);
            TableRowSorter<CustomTableModel> sorter2 = new TableRowSorter<CustomTableModel>(ctm2);
            /*sorter2.setComparator(ctm.findColumn("Track Number"), new TrackNumberStringComparator());
            sorter2.setComparator(ctm.findColumn("Artist"), new ArtistStringComparator());
            sorter2.setComparator(ctm.findColumn("Album"), new AlbumStringComparator());
            */
            playlistTable.setRowSorter(sorter2);
            playlistTable.addKeyListener(ctl);
            playlistTable.addMouseListener(ctl);
            playlistTable.addMouseMotionListener(ctl);

            javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
            jPanel2.setLayout(jPanel2Layout);
            jPanel2Layout.setHorizontalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel2Layout.createSequentialGroup()
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 313, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addContainerGap(326, Short.MAX_VALUE))
                .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 639, Short.MAX_VALUE)
            );
            jPanel2Layout.setVerticalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel2Layout.createSequentialGroup()
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                    .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 381, Short.MAX_VALUE))
            );

            jTabbedPane1.addTab(resourceMap.getString("jPanel2.TabConstraints.tabTitle"), jPanel2); // NOI18N

            jTabbedPane2.setTabLayoutPolicy(javax.swing.JTabbedPane.SCROLL_TAB_LAYOUT);
            jTabbedPane2.setTabPlacement(javax.swing.JTabbedPane.LEFT);
            jTabbedPane2.setName("jTabbedPane2"); // NOI18N
            jTabbedPane2.addMouseListener(new java.awt.event.MouseAdapter() {
                public void mouseClicked(java.awt.event.MouseEvent evt) {
                    jTabbedPane2MouseClicked(evt);
                }
            });

            jScrollPane9.setName("jScrollPane9"); // NOI18N

            jPanel3.setName("jPanel3"); // NOI18N
            jPanel3.addAncestorListener(new javax.swing.event.AncestorListener() {
                public void ancestorMoved(javax.swing.event.AncestorEvent evt) {
                    jPanel3AncestorMoved(evt);
                }
                public void ancestorAdded(javax.swing.event.AncestorEvent evt) {
                }
                public void ancestorRemoved(javax.swing.event.AncestorEvent evt) {
                }
            });
            jPanel3.setLayout(new java.awt.GridLayout(1, 0));

            //setAllLibraryImages();
            jPanel3.setLayout(new java.awt.GridLayout(jList1.getModel().getSize()/4,4));
            new Timer().schedule(new SetLibraryImages(), 3000);

            jScrollPane9.setViewportView(jPanel3);
            //setAllLibraryArtistImages();

            jTabbedPane2.addTab(resourceMap.getString("jScrollPane9.TabConstraints.tabTitle"), jScrollPane9); // NOI18N

            jScrollPane11.setName("jScrollPane11"); // NOI18N

            jPanel4.setName("jPanel4"); // NOI18N
            jPanel4.addAncestorListener(new javax.swing.event.AncestorListener() {
                public void ancestorMoved(javax.swing.event.AncestorEvent evt) {
                    jPanel4AncestorMoved(evt);
                }
                public void ancestorAdded(javax.swing.event.AncestorEvent evt) {
                }
                public void ancestorRemoved(javax.swing.event.AncestorEvent evt) {
                }
            });

            javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
            jPanel4.setLayout(jPanel4Layout);
            jPanel4Layout.setHorizontalGroup(
                jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGap(0, 686, Short.MAX_VALUE)
            );
            jPanel4Layout.setVerticalGroup(
                jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGap(0, 500, Short.MAX_VALUE)
            );

            jScrollPane11.setViewportView(jPanel4);
            jPanel4.setLayout(new java.awt.GridLayout(jList2.getModel().getSize()/4,4));
            //setAllLibraryAlbumImages();

            jTabbedPane2.addTab(resourceMap.getString("jScrollPane11.TabConstraints.tabTitle"), jScrollPane11); // NOI18N

            jTabbedPane1.addTab(resourceMap.getString("jTabbedPane2.TabConstraints.tabTitle"), jTabbedPane2); // NOI18N

            jPanel5.setName("jPanel5"); // NOI18N

            jScrollPane13.setName("jScrollPane13"); // NOI18N

            addressList.setModel(new DefaultListModel());
            addressList.setName("addressList"); // NOI18N
            jScrollPane13.setViewportView(addressList);
            addressList.addListSelectionListener(new ListSelectionListener(){
                public void valueChanged(ListSelectionEvent e)
                {
                    if(addressList.getSelectedIndex() >= 0 && addressList.getSelectedIndex() != previousSelection)
                    {
                        previousSelection = addressList.getSelectedIndex();
                        ((CustomTableModel)sharedLibraryTable.getModel()).removeAllRows();
                        SocketObject so = new SocketObject(null, "GetLibrary", "N/A", socket.getLocalAddress().getHostAddress(), ((String)addressList.getSelectedValue()), null);
                        (new SocketWriteThread(sl.getObjectOutputStream(), so, p)).start();
                        /*try
                        {
                            Thread.sleep(10000);
                        }
                        catch(Exception ex)
                        {
                            showErrorDialog(ex);
                        }*/
                        so = new SocketObject(null, "GetAllPlayLists", "N/A", socket.getLocalAddress().getHostAddress(), ((String)addressList.getSelectedValue()), null);
                        (new SocketWriteThread(sl.getObjectOutputStream(), so, p)).start();
                        /*try
                        {
                            sl.getObjectOutputStream().writeObject(so);
                            sl.getObjectOutputStream().flush();
                        }
                        catch(IOException ex)
                        {
                            showErrorDialog(ex);
                        }*/
                    }
                }
            });

            jScrollPane15.setName("jScrollPane15"); // NOI18N

            sharedLibraryTable.setName("sharedLibraryTable"); // NOI18N
            jScrollPane15.setViewportView(sharedLibraryTable);
            sharedLibraryTable.setRowHeight(16);
            TableRowSorter<CustomTableModel> sorter3 = new TableRowSorter<CustomTableModel>(sharedLibraryModel);
            /*sorter3.setComparator(ctm2.findColumn("Track Number"), new TrackNumberStringComparator());
            sorter3.setComparator(ctm2.findColumn("Artist"), new ArtistStringComparator());
            sorter3.setComparator(ctm2.findColumn("Album"), new AlbumStringComparator());*/
            sharedLibraryTable.setRowSorter(sorter3);
            //sharedLibraryTable.addKeyListener(ctl);
            sharedLibraryTable.addMouseListener(ctl);
            sharedLibraryTable.addMouseMotionListener(ctl);

            jScrollPane12.setName("jScrollPane12"); // NOI18N

            sharedPlayList.setModel(new DefaultListModel());
            sharedPlayList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
            sharedPlayList.setName("sharedPlayList"); // NOI18N
            jScrollPane12.setViewportView(sharedPlayList);
            jScrollPane12.setVisible(false);

            sharedPlayList.addListSelectionListener(new ListSelectionListener()
                {
                    public void valueChanged(ListSelectionEvent e)
                    {
                        if(e.getValueIsAdjusting())
                        {
                            previousSelection = addressList.getSelectedIndex();
                            ((CustomTableModel)sharedLibraryTable.getModel()).removeAllRows();
                            SocketObject so = new SocketObject(null, "GetPlaylist", (String)sharedPlayList.getSelectedValue(), socket.getLocalAddress().getHostAddress(), ((String)addressList.getSelectedValue()), null);
                            (new SocketWriteThread(sl.getObjectOutputStream(), so, p)).start();
                            /*try
                            {
                                sl.getObjectOutputStream().writeObject(so);
                                sl.getObjectOutputStream().flush();
                            }
                            catch(IOException ex)
                            {
                                showErrorDialog(ex);
                            }*/
                        }
                    }
                });

                javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
                jPanel5.setLayout(jPanel5Layout);
                jPanel5Layout.setHorizontalGroup(
                    jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane15, javax.swing.GroupLayout.DEFAULT_SIZE, 639, Short.MAX_VALUE)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(jScrollPane13, javax.swing.GroupLayout.PREFERRED_SIZE, 321, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane12, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE))
                );
                jPanel5Layout.setVerticalGroup(
                    jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jScrollPane13)
                            .addComponent(jScrollPane12, javax.swing.GroupLayout.DEFAULT_SIZE, 141, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jScrollPane15, javax.swing.GroupLayout.DEFAULT_SIZE, 370, Short.MAX_VALUE))
                );

                jTabbedPane1.addTab(resourceMap.getString("jPanel5.TabConstraints.tabTitle"), jPanel5); // NOI18N

                jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
                jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
                jLabel1.setName("jLabel1"); // NOI18N
                jLabel1.setOpaque(true);

                jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
                jLabel2.setMaximumSize(new java.awt.Dimension(22, 14));
                jLabel2.setMinimumSize(new java.awt.Dimension(22, 14));
                jLabel2.setName("jLabel2"); // NOI18N
                jLabel2.setPreferredSize(new java.awt.Dimension(22, 14));

                albumArtLabel.setText(resourceMap.getString("albumArtLabel.text")); // NOI18N
                albumArtLabel.setToolTipText(resourceMap.getString("albumArtLabel.toolTipText")); // NOI18N
                albumArtLabel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                albumArtLabel.setName("albumArtLabel"); // NOI18N

                shuffleButton.setText(resourceMap.getString("shuffleButton.text")); // NOI18N
                shuffleButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                shuffleButton.setName("shuffleButton"); // NOI18N

                jScrollPane7.setBorder(null);
                jScrollPane7.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                jScrollPane7.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
                jScrollPane7.setName("jScrollPane7"); // NOI18N

                jSlider1.getToolTipText();
                jSlider1.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                jSlider1.setName("jSlider1"); // NOI18N
                jSlider1.setPreferredSize(new java.awt.Dimension(100, 25));
                jScrollPane7.setViewportView(jSlider1);
                boolean foundMixer = dumpMixers();
                if(foundMixer == false)
                jScrollPane7.setViewportView(null);
                else{
                    //jScrollPane7.setBorder(null);
                    jScrollPane7.setViewportBorder(null);
                    //jSlider1.setBorder(null);
                }

                previousTrackButton.setText(resourceMap.getString("previousTrackButton.text")); // NOI18N
                previousTrackButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                previousTrackButton.setName("previousTrackButton"); // NOI18N

                nextTrackButton.setText(resourceMap.getString("nextTrackButton.text")); // NOI18N
                nextTrackButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                nextTrackButton.setName("nextTrackButton"); // NOI18N

                ratingLabel.setIcon(resourceMap.getIcon("ratingLabel.icon")); // NOI18N
                ratingLabel.setText(resourceMap.getString("ratingLabel.text")); // NOI18N
                ratingLabel.setToolTipText(resourceMap.getString("ratingLabel.toolTipText")); // NOI18N
                ratingLabel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                ratingLabel.setName("ratingLabel"); // NOI18N

                jButton7.setText(resourceMap.getString("jButton7.text")); // NOI18N
                jButton7.setToolTipText(resourceMap.getString("jButton7.toolTipText")); // NOI18N
                jButton7.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                jButton7.setName("jButton7"); // NOI18N

                jButton8.setText(resourceMap.getString("jButton8.text")); // NOI18N
                jButton8.setToolTipText(resourceMap.getString("jButton8.toolTipText")); // NOI18N
                jButton8.setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
                jButton8.setName("jButton8"); // NOI18N

                searchTextField.setText(resourceMap.getString("searchTextField.text")); // NOI18N
                searchTextField.setToolTipText(resourceMap.getString("searchTextField.toolTipText")); // NOI18N
                searchTextField.setName("searchTextField"); // NOI18N
                searchTextField.addMouseListener(new java.awt.event.MouseAdapter() {
                    public void mouseClicked(java.awt.event.MouseEvent evt) {
                        searchTextFieldMouseClicked(evt);
                    }
                });

                jScrollPane2.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                jScrollPane2.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
                jScrollPane2.setName("jScrollPane2"); // NOI18N

                currentTrackInfoBox.setColumns(20);
                currentTrackInfoBox.setEditable(false);
                currentTrackInfoBox.setRows(3);
                currentTrackInfoBox.setAutoscrolls(false);
                currentTrackInfoBox.setDoubleBuffered(true);
                currentTrackInfoBox.setName("currentTrackInfoBox"); // NOI18N
                jScrollPane2.setViewportView(currentTrackInfoBox);

                javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
                mainPanel.setLayout(mainPanelLayout);
                mainPanelLayout.setHorizontalGroup(
                    mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGap(10, 10, 10)
                                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 644, Short.MAX_VALUE)
                                .addContainerGap())
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jScrollPane7, javax.swing.GroupLayout.PREFERRED_SIZE, 99, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addGap(17, 17, 17)
                                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                            .addComponent(playButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                            .addComponent(stopButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                            .addComponent(pauseButton, javax.swing.GroupLayout.Alignment.LEADING))))
                                .addGap(18, 18, 18)
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(previousTrackButton)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(nextTrackButton)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 109, Short.MAX_VALUE)
                                        .addComponent(shuffleButton))
                                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 310, Short.MAX_VALUE))
                                .addGap(26, 26, 26)
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addGap(10, 10, 10)
                                        .addComponent(ratingLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(albumArtLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(searchTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 80, javax.swing.GroupLayout.PREFERRED_SIZE)
                                            .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                                .addComponent(jButton7, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                .addComponent(jButton8, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))))
                                .addGap(27, 27, 27))
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 45, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jProgressBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 544, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 45, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(16, 16, 16))))
                );
                mainPanelLayout.setVerticalGroup(
                    mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGap(14, 14, 14)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 81, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(playButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(pauseButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(stopButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(shuffleButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(nextTrackButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(previousTrackButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(jScrollPane7, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(jButton7, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jButton8, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 32, Short.MAX_VALUE)
                                        .addComponent(searchTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addComponent(albumArtLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 100, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(ratingLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, 20, Short.MAX_VALUE)
                            .addComponent(jProgressBar1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 545, Short.MAX_VALUE)
                        .addContainerGap())
                );

                playButton.addActionListener(cbh);
                pauseButton.setEnabled(false);
                pauseButton.addActionListener(cbh);
                stopButton.setEnabled(false);
                stopButton.addActionListener(cbh);
                jProgressBar1.addMouseListener(new MouseAdapter(){
                    public void mouseClicked(MouseEvent e){
                        try{
                            finalPercent = e.getX()/(double)jProgressBar1.getWidth();
                            double f = finalPercent * 100;
                            timeSkipped = finalPercent * (double)trackLength;
                            long finalP = (long)f;
                            long newPos = (long) Math.round((back.fileLength*finalP)/100);
                            back.doSeek = newPos;
                            back.toSkip = newPos;
                            changed = (int)f;
                        }catch(Exception ex){
                            //ex.printStackTrace();
                            showErrorDialog(ex);
                        }
                    }
                });
                albumArtLabel.addMouseListener(new MouseAdapter(){
                    public void mouseClicked(MouseEvent e){
                        if(e.getButton() == MouseEvent.BUTTON1)
                        showLargeAlbumArtwork();
                    }
                });
                shuffleButton.setEnabled(false);
                shuffleButton.addActionListener(cbh);
                previousTrackButton.setEnabled(false);
                previousTrackButton.addActionListener(cbh);
                nextTrackButton.setEnabled(false);
                nextTrackButton.addActionListener(cbh);
                ratingLabel.addMouseMotionListener(new MouseMotionAdapter(){
                    public void mouseMoved(MouseEvent e){
                        double x = (double)e.getX() / (double)ratingLabel.getWidth();
                        if(x <= 0.2){
                            ratingLabel.setIcon(oneSmile);
                            if(!currentlyPlaying2.isEmpty())
                            currentlyPlayingTable.setValueAt(oneSmile, back.currentLocation, 6);
                        }else if(x > 0.2 && x <= 0.4){
                            ratingLabel.setIcon(twoSmiles);
                            if(!currentlyPlaying2.isEmpty())
                            currentlyPlayingTable.setValueAt(twoSmiles, back.currentLocation, 6);
                        }else if(x > 0.4  && x <= 0.6){
                            ratingLabel.setIcon(threeSmiles);
                            if(!currentlyPlaying2.isEmpty())
                            currentlyPlayingTable.setValueAt(threeSmiles, back.currentLocation, 6);
                        }else if(x > 0.6  && x <= 0.8){
                            ratingLabel.setIcon(fourSmiles);
                            if(!currentlyPlaying2.isEmpty())
                            currentlyPlayingTable.setValueAt(fourSmiles, back.currentLocation, 6);
                        }else if(x > 0.8  && x <= 1.0){
                            ratingLabel.setIcon(fiveSmiles);
                            if(!currentlyPlaying2.isEmpty())
                            currentlyPlayingTable.setValueAt(fiveSmiles, back.currentLocation, 6);
                        }
                    }
                });

                ratingLabel.addMouseListener(new MouseAdapter(){
                    public void mouseClicked(MouseEvent e){
                        double x = (double)e.getX() / (double)ratingLabel.getWidth();
                        if(x <= 0.2){
                            back.updateRating(back.currentTrack, 1);
                            showRating();
                        }
                        else if(x > 0.2 && x <= 0.4){
                            back.updateRating(back.currentTrack, 2);
                            showRating();
                        }
                        else if(x > 0.4  && x <= 0.6){
                            back.updateRating(back.currentTrack, 3);
                            showRating();
                        }
                        else if(x > 0.6  && x <= 0.8){
                            back.updateRating(back.currentTrack, 4);
                            showRating();
                        }
                        else if(x > 0.8  && x <= 1.0){
                            back.updateRating(back.currentTrack, 5);
                            showRating();
                        }
                    }
                    public void mouseExited(MouseEvent e){
                        if(!currentlyPlaying2.isEmpty())
                        currentlyPlayingTable.setValueAt(smileys[Integer.parseInt(back.currentTrack.getRating())], back.currentLocation, 6);
                        showRating();
                    }
                });
                jButton7.setEnabled(false);

                jButton7.addActionListener(cbh);
                jButton8.setEnabled(false);

                jButton8.addActionListener(cbh);
                searchTextField.addKeyListener(new java.awt.event.KeyAdapter() {
                    public void keyTyped(KeyEvent e){
                        //if(e.getKeyCode() == KeyEvent.VK_ENTER){
                            //System.out.println(KeyEvent.getKeyText(e.getKeyCode()));
                            String filterText = searchTextField.getText();
                            if(KeyEvent.getKeyText(e.getKeyCode()).length() == 1 || e.isShiftDown()){
                                filterText += e.getKeyChar();
                            }
                            //System.out.println(filterText);
                            RowFilter<CustomTableModel, Object> rf = regexSearch(filterText);
                            if(jTabbedPane1.getSelectedIndex() == 0)
                            {
                                TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)currentlyShowingTable.getRowSorter();
                                sorter.setRowFilter(rf);
                                currentlyShowingTable.setRowSorter(sorter);
                            }
                            else if(jTabbedPane1.getSelectedIndex() == 1)
                            {
                                TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)currentlyPlayingTable.getRowSorter();
                                sorter.setRowFilter(rf);
                                currentlyPlayingTable.setRowSorter(sorter);
                            }
                            else if(jTabbedPane1.getSelectedIndex() == 3)
                            {
                                TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)playlistTable.getRowSorter();
                                sorter.setRowFilter(rf);
                                playlistTable.setRowSorter(sorter);
                            }
                            else if(jTabbedPane1.getSelectedIndex() == 5)
                            {
                                TableRowSorter<CustomTableModel> sorter = (TableRowSorter<CustomTableModel>)sharedLibraryTable.getRowSorter();
                                sorter.setRowFilter(rf);
                                sharedLibraryTable.setRowSorter(sorter);
                            }
                        }
                    });

                    menuBar.setDoubleBuffered(true);
                    menuBar.setName("menuBar"); // NOI18N

                    fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
                    fileMenu.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    fileMenu.setName("fileMenu"); // NOI18N

                    jMenuItem3.setText(resourceMap.getString("jMenuItem3.text")); // NOI18N
                    jMenuItem3.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenuItem3.setName("jMenuItem3"); // NOI18N
                    fileMenu.add(jMenuItem3);
                    jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            showFileChooser();
                        }
                    });

                    jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
                    jMenuItem1.setName("jMenuItem1"); // NOI18N
                    fileMenu.add(jMenuItem1);
                    jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            showDirChooser();
                        }
                    });
                    jMenuItem1.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

                    jMenuItem8.setText(resourceMap.getString("jMenuItem8.text")); // NOI18N
                    jMenuItem8.setName("jMenuItem8"); // NOI18N
                    jMenuItem8.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            jMenuItem8ActionPerformed(evt);
                        }
                    });
                    fileMenu.add(jMenuItem8);

                    jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
                    jMenuItem2.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenuItem2.setName("jMenuItem2"); // NOI18N
                    jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            jMenuItem2ActionPerformed(evt);
                        }
                    });
                    fileMenu.add(jMenuItem2);

                    javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getActionMap(MusicBoxView.class, this);
                    exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
                    exitMenuItem.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    exitMenuItem.setName("exitMenuItem"); // NOI18N
                    fileMenu.add(exitMenuItem);

                    menuBar.add(fileMenu);

                    jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
                    jMenu1.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenu1.setName("jMenu1"); // NOI18N

                    jMenuItem6.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_M, java.awt.event.InputEvent.CTRL_MASK));
                    jMenuItem6.setText(resourceMap.getString("jMenuItem6.text")); // NOI18N
                    jMenuItem6.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenuItem6.setName("jMenuItem6"); // NOI18N
                    jMenu1.add(jMenuItem6);
                    jMenuItem6.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            showMiniPlayer();
                        }
                    });

                    jMenuItem4.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_G, java.awt.event.InputEvent.CTRL_MASK));
                    jMenuItem4.setText(resourceMap.getString("jMenuItem4.text")); // NOI18N
                    jMenuItem4.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenuItem4.setName("jMenuItem4"); // NOI18N
                    jMenu1.add(jMenuItem4);
                    jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            String start = "&sa=N&start=" + imageResults + "&ndsp=21";
                            googleSearch = JOptionPane.showInputDialog("Search", "");
                            if(googleSearch != null && googleSearch.length() != 0)
                            back.googleImageSearch(googleSearch, start);
                        }
                    });

                    jMenuItem9.setText(resourceMap.getString("jMenuItem9.text")); // NOI18N
                    jMenuItem9.setName("jMenuItem9"); // NOI18N
                    jMenuItem9.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            jMenuItem9ActionPerformed(evt);
                        }
                    });
                    jMenu1.add(jMenuItem9);

                    jMenuItem7.setText(resourceMap.getString("jMenuItem7.text")); // NOI18N
                    jMenuItem7.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
                    jMenuItem7.setName("jMenuItem7"); // NOI18N
                    jMenuItem7.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            jMenuItem7ActionPerformed(evt);
                        }
                    });
                    jMenu1.add(jMenuItem7);

                    jMenuItem5.setText(resourceMap.getString("jMenuItem5.text")); // NOI18N
                    jMenuItem5.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    jMenuItem5.setName("jMenuItem5"); // NOI18N
                    jMenu1.add(jMenuItem5);
                    jMenuItem5.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                            showPreferences();
                        }
                    });

                    menuBar.add(jMenu1);

                    helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
                    helpMenu.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    helpMenu.setName("helpMenu"); // NOI18N

                    aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
                    aboutMenuItem.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    aboutMenuItem.setName("aboutMenuItem"); // NOI18N
                    helpMenu.add(aboutMenuItem);

                    menuBar.add(helpMenu);

                    statusPanel.setName("statusPanel"); // NOI18N

                    statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

                    statusMessageLabel.setName("statusMessageLabel"); // NOI18N

                    statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
                    statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

                    jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
                    jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
                    jLabel5.setName("jLabel5"); // NOI18N

                    javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
                    statusPanel.setLayout(statusPanelLayout);
                    statusPanelLayout.setHorizontalGroup(
                        statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(statusPanelLayout.createSequentialGroup()
                            .addContainerGap()
                            .addComponent(statusMessageLabel)
                            .addGap(18, 18, 18)
                            .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 608, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGap(28, 28, 28)
                            .addComponent(statusAnimationLabel)
                            .addContainerGap())
                        .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 674, Short.MAX_VALUE)
                    );
                    statusPanelLayout.setVerticalGroup(
                        statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(statusPanelLayout.createSequentialGroup()
                            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, statusPanelLayout.createSequentialGroup()
                                    .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(statusMessageLabel)
                                        .addComponent(statusAnimationLabel))
                                    .addGap(11, 11, 11))
                                .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE)))
                    );

                    PlayingPopupMenu.setName("PlayingPopupMenu"); // NOI18N

                    JMenuItem jI3 = new JMenuItem("Get Song Info");
                    JMenuItem jI6 = new JMenuItem("Create New Playlist");
                    JMenuItem jI14 = new JMenuItem("Remove from currently playing");
                    PlayingPopupMenu.add(jI14);
                    PlayingPopupMenu.add(jI6);
                    PlayingPopupMenu.add(jI3);
                    PlayingPopupMenu.add(jMenu3);
                    jI3.addActionListener(new java.awt.event.ActionListener()
                        {
                            public void actionPerformed(java.awt.event.ActionEvent evt)
                            {
                                showSongInfo(currentlyPlayingTable);
                            }
                        });
                        jI6.addActionListener(new java.awt.event.ActionListener()
                            {
                                public void actionPerformed(java.awt.event.ActionEvent evt)
                                {
                                    createNewPlayList();
                                }
                            });

                            jI14.addActionListener(new java.awt.event.ActionListener()
                                {
                                    public void actionPerformed(java.awt.event.ActionEvent evt)
                                    {
                                        removeFromCurrentlyPlaying();
                                    }
                                });

                                ShowingPopupMenu.setName("ShowingPopupMenu"); // NOI18N

                                JMenuItem jI4 = new JMenuItem("Append to playlist");
                                JMenuItem jI5 = new JMenuItem("Get Song Info");
                                JMenuItem jI8 = new JMenuItem("Create New Playlist");
                                JMenuItem removeFromLibrary = new JMenuItem("Remove from library");
                                ShowingPopupMenu.add(jI4);
                                ShowingPopupMenu.add(jI5);
                                ShowingPopupMenu.add(jI8);
                                ShowingPopupMenu.add(jMenu2);
                                ShowingPopupMenu.add(removeFromLibrary);
                                //ShowingPopupMenu.setInvoker(currentlyShowingTable);
                                jI4.addActionListener(new java.awt.event.ActionListener()
                                    {
                                        public void actionPerformed(java.awt.event.ActionEvent evt)
                                        {
                                            appendToPlayList(allMusic, currentlyShowingTable);
                                        }
                                    });

                                    jI5.addActionListener(new java.awt.event.ActionListener()
                                        {
                                            public void actionPerformed(java.awt.event.ActionEvent evt)
                                            {
                                                showSongInfo(currentlyShowingTable, allMusic);
                                            }
                                        });
                                        jI8.addActionListener(new java.awt.event.ActionListener()
                                            {
                                                public void actionPerformed(java.awt.event.ActionEvent evt)
                                                {
                                                    createNewPlayList();
                                                }
                                            });
                                            removeFromLibrary.addActionListener(new ActionListener()
                                                {
                                                    public void actionPerformed(ActionEvent evt)
                                                    {
                                                        removeSongsFromLibrary();
                                                    }
                                                });

                                                jPopupMenu3.setName("jPopupMenu3"); // NOI18N

                                                JMenuItem jI7 = new JMenuItem("Delete Selected Playlist");
                                                JMenuItem jI9 = new JMenuItem("Create New Playlist");
                                                jPopupMenu3.add(jI9);
                                                jPopupMenu3.add(jI7);

                                                jI7.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        String name = (String)jList3.getSelectedValue();
                                                        deletePlaylist(name);
                                                    }
                                                });
                                                jI9.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        createNewPlayList();
                                                    }
                                                });

                                                PlaylistPopupMenu.setName("PlaylistPopupMenu"); // NOI18N

                                                JMenuItem jI10 = new JMenuItem("Append to playlist");
                                                JMenuItem jI11 = new JMenuItem("Get Song Info");
                                                JMenuItem jI12 = new JMenuItem("Create New Playlist");
                                                JMenuItem jI13 = new JMenuItem("Remove song(s) from Playlist");
                                                PlaylistPopupMenu.add(jI10);
                                                PlaylistPopupMenu.add(jI11);
                                                PlaylistPopupMenu.add(jI12);
                                                PlaylistPopupMenu.add(jI13);

                                                jI10.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        appendToPlayList(playList, playlistTable,true);
                                                    }
                                                });

                                                jI11.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        showSongInfo(playlistTable);
                                                    }
                                                });
                                                jI12.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        createNewPlayList();
                                                    }
                                                });

                                                jI13.addActionListener(new java.awt.event.ActionListener() {
                                                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                                                        removeSongFromPlayList();
                                                    }
                                                });

                                                SharedLibraryPopupMenu.setName("SharedLibraryPopupMenu"); // NOI18N

                                                JMenuItem sLATP = new JMenuItem("Append to playlist");
                                                SharedLibraryPopupMenu.add(sLATP);
                                                sLATP.addActionListener(new java.awt.event.ActionListener()
                                                    {
                                                        public void actionPerformed(java.awt.event.ActionEvent evt)
                                                        {
                                                            appendToPlayList(sharedLibrary, sharedLibraryTable);
                                                        }
                                                    });

                                                    setComponent(mainPanel);
                                                    setMenuBar(menuBar);
                                                    setStatusBar(statusPanel);
                                                }// </editor-fold>//GEN-END:initComponents

    public void removeSongsFromLibrary()
    {
        int[] locations = currentlyShowingTable.getSelectedRows();
        //Collections.sort(allMusic, new ArtistComparator());
        int currentlyShowingCount = 0;
        for(int loc: locations)
        {
            loc = currentlyShowingTable.convertRowIndexToModel(loc - (currentlyShowingCount++));
            ((CustomTableModel)currentlyShowingTable.getModel()).removeRow(loc);
            if(SwingUtilities.isEventDispatchThread())
            {
                ((CustomTableModel)currentlyShowingTable.getModel()).fireTableDataChanged();
            }
            else
            {
                SwingUtilities.invokeLater(new Thread()
                {
                    public void run()
                    {
                        ((CustomTableModel)currentlyShowingTable.getModel()).fireTableDataChanged();
                    }
                });
            }
            MusicOutputDesign mod = allMusic.elementAt(loc);
            allMusic.remove(mod);
            try 
            {
                db.deleteFromMusicLibrary(db.getConnection().createStatement(), mod);
            }
            catch (SQLException ex) 
            {
                showErrorDialog(ex);
            }
            if(currentlyPlaying2.indexOf(mod.getId()) > -1)
            {
                int index = currentlyPlaying2.indexOf(mod.getId());
                currentlyPlaying2.removeElement(mod.getId());
                db.deleteFromCurrentlyPlaying(mod.getId());
                ((CustomTableModel)currentlyPlayingTable.getModel()).removeRow(index);
                if(SwingUtilities.isEventDispatchThread())
                {
                    ((CustomTableModel)currentlyPlayingTable.getModel()).fireTableDataChanged();
                }
                else
                {
                    SwingUtilities.invokeLater(new Thread()
                    {
                        public void run()
                        {
                            ((CustomTableModel)currentlyPlayingTable.getModel()).fireTableDataChanged();
                        }
                    });
                }
            }
            if(playList.indexOf(mod.getId()) > -1)
            {
                int index = playList.indexOf(mod.getId());
                playList.removeElement(mod.getId());
                ((CustomTableModel)playlistTable.getModel()).removeRow(index);
                if(SwingUtilities.isEventDispatchThread())
                {
                    ((CustomTableModel)playlistTable.getModel()).fireTableDataChanged();
                }
                else
                {
                    SwingUtilities.invokeLater(new Thread()
                    {
                        public void run()
                        {
                            ((CustomTableModel)playlistTable.getModel()).fireTableDataChanged();
                        }
                    });
                }
            }
            db.deleteSongFromPlaylists(mod.getId());
        }
    }
    
    private void jTabbedPane1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTabbedPane1MouseClicked
        if (jTabbedPane1.getSelectedIndex() != 4 && smallArt != null && smallArt.isVisible()) 
        {
            smallArt.dispose();
            smallArt.googleSearched = false;
        }
    }//GEN-LAST:event_jTabbedPane1MouseClicked

    private void jTabbedPane2MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTabbedPane2MouseClicked
        if (smallArt != null && smallArt.isVisible()) {
            smallArt.dispose();
            smallArt.googleSearched = false;
        }
    }//GEN-LAST:event_jTabbedPane2MouseClicked

    private void jPanel3AncestorMoved(javax.swing.event.AncestorEvent evt) {//GEN-FIRST:event_jPanel3AncestorMoved
        if (smallArt != null && smallArt.isVisible()) {
            smallArt.dispose();
            smallArt.googleSearched = false;
        }
    }//GEN-LAST:event_jPanel3AncestorMoved

    private void jPanel4AncestorMoved(javax.swing.event.AncestorEvent evt) {//GEN-FIRST:event_jPanel4AncestorMoved
        if (smallArt != null && smallArt.isVisible()) {
            smallArt.dispose();
            smallArt.googleSearched = false;
        }
    }//GEN-LAST:event_jPanel4AncestorMoved

    private void jProgressBar1MouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jProgressBar1MouseDragged
        try {
            finalPercent = evt.getX() / (double) jProgressBar1.getWidth();
            double f = finalPercent * 100;
            timeSkipped = finalPercent * (double) trackLength;
            long finalP = (long) f;
            long newPos = (long) Math.round((back.fileLength * finalP) / 100);
            back.doSeek = newPos;
            back.toSkip = newPos;
            changed = (int) f;
        } catch (Exception ex) {
            showErrorDialog(ex);
        }
    }//GEN-LAST:event_jProgressBar1MouseDragged

    private void searchTextFieldMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_searchTextFieldMouseClicked
        if (searchTextField.getText().equals("Search")) {
            searchTextField.setText("");
        }
    }//GEN-LAST:event_searchTextFieldMouseClicked

    private void mainPanelComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_mainPanelComponentResized
        if (playing) {
            String beginning = "";
            Font f = currentTrackInfoBox.getFont();
            int fontWidth = currentTrackInfoBox.getFontMetrics(f).stringWidth(" ");
            int boxWidth = currentTrackInfoBox.getWidth();
            int currentWidth = 0;
            //System.out.println(back.sT.getBeginning().length());
            while (currentWidth <= (boxWidth + fontWidth + fontWidth)) {
                beginning += " ";
                currentWidth += fontWidth;
            }
            back.sT.setBeginning(beginning);
        }
    }//GEN-LAST:event_mainPanelComponentResized

    private void jMenuItem8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem8ActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setFileFilter(new FileNameExtensionFilter("Music Box Library", "script"));
        int retVal = jfc.showOpenDialog(mainPanel);
        if (retVal == JFileChooser.APPROVE_OPTION) {
            File f = jfc.getSelectedFile();
            if (f.getName().startsWith("MDB.odb")) {
                JOptionPane.showMessageDialog(mainPanel, "Inserting data from old library", "Load Library", JOptionPane.PLAIN_MESSAGE);
                back.db.getCustomConnection(new File(f.getParentFile().getAbsolutePath() + "/MDB.odb"));
                back.db.insertOldIntoNew(back.db.selectAllCustom());
                back.db.closeCustomConnection();
                JOptionPane.showMessageDialog(mainPanel, "Finished Loading Data", "Finished", JOptionPane.PLAIN_MESSAGE);
                getAllMusic();
                currentlyShowingTable.setModel(populateTableModel((CustomTableModel) currentlyShowingTable.getModel()));
                DefaultListModel dlm = new DefaultListModel();
                dlm.addElement("(All Artists)");
                String previousArtist = "awesome";
                for (int i = 0; i < allMusic.size(); i++) {
                    if (i - 1 >= 0) {
                        previousArtist = allMusic.elementAt(i - 1).getArtist();
                    }
                    if (previousArtist.compareToIgnoreCase(allMusic.elementAt(i).getArtist()) != 0 && !dlm.contains(allMusic.elementAt(i).getArtist())) {
                        dlm.addElement(allMusic.elementAt(i).getArtist());
                    }
                }
                dlm = sortListModel(dlm);
                jList1.setModel(dlm);
                dlm1 = new DefaultListModel();
                dlm1.addElement("(All Albums)");
                String previousAlbum = "awesome";
                for (int i = 0; i < allMusic.size(); i++) {
                    if (i - 1 >= 0) {
                        previousAlbum = allMusic.elementAt(i - 1).getAlbum();
                    }
                    if (previousAlbum.compareToIgnoreCase(allMusic.elementAt(i).getAlbum()) != 0 && !dlm1.contains(allMusic.elementAt(i).getAlbum())) {
                        dlm1.addElement(allMusic.elementAt(i).getAlbum());
                    }
                }
                dlm1 = sortListModel(dlm1);
                jList2.setModel(dlm1);
            }
        } else {
            return;
        }
    }//GEN-LAST:event_jMenuItem8ActionPerformed

    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
        try {
            int retVal = JOptionPane.showConfirmDialog(mainPanel, "Delete All Music?", "Delete All?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
            if (retVal == JOptionPane.YES_OPTION) {
                back.db.deleteAll(back.db.getConnection().createStatement());
                JOptionPane.showMessageDialog(mainPanel, "Done", "Done", JOptionPane.INFORMATION_MESSAGE);
                getAllMusic();
                ((CustomTableModel) currentlyShowingTable.getModel()).removeAllRows();
                currentlyShowingTable.setModel(populateTableModel((CustomTableModel) currentlyShowingTable.getModel()));
                DefaultListModel dlm = new DefaultListModel();
                dlm.addElement("(All Artists)");
                String previousArtist = "awesome";
                for (int i = 0; i < allMusic.size(); i++) {
                    if (i - 1 >= 0) {
                        previousArtist = allMusic.elementAt(i - 1).getArtist();
                    }
                    if (previousArtist.compareToIgnoreCase(allMusic.elementAt(i).getArtist()) != 0 && !dlm.contains(allMusic.elementAt(i).getArtist())) {
                        dlm.addElement(allMusic.elementAt(i).getArtist());
                    }
                }
                dlm = sortListModel(dlm);
                jList1.setModel(dlm);
                dlm1 = new DefaultListModel();
                dlm1.addElement("(All Albums)");
                String previousAlbum = "awesome";
                for (int i = 0; i < allMusic.size(); i++) {
                    if (i - 1 >= 0) {
                        previousAlbum = allMusic.elementAt(i - 1).getAlbum();
                    }
                    if (previousAlbum.compareToIgnoreCase(allMusic.elementAt(i).getAlbum()) != 0 && !dlm1.contains(allMusic.elementAt(i).getAlbum())) {
                        dlm1.addElement(allMusic.elementAt(i).getAlbum());
                    }
                }
                dlm1 = sortListModel(dlm1);
                jList2.setModel(dlm1);
            }
        } catch (SQLException ex) {
            showErrorDialog(ex);
        }
    }//GEN-LAST:event_jMenuItem2ActionPerformed

    private void jMenuItem7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem7ActionPerformed
        // temporary
        if (jMenuItem7.getText().equals("Connect")) 
        {
            new Thread() 
            {
                public void run() 
                {
                    connectToServer();
                }
            }.start();
        }
        else
        {
            sl.disconnect();
            jMenuItem7.setText("Connect");
        }
    }//GEN-LAST:event_jMenuItem7ActionPerformed

    private void jMenuItem9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem9ActionPerformed
        try 
        {
            ResultSet rs = db.getConnection().createStatement().executeQuery("Select * from MusicLibrary");//db.list(db.getConnection().createStatement(), "FilePath,File,ID");
            while(rs.next())
            {
                if(!new File(rs.getString("FilePath")+ "/" + rs.getString("File")).exists())
                {
                    String id = rs.getString("ID");
                    if(currentlyPlaying2.indexOf(id) > -1)
                    {
                        currentlyPlaying2.remove(id);
                        if(id.indexOf("\'") > -1)
                        {
                            id = id.replace("\'", "\'\'");
                        }
                        db.getConnection().createStatement().executeUpdate("Delete from CurrentlyPlaying where ID = '"+id+"'");
                    }
                    if(id.indexOf("\'") > -1)
                    {
                        id = id.replace("\'", "\'\'");
                    }
                    String query = "Delete from MusicLibrary where ID = \'"+id+"\'";
                    int retVal = db.getConnection().createStatement().executeUpdate(query);
                }
            }
            rs.close();
        } 
        catch (SQLException ex) 
        {
            showErrorDialog(ex);
        }
        getAllMusic();
        CustomTableModel ctm = new CustomTableModel();
        currentlyShowingTable = new musicbox.gui.CustomTable(populateTableModel(ctm));
        if(SwingUtilities.isEventDispatchThread())
        {
            ctm.fireTableDataChanged();
        }
        else
        {
            SwingUtilities.invokeLater(new Thread()
            {
                @Override
                public void run()
                {
                    ((CustomTableModel)currentlyShowingTable.getModel()).fireTableDataChanged();
                }
            });
        }
    }//GEN-LAST:event_jMenuItem9ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    public javax.swing.JPopupMenu PlayingPopupMenu;
    public javax.swing.JPopupMenu PlaylistPopupMenu;
    public javax.swing.JPopupMenu SharedLibraryPopupMenu;
    public javax.swing.JPopupMenu ShowingPopupMenu;
    public javax.swing.JList addressList;
    public javax.swing.JLabel albumArtLabel;
    public javax.swing.JTextArea currentTrackInfoBox;
    public javax.swing.JTable currentlyPlayingTable;
    public javax.swing.JTable currentlyShowingTable;
    public javax.swing.JButton jButton7;
    public javax.swing.JButton jButton8;
    public javax.swing.JEditorPane jEditorPane1;
    public javax.swing.JLabel jLabel1;
    public javax.swing.JLabel jLabel2;
    public javax.swing.JLabel jLabel5;
    public javax.swing.JList jList1;
    public javax.swing.JList jList2;
    public javax.swing.JList jList3;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JMenuItem jMenuItem7;
    private javax.swing.JMenuItem jMenuItem8;
    private javax.swing.JMenuItem jMenuItem9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    public javax.swing.JPopupMenu jPopupMenu3;
    public javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane10;
    private javax.swing.JScrollPane jScrollPane11;
    public javax.swing.JScrollPane jScrollPane12;
    private javax.swing.JScrollPane jScrollPane13;
    private javax.swing.JScrollPane jScrollPane15;
    public javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    public javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    public javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JScrollPane jScrollPane8;
    private javax.swing.JScrollPane jScrollPane9;
    public javax.swing.JSlider jSlider1;
    public javax.swing.JTabbedPane jTabbedPane1;
    public javax.swing.JTabbedPane jTabbedPane2;
    public javax.swing.JPanel mainPanel;
    public javax.swing.JMenuBar menuBar;
    public javax.swing.JButton nextTrackButton;
    public javax.swing.JButton pauseButton;
    public javax.swing.JButton playButton;
    public javax.swing.JTable playlistTable;
    public javax.swing.JButton previousTrackButton;
    public javax.swing.JLabel ratingLabel;
    private javax.swing.JTextField searchTextField;
    public javax.swing.JTable sharedLibraryTable;
    public javax.swing.JList sharedPlayList;
    public javax.swing.JButton shuffleButton;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    public javax.swing.JButton stopButton;
    // End of variables declaration//GEN-END:variables
    /**
     *
     */
    public JMenu jMenu2;
    /**
     *
     */
    public JMenu jMenu3;
    /**
     *
     */
    public Backend back;
    /**
     *
     */
    public Vector initial;
    /**
     *
     */
    public Vector<MusicOutputDesign> allMusic = new Vector<MusicOutputDesign>();
    /**
     *
     */
    public double trackLength;
    /**
     *
     */
    public Timer timer;
    /**
     *
     */
    //public Vector<MusicOutputDesign> currentlyPlaying = new Vector<MusicOutputDesign>();
    //public Vector<Integer> currentlyPlaying = new Vector<Integer>();
    public Vector<String> currentlyPlaying2 = new Vector<String>();
    /**
     *
     */
    public boolean originalCP = false;
    /**
     *
     */
    //public Vector<MusicOutputDesign> currentlyShowing = new Vector<MusicOutputDesign>();
    /**
     *
     */
    //public Vector<MusicOutputDesign> currentlyShowingClone = new Vector<MusicOutputDesign>();
    /**
     *
     */
    public boolean originalCS = false;
    /**
     *
     */
    public Vector<String> playList = new Vector<String>();
    /**
     *
     */
    public boolean originalPL = false;
    boolean pause = false;
    private final javax.swing.Timer messageTimer;
    /**
     *
     */
    public javax.swing.Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private JDialog aboutBox;
    private JDialog fileChooser;
    private JDialog dirChooser;
    /**
     *
     */
    protected SongInfo songInfo;
    private JDialog errorDialog;
    /**
     *
     */
    public SongSplash songSplash;
    /**
     *
     */
    public boolean playing = false;
    /**
     *
     */
    public String[] columnNames = new String[]{
        "Track Number", "Track Title", "Artist", "Album", "Length", "Play Count", "Rating"
    };
    private Document doc;
    /**
     *
     */
    public Comparator previousComparator;
    /**
     *
     */
    public ImageIcon zeroSmiles;
    /**
     *
     */
    public ImageIcon oneSmile;
    /**
     *
     */
    public ImageIcon twoSmiles;
    /**
     *
     */
    public ImageIcon threeSmiles;
    /**
     *
     */
    public ImageIcon fourSmiles;
    /**
     *
     */
    public ImageIcon fiveSmiles;
    /**
     *
     */
    public ImageIcon noImage;
    /**
     *
     */
    public ImageIcon[] smileys;
    public ImageIcon musicBoxIcon16;
    private int screenHeight;
    private int screenWidth;
    private int sSXLocation;
    private int sSYLocation;
    public Database db;
    /**
     *
     */
    public boolean clicked = false;
    /**
     *
     */
    public double timeSkipped = 0;
    /**
     *
     */
    public double finalPercent;
    /**
     *
     */
    public int changed = 0;
    /**
     *Used for calculating the progress of the progress bar
     */
    public Task task;
    /**
     *
     */
    public MusicBoxView p;
    private DefaultListModel dlm1;
    /**
     *
     */
    public int googleImageLocation = 0;
    /**
     *
     */
    public int imageResults = 0;
    /**
     *
     */
    public String googleSearch = new String();
    private String previousList = " ";
    /**
     *
     */
    public Font projectFont;
    /**
     *
     */
    public Color fontColor;
    /**
     *
     */
    public boolean internetConnection;
    /**
     *
     */
    public boolean errorMessage;
    /**
     *
     */
    public Color progressBarColor;
    private LocalStorage ls;
    /**
     *
     */
    public LookAndFeel laf;
    /**
     *
     */
    public MiniPlayer mini;
    /**
     *
     */
    public JFrame pref;
    private boolean foundArt;
    private String previousLoc;
    /**
     *
     */
    protected boolean foundMixer = false;
    protected SmallAlbumArt smallArt;
    protected CustomTableListener ctl;
    private CustomButtonHandler cbh;
    private Socket socket;
    private int previousSelection = -1;
    private SocketListener sl;
    private CustomMouseAdapter cma;
    public Vector<MusicOutputDesign> sharedLibrary;
    public String serverAddress;
}

class Splash extends Thread 
{

    public boolean cont = true;

    public Splash() 
    {
    }

    @Override
    @SuppressWarnings("static-access")
    public void run() 
    {
        if (SplashScreen.getSplashScreen().isVisible()) 
        {
            final SplashScreen splash = SplashScreen.getSplashScreen();
            String[] images = {"resources/No Talent Splash/NoTalent1.png", "resources/No Talent Splash/NoTalent2.png",
                "resources/No Talent Splash/NoTalent3.png", "resources/No Talent Splash/NoTalent4.png", "resources/No Talent Splash/NoTalent5.png",
                "resources/No Talent Splash/NoTalent6.png", "resources/No Talent Splash/NoTalent7.png", "resources/No Talent Splash/NoTalent8.png",
                "resources/No Talent Splash/NoTalent9.png", "resources/No Talent Splash/NoTalent10.png", "resources/No Talent Splash/NoTalent11.png",
                "resources/No Talent Splash/NoTalent12.png", "resources/No Talent Splash/NoTalent13.png", "resources/No Talent Splash/NoTalent14.png",
                "resources/No Talent Splash/NoTalent15.png", "resources/No Talent Splash/NoTalent16.png", "resources/No Talent Splash/NoTalent17.png",
                "resources/No Talent Splash/NoTalent18.png", "resources/No Talent Splash/NoTalent19.png", "resources/No Talent Splash/NoTalent20.png",
                "resources/No Talent Splash/NoTalent21.png", "resources/No Talent Splash/NoTalent22.png", "resources/No Talent Splash/NoTalent23.png",
                "resources/No Talent Splash/NoTalent24.png", "resources/No Talent Splash/NoTalent25.png", "resources/No Talent Splash/NoTalent26.png",
                "resources/No Talent Splash/NoTalent27.png", "resources/No Talent Splash/NoTalent28.png", "resources/No Talent Splash/NoTalent29.png",
                "resources/No Talent Splash/NoTalent0.png"};
            int i = 0;
            while (cont && splash.isVisible()) 
            {
                try 
                {
                    if (splash.isVisible()) 
                    {
                        splash.setImageURL(new File(images[(i / 2) % 30]).toURI().toURL());
                    }
                    i++;
                    Thread.sleep(100);
                } 
                catch (NullPointerException ex) 
                {
                    ex.printStackTrace();
                    //showErrorDialog(ex);
                } 
                catch (IOException ex) 
                {
                    ex.printStackTrace();
                    //showErrorDialog(ex);
                } 
                catch (IllegalStateException ex) 
                {
                    ex.printStackTrace();
                    //showErrorDialog(ex);
                } 
                catch (InterruptedException ex) 
                {
                    ex.printStackTrace();
                    //showErrorDialog(ex);
                }
            }
            splash.close();
            Thread.currentThread().stop();
        }
    }
}

abstract class CustomRowFilter<M, I> extends RowFilter<M, I> 
{
    /*public static <M,I> RowFilter<M, I> regexFilter(String regex, int... indices) 
    {
        return (RowFilter<M, I>) new RegexFilter(Pattern.compile(regex, Pattern.CASE_INSENSITIVE), indices);
    }*/

    @Override
    public boolean include(javax.swing.RowFilter.Entry<? extends M, ? extends I> entry) 
    {
        // TODO Auto-generated method stub
        return false;
    }

    private void checkIndices(int[] columns) 
    {
        for (int i = columns.length - 1; i >= 0; i--) 
        {
            if (columns[i] < 0) 
            {
                throw new IllegalArgumentException("Index must be >= 0");
            }
        }
    }

    private abstract class GeneralFilter extends RowFilter<Object, Object> 
    {
        private int[] columns;

        GeneralFilter(int[] columns) 
        {
            checkIndices(columns);
            this.columns = columns;
        }

        public boolean include(Entry<? extends Object, ? extends Object> value) 
        {
            int count = value.getValueCount();
            if (columns.length > 0) 
            {
                for (int i = columns.length - 1; i >= 0; i--) 
                {
                    int index = columns[i];
                    if (index < count) 
                    {
                        if (include(value, index)) 
                        {
                            return true;
                        }
                    }
                }
            } 
            else 
            {
                while (--count >= 0) 
                {
                    if (include(value, count)) 
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        protected abstract boolean include(Entry<? extends Object, ? extends Object> value, int index);
    }

    private class RegexFilter extends GeneralFilter 
    {
        private Matcher matcher;

        RegexFilter(Pattern regex, int[] columns) 
        {
            super(columns);
            if (regex == null) 
            {
                throw new IllegalArgumentException("Pattern must be non-null");
            }
            matcher = regex.matcher("");
        }

        protected boolean include(
                Entry<? extends Object, ? extends Object> value, int index) {
            matcher.reset(value.getStringValue(index));
            return matcher.find();
        }
    }
}
