/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Preferences.java
 *
 * Created on Dec 23, 2009, 1:11:40 AM
 */
package musicbox.gui;

import com.sun.java.swing.plaf.motif.MotifLookAndFeel;
import java.awt.Color;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Map;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.JFrame;
import javax.swing.JProgressBar;
import javax.swing.JRootPane;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.OceanTheme;
import musicbox.backend.MusicBoxApp;
import org.jdesktop.application.ApplicationContext;
import org.jdesktop.application.LocalStorage;
import org.pushingpixels.substance.api.SubstanceLookAndFeel;
import org.pushingpixels.substance.api.skin.SkinInfo;
import org.pushingpixels.substance.api.skin.SubstanceAutumnLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceBusinessBlackSteelLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceBusinessBlueSteelLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceBusinessLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceChallengerDeepLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceCremeCoffeeLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceCremeLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceDustCoffeeLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceDustLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceEmeraldDuskLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceGeminiLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceGraphiteAquaLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceGraphiteGlassLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceGraphiteLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceMagellanLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceMistAquaLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceMistSilverLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceModerateLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceNebulaBrickWallLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceNebulaLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceOfficeBlue2007LookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceOfficeSilver2007LookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceRavenLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceSaharaLookAndFeel;
import org.pushingpixels.substance.api.skin.SubstanceTwilightLookAndFeel;

/**
 *
 * @author Isaac Hammon
 */
public class Preferences extends javax.swing.JFrame 
{

    Font[] fonts;
    String[] fontNames;
    int[] styles = {Font.PLAIN, Font.ITALIC, Font.BOLD};
    int[] sizes = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33,
        34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48};
    Font f;
    LocalStorage ls;
    Color progressBarColor;
    Color fontColor;
    boolean internetConnection;
    boolean errorMessages;
    boolean lafChange;
    MusicBoxView mbv;

    /** Creates new form Preferences */
    public Preferences(MusicBoxView mbv) 
    {
        this.mbv = mbv;
        initComponents();
        this.setIconImage(mbv.musicBoxIcon16.getImage());
        ApplicationContext appContext = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext();
        ls = appContext.getLocalStorage();
        ls.setDirectory(new File("resources/session"));
        laf = mbv.laf;
        selectCurrentLAF();
        lafChange = false;
        f = mbv.projectFont;//(Font) ls.load("preferences.xml");
        progressBarColor = mbv.progressBarColor;//(Color) ls.load("progressbarcolor.xml");
        fontColor = mbv.fontColor;//(Color) ls.load("fontcolor.xml");
        internetConnection = mbv.internetConnection;//(Boolean) ls.load("internetconnection.xml");
        errorMessages = mbv.errorMessage;//(Boolean) ls.load("errormessages.xml");
        jCheckBox1.setSelected(internetConnection);
        jCheckBox2.setSelected(errorMessages);
        jTextField1.setText(mbv.serverAddress);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel3 = new javax.swing.JPanel();
        jCheckBox1 = new javax.swing.JCheckBox();
        jCheckBox2 = new javax.swing.JCheckBox();
        jLabel3 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        int style = Font.PLAIN;
        jList2 = new javax.swing.JList();
        jScrollPane3 = new javax.swing.JScrollPane();
        jList3 = new javax.swing.JList();
        jScrollPane1 = new javax.swing.JScrollPane();
        fonts = GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts();
        int loc = 0;
        DefaultListModel dlm1 = new DefaultListModel();
        fontNames = new String[fonts.length];
        for(int i = 0; i < fontNames.length; i++){
            dlm1.addElement(fonts[i].getName());
            if(mbv.projectFont != null)
            if(mbv.projectFont.getName().compareTo(fonts[i].getName()) == 0)
            loc = i;
        }
        jList1 = new javax.swing.JList();
        jScrollPane4 = new javax.swing.JScrollPane();
        jLabel1 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jColorChooser1 = new javax.swing.JColorChooser();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton3 = new javax.swing.JRadioButton();
        jPanel4 = new javax.swing.JPanel();
        jComboBox1 = new javax.swing.JComboBox();
        Map<String, SkinInfo> skins = SubstanceLookAndFeel.getAllSkins();

        skinModel = new DefaultComboBoxModel();
        for (SkinInfo skinInfo : skins.values()) {
            skinModel.addElement(skinInfo.getDisplayName());
        }
        jComboBox2 = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jButton2 = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getResourceMap(Preferences.class);
        setTitle(resourceMap.getString("formy.title")); // NOI18N
        setName("formy"); // NOI18N
        setResizable(false);

        jTabbedPane1.setDoubleBuffered(true);
        jTabbedPane1.setName("jTabbedPane1"); // NOI18N

        jPanel3.setName("jPanel3"); // NOI18N

        jCheckBox1.setText(resourceMap.getString("jCheckBox1.text")); // NOI18N
        jCheckBox1.setName("jCheckBox1"); // NOI18N

        jCheckBox2.setText(resourceMap.getString("jCheckBox2.text")); // NOI18N
        jCheckBox2.setName("jCheckBox2"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jTextField1.setText(resourceMap.getString("jTextField1.text")); // NOI18N
        jTextField1.setName("jTextField1"); // NOI18N

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jCheckBox1)
                    .addComponent(jCheckBox2)
                    .addComponent(jLabel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jTextField1))
                .addContainerGap(209, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGap(34, 34, 34)
                .addComponent(jCheckBox1)
                .addGap(18, 18, 18)
                .addComponent(jCheckBox2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(299, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel3.TabConstraints.tabTitle"), jPanel3); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        jScrollPane2.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane2.border.title"))); // NOI18N
        jScrollPane2.setName("jScrollPane2"); // NOI18N

        DefaultListModel dlm2 = new DefaultListModel();
        dlm2.addElement("Plain");
        dlm2.addElement("Italic");
        dlm2.addElement("Bold");
        jList2.setModel(dlm2);
        jList2.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        jList2.setToolTipText(resourceMap.getString("style.toolTipText")); // NOI18N
        jList2.setName("style"); // NOI18N
        jScrollPane2.setViewportView(jList2);
        if(style == Font.PLAIN)
        jList2.setSelectedIndex(0);
        if(style == Font.ITALIC)
        jList2.setSelectedIndex(1);
        if(style == Font.BOLD)
        jList2.setSelectedIndex(2);

        jList2.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent e){
                fontChange = true;
                f = new Font(fonts[jList1.getSelectedIndex()].getName(), styles[jList2.getSelectedIndex()], sizes[jList3.getSelectedIndex()]);
                jLabel1.setFont(f);
            }
        });

        jScrollPane3.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane3.border.title"))); // NOI18N
        jScrollPane3.setName("jScrollPane3"); // NOI18N

        DefaultListModel dlm3 = new DefaultListModel();
        for(int i = 0; i < sizes.length; i++)
        dlm3.addElement(sizes[i]);
        jList3.setModel(dlm3);
        jList3.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        jList3.setToolTipText(resourceMap.getString("size.toolTipText")); // NOI18N
        jList3.setName("size"); // NOI18N
        jScrollPane3.setViewportView(jList3);
        if(mbv.projectFont != null){
            jList3.setSelectedIndex(mbv.projectFont.getSize());
            jList3.ensureIndexIsVisible(mbv.projectFont.getSize());
        }else{
            jList3.setSelectedIndex(11);
            jList3.ensureIndexIsVisible(11);
        }

        jList3.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent e){
                fontChange = true;
                f = new Font(fonts[jList1.getSelectedIndex()].getName(), styles[jList2.getSelectedIndex()], sizes[jList3.getSelectedIndex()]);
                jLabel1.setFont(f);
            }
        });

        jScrollPane1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jScrollPane1.border.title"))); // NOI18N
        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jList1.setModel(dlm1);
        jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        jList1.setToolTipText(resourceMap.getString("name.toolTipText")); // NOI18N
        jList1.setName("name"); // NOI18N
        jList1.setSelectedIndex(loc);
        jList1.ensureIndexIsVisible(loc);
        jScrollPane1.setViewportView(jList1);
        jList1.addListSelectionListener(new ListSelectionListener(){
            public void valueChanged(ListSelectionEvent e){
                fontChange = true;
                f = new Font(fonts[jList1.getSelectedIndex()].getName(), styles[jList2.getSelectedIndex()], sizes[jList3.getSelectedIndex()]);
                jLabel1.setFont(f);
            }
        });

        jScrollPane4.setBorder(null);
        jScrollPane4.setName("jScrollPane4"); // NOI18N

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jLabel1.border.title"))); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N
        jLabel1.setFont(mbv.projectFont);
        jScrollPane4.setViewportView(jLabel1);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 219, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(30, 30, 30)
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 144, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(27, 27, 27)
                        .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 93, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jScrollPane4, javax.swing.GroupLayout.DEFAULT_SIZE, 513, Short.MAX_VALUE))
                .addGap(30, 30, 30))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(50, 50, 50)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(48, 48, 48)
                .addComponent(jScrollPane4, javax.swing.GroupLayout.PREFERRED_SIZE, 133, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(160, 160, 160))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel1.TabConstraints.tabTitle"), jPanel1); // NOI18N

        jPanel2.setName("jPanel2"); // NOI18N

        jColorChooser1.setName("jColorChooser1"); // NOI18N

        jRadioButton1.setText(resourceMap.getString("jRadioButton1.text")); // NOI18N
        jRadioButton1.setName("jRadioButton1"); // NOI18N

        jRadioButton3.setText(resourceMap.getString("jRadioButton3.text")); // NOI18N
        jRadioButton3.setName("jRadioButton3"); // NOI18N

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addGap(40, 40, 40)
                        .addComponent(jColorChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addGap(115, 115, 115)
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jRadioButton3)
                            .addComponent(jRadioButton1))))
                .addContainerGap(76, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jRadioButton1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jRadioButton3)
                .addGap(28, 28, 28)
                .addComponent(jColorChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(23, Short.MAX_VALUE))
        );

        buttonGroup1.add(jRadioButton1);
        jRadioButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jColorChooser1.setColor(fontColor);
            }
        });
        buttonGroup1.add(jRadioButton3);

        jRadioButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jColorChooser1.setColor(progressBarColor);
            }
        });

        jTabbedPane1.addTab(resourceMap.getString("jPanel2.TabConstraints.tabTitle"), jPanel2); // NOI18N

        jPanel4.setName("jPanel4"); // NOI18N

        jComboBox1.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Java Default", "System Default", "Motif", "Nimbus", "Substance" }));
        jComboBox1.setName("jComboBox1"); // NOI18N
        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });

        jComboBox2.setModel(skinModel);
        jComboBox2.setName("jComboBox2"); // NOI18N
        jComboBox2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox2ActionPerformed(evt);
            }
        });

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, 201, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addGap(61, 61, 61)
                        .addComponent(jComboBox2, javax.swing.GroupLayout.PREFERRED_SIZE, 199, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jLabel2)))
                .addContainerGap(285, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jComboBox2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addContainerGap(343, Short.MAX_VALUE))
        );

        lafChange = false;
        jComboBox2.setVisible(false);
        lafChange = false;
        if(mbv.projectFont == null)
        jLabel2.setVisible(true);
        else
        jLabel2.setVisible(false);

        jTabbedPane1.addTab(resourceMap.getString("jPanel4.TabConstraints.tabTitle"), jPanel4); // NOI18N

        jButton1.setText(resourceMap.getString("ok.text")); // NOI18N
        jButton1.setName("ok"); // NOI18N

        jButton2.setText(resourceMap.getString("cancel.text")); // NOI18N
        jButton2.setName("cancel"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(438, Short.MAX_VALUE)
                .addComponent(jButton1)
                .addGap(10, 10, 10)
                .addComponent(jButton2)
                .addContainerGap())
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 550, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(14, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 472, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton1)
                    .addComponent(jButton2)))
        );

        jButton1.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                save();
            }
        });
        jButton2.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                dispose();
            }
        });

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        lafChange = true;
        if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Substance") == 0) 
        {
            jComboBox2.setModel(skinModel);
            jComboBox2.setVisible(true);
        } 
        else if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Java Default") == 0) 
        {
            jComboBox2.setModel(new javax.swing.DefaultComboBoxModel(metalThemes));
            jComboBox2.setVisible(true);
        } 
        else 
        {
            jComboBox2.setVisible(false);
        }
    }//GEN-LAST:event_jComboBox1ActionPerformed

    private void jComboBox2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox2ActionPerformed
        lafChange = true;
    }//GEN-LAST:event_jComboBox2ActionPerformed

    @SuppressWarnings("static-access")
    private void save() 
    {
        mbv.projectFont = f;
        mbv.setFont();
        internetConnection = jCheckBox1.isSelected();
        errorMessages = jCheckBox2.isSelected();
        mbv.internetConnection = internetConnection;
        mbv.errorMessage = errorMessages;
        mbv.serverAddress = jTextField1.getText();
        if (jRadioButton1.isSelected()) 
        {
            fontColor = jColorChooser1.getColor();
            mbv.fontColor = fontColor;
            mbv.setFontColor();
        } 
        else if (jRadioButton3.isSelected()) 
        {
            progressBarColor = jColorChooser1.getColor();
            UIManager.put("nimbusOrange", progressBarColor);
            mbv.jProgressBar1.setBackground(progressBarColor);
        }
        if (lafChange) 
        {
            if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Java Default") == 0) 
            {
                SwingUtilities.invokeLater(new Runnable() 
                {
                    @Override
                    public void run() 
                    {
                        try 
                        {
                            if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Default") == 0) 
                            {
                                MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
                            } 
                            else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Ocean") == 0) 
                            {
                                MetalLookAndFeel.setCurrentTheme(new OceanTheme());
                            }
                            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                            laf = UIManager.getLookAndFeel();
                            SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                            SwingUtilities.updateComponentTreeUI(Preferences.this.getRootPane());
                            SwingUtilities.updateComponentTreeUI(mbv.PlayingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.ShowingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.jPopupMenu3);
                            SwingUtilities.updateComponentTreeUI(mbv.PlaylistPopupMenu);
                            if (mbv.songInfo != null && mbv.songInfo.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songInfo.getRootPane());
                            }
                        } 
                        catch (ClassNotFoundException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (InstantiationException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (IllegalAccessException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (UnsupportedLookAndFeelException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        }
                    }
                });

            } 
            else if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("System Default") == 0) 
            {
                SwingUtilities.invokeLater(new Runnable() 
                {
                    @Override
                    public void run() 
                    {
                        try 
                        {
                            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                            laf = UIManager.getLookAndFeel();
                            SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                            SwingUtilities.updateComponentTreeUI(Preferences.this.getRootPane());
                            SwingUtilities.updateComponentTreeUI(mbv.PlayingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.ShowingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.jPopupMenu3);
                            SwingUtilities.updateComponentTreeUI(mbv.PlaylistPopupMenu);
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.createBufferStrategy(2);
                            mainFrame.dispose();
                            mainFrame.setUndecorated(false);
                            mainFrame.pack();
                            mainFrame.setVisible(true);
                            Preferences.this.createBufferStrategy(2);
                            Preferences.this.dispose();
                            Preferences.this.setUndecorated(false);
                            Preferences.this.pack();
                            Preferences.this.setVisible(true);
                            mbv.playlistTable.setRowHeight(16);
                            mbv.currentlyPlayingTable.setRowHeight(16);
                            mbv.currentlyShowingTable.setRowHeight(16);
                            if (mbv.songInfo != null && mbv.songInfo.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songInfo.getRootPane());
                            }
                        } 
                        catch (ClassNotFoundException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (InstantiationException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (IllegalAccessException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (UnsupportedLookAndFeelException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        }
                    }
                });

            } 
            else if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Motif") == 0) 
            {
                SwingUtilities.invokeLater(new Runnable() 
                {
                    @Override
                    public void run() 
                    {
                        try 
                        {
                            UIManager.setLookAndFeel(new MotifLookAndFeel());
                            laf = UIManager.getLookAndFeel();
                            SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                            SwingUtilities.updateComponentTreeUI(Preferences.this.getRootPane());
                            SwingUtilities.updateComponentTreeUI(mbv.PlayingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.ShowingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.jPopupMenu3);
                            SwingUtilities.updateComponentTreeUI(mbv.PlaylistPopupMenu);
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.createBufferStrategy(2);
                            mainFrame.dispose();
                            mainFrame.setUndecorated(false);
                            mainFrame.pack();
                            mainFrame.setVisible(true);
                            Preferences.this.createBufferStrategy(2);
                            Preferences.this.dispose();
                            Preferences.this.setUndecorated(false);
                            Preferences.this.pack();
                            Preferences.this.setVisible(true);
                            mbv.playlistTable.setRowHeight(16);
                            mbv.currentlyPlayingTable.setRowHeight(16);
                            mbv.currentlyShowingTable.setRowHeight(16);
                            if (mbv.songInfo != null && mbv.songInfo.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songInfo.getRootPane());
                            }
                        } 
                        catch (UnsupportedLookAndFeelException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        }
                    }
                });

            } 
            else if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Nimbus") == 0) 
            {
                SwingUtilities.invokeLater(new Runnable() 
                {
                    @Override
                    public void run() 
                    {
                        try 
                        {
                            UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
                            laf = UIManager.getLookAndFeel();
                            SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                            SwingUtilities.updateComponentTreeUI(Preferences.this.getRootPane());
                            SwingUtilities.updateComponentTreeUI(mbv.PlayingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.ShowingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.jPopupMenu3);
                            SwingUtilities.updateComponentTreeUI(mbv.PlaylistPopupMenu);
                            if (mbv.songInfo != null && mbv.songInfo.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songInfo.getRootPane());
                            }
                            //JDialog.setDefaultLookAndFeelDecorated(false);
                            //JFrame.setDefaultLookAndFeelDecorated(false);
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.createBufferStrategy(2);
                            mainFrame.dispose();
                            mainFrame.setUndecorated(false);
                            mainFrame.pack();
                            mainFrame.setVisible(true);
                            Preferences.this.createBufferStrategy(2);
                            Preferences.this.dispose();
                            Preferences.this.setUndecorated(false);
                            Preferences.this.pack();
                            Preferences.this.setVisible(true);
                            mbv.playlistTable.setRowHeight(16);
                            mbv.currentlyPlayingTable.setRowHeight(16);
                            mbv.currentlyShowingTable.setRowHeight(16);
                        } 
                        catch (ClassNotFoundException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (InstantiationException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (IllegalAccessException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        } 
                        catch (UnsupportedLookAndFeelException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        }
                    }
                });

            } 
            else if (((String) jComboBox1.getSelectedItem()).compareToIgnoreCase("Substance") == 0) 
            {
                if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Autumn") == 0) 
                {
                    laf = new SubstanceAutumnLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Business Black Steel") == 0) 
                {
                    laf = new SubstanceBusinessBlackSteelLookAndFeel();
                }
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Business Blue Steel") == 0) 
                {
                    laf = new SubstanceBusinessBlueSteelLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Business") == 0) 
                {
                    laf = new SubstanceBusinessLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Challenger Deep") == 0) 
                {
                    laf = new SubstanceChallengerDeepLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Creme Coffee") == 0) 
                {
                    laf = new SubstanceCremeCoffeeLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Creme") == 0) 
                {
                    laf = new SubstanceCremeLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Dust Coffee") == 0) 
                {
                    laf = new SubstanceDustCoffeeLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Dust") == 0) 
                {
                    laf = new SubstanceDustLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Emerald Dusk") == 0) 
                {
                    laf = new SubstanceEmeraldDuskLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Gemini") == 0) 
                {
                    laf = new SubstanceGeminiLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Graphite") == 0) 
                {
                    laf = new SubstanceGraphiteLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Graphite Aqua") == 0) 
                {
                    laf = new SubstanceGraphiteAquaLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Graphite Glass") == 0) 
                {
                    laf = new SubstanceGraphiteGlassLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Magellan") == 0) 
                {
                    laf = new SubstanceMagellanLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Mist Aqua") == 0) 
                {
                    laf = new SubstanceMistAquaLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Mist Silver") == 0) 
                {
                    laf = new SubstanceMistSilverLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Moderate") == 0) 
                {
                    laf = new SubstanceModerateLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Nebula Brick Wall") == 0) 
                {
                    laf = new SubstanceNebulaBrickWallLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Nebula") == 0) 
                {
                    laf = new SubstanceNebulaLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Office Blue 2007") == 0) 
                {
                    laf = new SubstanceOfficeBlue2007LookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Office Silver 2007") == 0) 
                {
                    laf = new SubstanceOfficeSilver2007LookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Raven") == 0) 
                {
                    laf = new SubstanceRavenLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Sahara") == 0) 
                {
                    laf = new SubstanceSaharaLookAndFeel();
                } 
                else if (((String) jComboBox2.getSelectedItem()).compareToIgnoreCase("Twilight") == 0) 
                {
                    laf = new SubstanceTwilightLookAndFeel();
                }
                java.awt.EventQueue.invokeLater(new Runnable() 
                {
                    @Override
                    public void run() 
                    {
                        try 
                        {
                            UIManager.setLookAndFeel(laf);
                            SwingUtilities.updateComponentTreeUI(MusicBoxApp.getApplication().getMainFrame().getRootPane());
                            SwingUtilities.updateComponentTreeUI(Preferences.this.getRootPane());
                            SwingUtilities.updateComponentTreeUI(mbv.PlayingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.ShowingPopupMenu);
                            SwingUtilities.updateComponentTreeUI(mbv.jPopupMenu3);
                            SwingUtilities.updateComponentTreeUI(mbv.PlaylistPopupMenu);
                            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
                            mainFrame.createBufferStrategy(2);
                            mainFrame.dispose();
                            mainFrame.setUndecorated(true);
                            mainFrame.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
                            mainFrame.pack();
                            mainFrame.setVisible(true);
                            Preferences.this.createBufferStrategy(2);
                            Preferences.this.dispose();
                            Preferences.this.setUndecorated(true);
                            Preferences.this.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
                            Preferences.this.pack();
                            Preferences.this.setVisible(true);
                            mbv.playlistTable.setRowHeight(16);
                            mbv.currentlyPlayingTable.setRowHeight(16);
                            mbv.currentlyShowingTable.setRowHeight(16);
                            if (mbv.songInfo != null && mbv.songInfo.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songInfo);
                            }
                            if (mbv.songSplash != null && mbv.songSplash.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.songSplash);
                            }
                            if (mbv.mini != null && mbv.mini.isVisible()) 
                            {
                                SwingUtilities.updateComponentTreeUI(mbv.mini);
                            }
                        } 
                        catch (UnsupportedLookAndFeelException ex) 
                        {
                            mbv.showErrorDialog(ex);
                        }
                    }
                });



            }
            //laf = UIManager.getLookAndFeel();
            mbv.laf = laf;
            progressBarColor = laf.getDefaults().getColor(JProgressBar.class);
            fontColor = laf.getDefaults().getColor(Font.class);
            if (laf.getDefaults().getFont(Font.class) != null) 
            {
                f = laf.getDefaults().getFont(Font.class);
                mbv.projectFont = f;
            }
            if (f == null) 
            {
                jLabel2.setVisible(true);
            }
            mbv.fontColor = fontColor;
            mbv.progressBarColor = progressBarColor;
            mbv.jProgressBar1.setBackground(progressBarColor);
            UIManager.put("nimbusOrange", progressBarColor);
            mbv.setFontColor();
            mbv.setFont();

            /*java.awt.EventQueue.invokeLater(new Runnable() {

            @Override
            public void run() {
            if (laf.getSupportsWindowDecorations()) {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(true);
            JFrame.setDefaultLookAndFeelDecorated(true);

            mainFrame.dispose();
            mainFrame.setUndecorated(true);
            mainFrame.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            if (mbv.mini == null || !mbv.mini.isVisible()) {
            mainFrame.setVisible(true);
            }
            if (mbv.songInfo != null) {
            mbv.songInfo.dispose();
            mbv.songInfo.setUndecorated(true);
            mbv.songInfo.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            mbv.songInfo.setVisible(true);
            }
            if (mbv.songSplash != null && mbv.songSplash.isVisible()) {
            mbv.songSplash.dispose();
            mbv.songSplash.setUndecorated(true);
            mbv.songSplash.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            mbv.songSplash.setVisible(true);
            }
            if (mbv.mini != null) {
            mbv.mini.dispose();
            mbv.mini.setUndecorated(true);
            mbv.mini.getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
            if (!mainFrame.isVisible()) {
            mbv.mini.setVisible(true);
            }
            }
            } else {
            JFrame mainFrame = MusicBoxApp.getApplication().getMainFrame();
            JDialog.setDefaultLookAndFeelDecorated(false);
            JFrame.setDefaultLookAndFeelDecorated(false);

            mainFrame.dispose();
            mainFrame.setUndecorated(false);
            if (mbv.mini == null || !mbv.mini.isVisible()) {
            mainFrame.setVisible(true);
            }
            if (mbv.songInfo != null) {
            mbv.songInfo.dispose();
            mbv.songInfo.setUndecorated(false);
            mbv.songInfo.setVisible(lafChange);
            }
            if (mbv.songSplash != null && mbv.songSplash.isVisible()) {
            mbv.songSplash.dispose();
            mbv.songSplash.setUndecorated(false);
            mbv.songSplash.setVisible(true);
            }
            if (mbv.mini != null) {
            mbv.mini.dispose();
            mbv.mini.setUndecorated(false);
            if (!mainFrame.isVisible()) {
            mbv.mini.setVisible(true);
            }
            }
            Preferences.this.dispose();
            Preferences.this.setUndecorated(false);
            Preferences.this.setVisible(true);
            }
            mbv.jTable1.setRowHeight(16);
            mbv.jTable2.setRowHeight(16);
            mbv.jTable3.setRowHeight(16);
            }
            });*/

            lafChange = false;
        }
        try 
        {
            ls.save(f, "preferences.xml");
            ls.save(laf, "lookandfeel.xml");
            ls.save(progressBarColor, "progressbarcolor.xml");
            ls.save(fontColor, "fontcolor.xml");
            ls.save(internetConnection, "internetconnection.xml");
            ls.save(errorMessages, "errormessages.xml");
            ls.save(jTextField1.getText(), "serveraddress.xml");
        } 
        catch (IOException ex) 
        {
            //ex.printStackTrace();
            mbv.showErrorDialog(ex);
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JCheckBox jCheckBox2;
    private javax.swing.JColorChooser jColorChooser1;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JComboBox jComboBox2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JList jList1;
    private javax.swing.JList jList2;
    private javax.swing.JList jList3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextField jTextField1;
    // End of variables declaration//GEN-END:variables
    /*private String[] substanceSkins = {"Autumn", "Business Black Steel", "Business Blue Steel", "Business", "Challenger Deep",
    "Creme Coffee", "Creme", "Dust Coffee", "Dust", "Emerald Dusk", "Gemini", "Graphite",
    "Graphite Aqua", "Graphite Glass", "Magellan", "Mist Aqua", "Mist Silver",
    "Moderate", "Nebula Brick Wall", "Nebula", "Office Blue 2007", "Office Silver 2007",
    "Raven", "Sahara", "Twilight"};*/
    private String[] metalThemes = {"Default", "Ocean"};
    private LookAndFeel laf;
    /**
     *
     */
    public boolean fontChange;
    private DefaultComboBoxModel skinModel;

    private void selectCurrentLAF() 
    {
        if (laf.isNativeLookAndFeel()) 
        {
            jComboBox1.setSelectedItem("System Default");
        } 
        else if (laf.getName().equals("Metal")) {
            jComboBox1.setSelectedItem("Java Default");
        } else if (laf.getName().equals("CDE/Motif")) 
        {
            jComboBox1.setSelectedItem("Motif");
        }
        else if (laf.getName().equals("Nimbus")) 
        {
            jComboBox1.setSelectedItem("Nimbus");
        }
        else if (laf.getName().contains("Substance")) 
        {
            jComboBox1.setSelectedItem("Substance");
            for (int i = 0; i < jComboBox2.getModel().getSize(); i++) 
            {
                if (laf.getName().substring(laf.getName().indexOf(" ") + 1, laf.getName().length()).compareTo(jComboBox2.getModel().getElementAt(i).toString()) == 0) 
                {
                    jComboBox2.setSelectedItem(jComboBox2.getModel().getElementAt(i).toString());
                    break;
                }
            }
        }
    }
}
