/*
Music Box
Copyright (C) 2009-2012  Isaac Hammon

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR v PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * SmallAlbumArt.java
 *
 * Created on Feb 27, 2010, 3:45:42 PM
 */
package musicbox.gui;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collections;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import musicbox.backend.comparators.AlbumComparator;
import musicbox.backend.comparators.ArtistComparator;
import noTalent.MusicOutputDesign;

/**
 *
 * @author Isaac Hammon
 */
public class SmallAlbumArt extends javax.swing.JDialog 
{

    private ImageIcon i;
    private String artist;
    /**
     *
     */
    public boolean googleSearched;
    private int googleImageLocation = 0;
    private Vector<String> googleImages;
    private JLabel j;
    private String custom;
    private MusicBoxView mbv;
    private JScrollPane scroll;

    /** Creates new form SmallAlbumArt
     * @param parent
     * @param j
     * @param i
     * @param s 
     */
    public SmallAlbumArt(java.awt.Frame parent, ImageIcon i, String s, JLabel j, MusicBoxView mbv) 
    {
        super(parent);
        this.i = i;
        this.artist = s;
        this.j = j;
        this.mbv = mbv;
        initComponents();
        setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
        setIconImage(mbv.musicBoxIcon16.getImage());
        getContentPane().remove(bottomPanel);
        getContentPane().remove(topPanel);
        pack();
    }

    private void appendToPlayList() 
    {
        Vector<MusicOutputDesign> temp = (Vector<MusicOutputDesign>) mbv.allMusic.clone();
        Vector<MusicOutputDesign> append = new Vector<MusicOutputDesign>();
        int sel = mbv.jTabbedPane2.getSelectedIndex();
        if (sel == 0) {
            Collections.sort(temp, new ArtistComparator());
        } else {
            Collections.sort(temp, new AlbumComparator());
        }
        int loc = 0;
        while (loc != -1) 
        {
            if (sel == 0) 
            {
                loc = binarySearchArtist(temp, 0, temp.size());
            } 
            else 
            {
                loc = binarySearchAlbum(temp, 0, temp.size());
            }
            if (loc != -1) 
            {
                append.add(temp.remove(loc));
            }
        }
        Collections.sort(append, new AlbumComparator());
        mbv.appendToPlayList(append);
    }

    private int binarySearchArtist(Vector<MusicOutputDesign> v, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        String s = v.elementAt(mid).getArtist().replace("\\", "/");
        if (s.compareToIgnoreCase(artist) > 0) 
        {
            return binarySearchArtist(v, low, mid - 1);
        } 
        else if (s.compareToIgnoreCase(artist) < 0) 
        {
            return binarySearchArtist(v, mid + 1, high);
        } 
        else 
        {
            return mid; // found
        }
    }

    private int binarySearchAlbum(Vector<MusicOutputDesign> v, int low, int high) 
    {
        if (high < low) 
        {
            return -1; // not found
        }
        int mid = (high + low) / 2;
        String s = v.elementAt(mid).getAlbum().replace("\\", "/");
        if (s.compareToIgnoreCase(artist) > 0) 
        {
            return binarySearchAlbum(v, low, mid - 1);
        }
        else if (s.compareToIgnoreCase(artist) < 0) 
        {
            return binarySearchAlbum(v, mid + 1, high);
        } 
        else 
        {
            return mid; // found
        }
    }

    private void googleImageSearch() 
    {
        bottomShowing = true;
        googleSearched = true;
        googleImageLocation = 0;
        googleImages = new Vector<String>();
        custom = "";
        int r = JOptionPane.showConfirmDialog(this, "Customize google search?", "Google Search", JOptionPane.YES_NO_OPTION);
        if (r == JOptionPane.YES_OPTION) 
        {
            custom = JOptionPane.showInputDialog("Custom Search", "");
        } 
        else 
        {
            custom = artist;
        }
        try 
        {
            String u = "http://images.google.com/images?q=" + custom;
            if (u.contains(" ")) 
            {
                u = u.replace(" ", "+");
            }
            URL url = new URL(u);

            HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
            httpcon.addRequestProperty("User-Agent", "Mozilla/4.76");

            BufferedReader readIn = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));
            googleImages.clear();
            String lin = new String();
            while ((lin = readIn.readLine()) != null) {
                while (lin.contains("href=\"/imgres?imgurl=")) 
                {
                    while (!lin.contains(">")) 
                    {
                        lin += readIn.readLine();
                    }
                    String s = lin.substring(lin.indexOf("href=\"/imgres?imgurl="), lin.indexOf(">", lin.indexOf("href=\"/imgres?imgurl=")));
                    lin = lin.substring(lin.indexOf(">", lin.indexOf("href=\"/imgres?imgurl=")));
                    if (s.contains("&amp;") && s.indexOf("http://") < s.indexOf("&amp;")) 
                    {
                        s = s.substring(s.indexOf("http://"), s.indexOf("&amp;"));
                    } 
                    else 
                    {
                        s = s.substring(s.indexOf("http://"), s.length());
                    }
                    googleImages.add(s);
                }
            }
            readIn.close();
        } 
        catch (Exception ex4) 
        {
            mbv.showErrorDialog(ex4);
            //ex4.printStackTrace();
        }

        bottomPreviousButton.setEnabled(false);
        getContentPane().remove(jLabel1);
        if(scroll != null && scroll.isVisible())
        {
            getContentPane().remove(scroll);
        }
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
            int h = icon.getIconHeight();
            int w = icon.getIconWidth();
            if (h < 1 || w < 1) 
            {
                icon = mbv.noImage;
                h = icon.getIconHeight();
                w = icon.getIconWidth();
            }
            jLabel1.setSize(w, h);
            jLabel1.setIcon(icon);
            if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
            {
                scroll = new JScrollPane();
                scroll.setViewportView(jLabel1);
                scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                add(scroll, BorderLayout.CENTER);
                Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                setPreferredSize(dim);
            }
            else
            {
                add(jLabel1, BorderLayout.CENTER);
                setPreferredSize(getMinimumSize());
            }
        } 
        catch (MalformedURLException ex) 
        {
            mbv.showErrorDialog(ex);
            jLabel1.setIcon(mbv.noImage);
            setPreferredSize(getMinimumSize());
        }
        add(bottomPanel, BorderLayout.PAGE_END);
        pack();
    }

    /**
     *
     * @param start
     */
    public void googleImageSearch(String start) 
    {
        try 
        {
            String u = "http://images.google.com/images?q=" + custom + start;
            if (u.contains(" ")) 
            {
                u = u.replace(" ", "+");
            }
            URL url = new URL(u);

            HttpURLConnection httpcon = (HttpURLConnection) url.openConnection();
            httpcon.addRequestProperty("User-Agent", "Mozilla/4.76");
            BufferedReader readIn = new BufferedReader(new InputStreamReader(httpcon.getInputStream()));
            //googleImages.clear();
            
            String text = "";
            String lin = "";
            while((lin = readIn.readLine()) != null)
            {
                text += lin;
            }
            readIn.close();
            if(text.contains("\n"))
            {
                text = text.replace("\n", "");
            }
            String[] array = text.split("\\Qhref=\"/imgres?imgurl=\\E");
            for(String s: array)
            {
                if(s.startsWith("http://") || s.startsWith("https://") && s.contains("&amp;"))
                {
                    String s1 = s.substring(0, s.indexOf("&amp;"));
                    googleImages.add(s1);
                }
            }
        } 
        catch (Exception ex4) 
        {
            mbv.showErrorDialog(ex4);
            //ex4.printStackTrace();
        }
        topNextButton.setEnabled(true);
        topPreviousButton.setEnabled(false);
        bottomNextButton.setEnabled(true);
        bottomPreviousButton.setEnabled(false);
        getContentPane().remove(jLabel1);
        if(scroll != null && scroll.isVisible())
        {
            getContentPane().remove(scroll);
        }
        ImageIcon icon;
        try 
        {
            icon = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
            int h = icon.getIconHeight();
            int w = icon.getIconWidth();
            if (h < 1 || w < 1) 
            {
                icon = mbv.noImage;
                h = icon.getIconHeight();
                w = icon.getIconWidth();
            }
            jLabel1.setSize(w, h);
            jLabel1.setIcon(icon);
            if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
            {
                scroll = new JScrollPane();
                scroll.setViewportView(jLabel1);
                scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                add(scroll, BorderLayout.CENTER);
                Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                setPreferredSize(dim);

            }
            else
            {
                add(jLabel1, BorderLayout.CENTER);
                setPreferredSize(getMinimumSize());
            }
        } 
        catch (MalformedURLException ex) 
        {
            mbv.showErrorDialog(ex);
            jLabel1.setIcon(mbv.noImage);
            setPreferredSize(getMinimumSize());
        }
        add(bottomPanel, BorderLayout.PAGE_END);
        pack();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPopupMenu1 = new javax.swing.JPopupMenu();
        jLabel1 = new javax.swing.JLabel();
        bottomPanel = new javax.swing.JPanel();
        bottomPreviousButton = new javax.swing.JButton();
        bottomNextButton = new javax.swing.JButton();
        topPanel = new javax.swing.JPanel();
        topPreviousButton = new javax.swing.JButton();
        topNextButton = new javax.swing.JButton();

        jPopupMenu1.setName("jPopupMenu1"); // NOI18N

        JMenuItem append = new JMenuItem("Append to playlist");
        JMenuItem google = new JMenuItem("Google Image Search");
        jPopupMenu1.add(append);
        jPopupMenu1.add(google);
        append.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent evt)
                {
                    appendToPlayList();
                }
            });
            google.addActionListener(new java.awt.event.ActionListener()
                {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        if(mbv.internetConnection)
                        {
                            googleImageSearch();
                        }
                    }
                });

                setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
                setName("Form"); // NOI18N
                setResizable(false);
                setUndecorated(true);

                org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(musicbox.backend.MusicBoxApp.class).getContext().getResourceMap(SmallAlbumArt.class);
                jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
                jLabel1.setName("jLabel1"); // NOI18N
                jLabel1.setIcon(i);
                jLabel1.setToolTipText("<HTML>" + artist + "<BR />(click to close)</HTML>");
                jLabel1.setSize(130, 125);
                jLabel1.addMouseListener(new java.awt.event.MouseAdapter() {
                    public void mouseClicked(java.awt.event.MouseEvent evt) {
                        jLabel1MouseClicked(evt);
                    }
                });
                jLabel1.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
                    public void mouseDragged(java.awt.event.MouseEvent evt) {
                        jLabel1MouseDragged(evt);
                    }
                    public void mouseMoved(java.awt.event.MouseEvent evt) {
                        jLabel1MouseMoved(evt);
                    }
                });
                jLabel1.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                getContentPane().add(jLabel1, java.awt.BorderLayout.CENTER);

                bottomPanel.setName("bottomPanel"); // NOI18N
                bottomPanel.setPreferredSize(new java.awt.Dimension(100, 30));
                bottomPanel.setLayout(new java.awt.BorderLayout());

                bottomPreviousButton.setText(resourceMap.getString("bottomPreviousButton.text")); // NOI18N
                bottomPreviousButton.setName("bottomPreviousButton"); // NOI18N
                bottomPreviousButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        bottomPreviousButtonActionPerformed(evt);
                    }
                });
                bottomPanel.add(bottomPreviousButton, java.awt.BorderLayout.LINE_START);
                bottomPreviousButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

                bottomNextButton.setText(resourceMap.getString("bottomNextButton.text")); // NOI18N
                bottomNextButton.setName("bottomNextButton"); // NOI18N
                bottomNextButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        bottomNextButtonActionPerformed(evt);
                    }
                });
                bottomPanel.add(bottomNextButton, java.awt.BorderLayout.LINE_END);
                bottomNextButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

                getContentPane().add(bottomPanel, java.awt.BorderLayout.PAGE_END);

                topPanel.setName("topPanel"); // NOI18N
                topPanel.setPreferredSize(new java.awt.Dimension(100, 30));
                topPanel.setLayout(new java.awt.BorderLayout());

                topPreviousButton.setText(resourceMap.getString("topPreviousButton.text")); // NOI18N
                topPreviousButton.setName("topPreviousButton"); // NOI18N
                topPreviousButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        topPreviousButtonActionPerformed(evt);
                    }
                });
                topPreviousButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                topPanel.add(topPreviousButton, java.awt.BorderLayout.LINE_START);

                topNextButton.setText(resourceMap.getString("topNextButton.text")); // NOI18N
                topNextButton.setName("topNextButton"); // NOI18N
                topNextButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        topNextButtonActionPerformed(evt);
                    }
                });
                topPanel.add(topNextButton, java.awt.BorderLayout.LINE_END);
                topNextButton.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

                getContentPane().add(topPanel, java.awt.BorderLayout.PAGE_START);

                pack();
            }// </editor-fold>//GEN-END:initComponents

    private void jLabel1MouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jLabel1MouseDragged
        if (googleSearched) 
        {
            setLocation(evt.getLocationOnScreen());
        }
    }//GEN-LAST:event_jLabel1MouseDragged

    private void jLabel1MouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jLabel1MouseMoved
        if (bottomShowing == true && ((double) evt.getY() / (double) jLabel1.getHeight()) * 100 < 50) 
        {
            bottomShowing = false;
            topShowing = true;
            getContentPane().remove(bottomPanel);
            getContentPane().add(topPanel, BorderLayout.PAGE_START);
            pack();
        } 
        else if (topShowing == true && ((double) evt.getY() / (double) jLabel1.getHeight()) * 100 >= 50) 
        {
            bottomShowing = true;
            topShowing = false;
            getContentPane().remove(topPanel);
            getContentPane().add(bottomPanel, BorderLayout.PAGE_END);
            pack();
        }
    }//GEN-LAST:event_jLabel1MouseMoved
    private void updateJLabelImage()
    {
        ImageIcon icon1;
        getContentPane().remove(jLabel1);
        if(scroll != null && scroll.isVisible())
        {
            getContentPane().remove(scroll);
        }
        try 
        {
            icon1 = new ImageIcon(new URL(googleImages.elementAt(googleImageLocation)));
            int h = icon1.getIconHeight();
            int w = icon1.getIconWidth();
            if (h < 1 || w < 1) 
            {
                icon1 = mbv.noImage;
                h = icon1.getIconHeight();
                w = icon1.getIconWidth();
            }
            jLabel1.setSize(w, h);
            jLabel1.setIcon(icon1);
            if(w >= getMaximumSize().getWidth() || h >= getMaximumSize().getHeight())
            {
                scroll = new JScrollPane();
                scroll.setViewportView(jLabel1);
                scroll.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
                add(scroll, BorderLayout.CENTER);
                Dimension dim = new Dimension(Toolkit.getDefaultToolkit().getScreenSize());
                dim.setSize(Math.min(dim.getWidth()-100,w), Math.min(dim.getHeight()-100,h));
                setPreferredSize(dim);

            }
            else
            {
                add(jLabel1, BorderLayout.CENTER);
                setPreferredSize(getMinimumSize());
            }
            pack();
        } 
        catch (MalformedURLException ex1) 
        {
            //ex1.printStackTrace();
            mbv.showErrorDialog(ex1);
            icon1 = mbv.noImage;
            int h = icon1.getIconHeight();
            int w = icon1.getIconWidth();
            jLabel1.setSize(w, h);
            jLabel1.setIcon(icon1);
            add(jLabel1, BorderLayout.CENTER);
            pack();
            setPreferredSize(getMinimumSize());
        }
        jLabel1.setToolTipText("<HTML>" + artist + "<BR />" + googleImages.elementAt(googleImageLocation) + "<BR />(click to close)</HTML>");
    }
    private void topPreviousButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_topPreviousButtonActionPerformed

        googleImageLocation -= 1;
        updateJLabelImage();
        if (googleImageLocation == 0) 
        {
            bottomPreviousButton.setEnabled(false);
            topPreviousButton.setEnabled(false);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        else
        {
            bottomPreviousButton.setEnabled(true);
            topPreviousButton.setEnabled(true);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
    }//GEN-LAST:event_topPreviousButtonActionPerformed

    private void jLabel1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jLabel1MouseClicked
        if (evt.getButton() == 3) 
        {
            jPopupMenu1.show(evt.getComponent(), evt.getX(), evt.getY());
        }
        if (evt.getButton() == 1) 
        {
            if (googleSearched) 
            {
                int r = JOptionPane.showConfirmDialog(jLabel1, "Use this image?", "Google Image", JOptionPane.YES_NO_OPTION);
                if (r == JOptionPane.YES_OPTION) 
                {
                    ImageIcon ii = (ImageIcon) jLabel1.getIcon();
                    j.setIcon(new ImageIcon(ii.getImage().getScaledInstance(100, 100, Image.SCALE_SMOOTH)));
                    mbv.back.setImagePath(googleImages.elementAt(googleImageLocation), artist);
                }
            }
            googleSearched = false;
            dispose();
        }
    }//GEN-LAST:event_jLabel1MouseClicked

    private void topNextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_topNextButtonActionPerformed
        googleImageLocation += 1;
        updateJLabelImage();
        if (googleImageLocation == 0) 
        {
            bottomPreviousButton.setEnabled(false);
            topPreviousButton.setEnabled(false);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        else
        {
            bottomPreviousButton.setEnabled(true);
            topPreviousButton.setEnabled(true);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        //if (googleImageLocation < googleImages.size() - 1) 
        //{
       //     bottomNextButton.setEnabled(true);
        //    topNextButton.setEnabled(true);
        //} 
        if(googleImageLocation == googleImages.size()-1) 
        {
            imageResults += 21;
            //googleImageLocation = 0;
            String start = "&sa=N&start=" + imageResults + "&ndsp=21";
            googleImageSearch(start);
        }
    }//GEN-LAST:event_topNextButtonActionPerformed

    private void bottomPreviousButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bottomPreviousButtonActionPerformed
        googleImageLocation -= 1;
        updateJLabelImage();
        if (googleImageLocation == 0) 
        {
            bottomPreviousButton.setEnabled(false);
            topPreviousButton.setEnabled(false);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        else
        {
            bottomPreviousButton.setEnabled(true);
            topPreviousButton.setEnabled(true);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
    }//GEN-LAST:event_bottomPreviousButtonActionPerformed

    private void bottomNextButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bottomNextButtonActionPerformed
        googleImageLocation += 1;
        updateJLabelImage();
        if (googleImageLocation == 0) 
        {
            bottomPreviousButton.setEnabled(false);
            topPreviousButton.setEnabled(false);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        else
        {
            bottomPreviousButton.setEnabled(true);
            topPreviousButton.setEnabled(true);
            topNextButton.setEnabled(true);
            bottomNextButton.setEnabled(true);
        }
        //if (googleImageLocation < googleImages.size() - 1) 
        //{
        //    bottomNextButton.setEnabled(true);
        //    topNextButton.setEnabled(true);
        //} 
        if(googleImageLocation == googleImages.size() - 1)
        {
            imageResults += 21;
            //googleImageLocation = 0;
            String start = "&sa=N&start=" + imageResults + "&ndsp=21";
            googleImageSearch(start);
        }
    }//GEN-LAST:event_bottomNextButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bottomNextButton;
    private javax.swing.JPanel bottomPanel;
    private javax.swing.JButton bottomPreviousButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPopupMenu jPopupMenu1;
    private javax.swing.JButton topNextButton;
    private javax.swing.JPanel topPanel;
    private javax.swing.JButton topPreviousButton;
    // End of variables declaration//GEN-END:variables
    private boolean topShowing = false;
    private boolean bottomShowing = false;
    /**
     *
     */
    public int imageResults = 0;
}
